@@ -23,6 +23,7 @@
 
 /* -------------------------------------------------------------------- */
 
+/* Force inclusion of all modes ! */
 #define HAVE_CXX
 #define HAVE_XPM
 #define USE_GL
@@ -30,6 +31,8 @@
 #define USE_BOMB
 #define HAVE_TTF
 #define HAVE_GLTT
+#define HAVE_FREETYPE
+#define HAVE_FTGL
 
 typedef struct {
 	int	 dummy;
@@ -70,6 +73,9 @@ LockStruct  LockProcs[] =
 	{"ant",
 	 1000, -3, 40000, -7, 64, 1.0, "",
 	 "Shows Langton's and Turk's generalized ants", 0, NULL, NULL},
+	{"ant3d",
+	 5000, -3, 100000, 1, 64, 1.0, "",
+	 "Shows 3D ants", 0, NULL, NULL},
 	{"apollonian",
 	 1000000, 64, 20, 1, 64, 1.0, "",
 	"Shows Apollonian Circles", 0, NULL, NULL},
@@ -78,8 +84,8 @@ LockStruct  LockProcs[] =
 	 25000, 4, 100, 6000, 64, 1.0, "",
 	 "Shows moving sharks/whales/dolphin", 0, NULL, "#ifdef USE_GL"},
 	{"atunnels",
-	 25000, 1, 1, 400, 64, 1.0, "",
-	 "Shows a n OpenGL advanced tunnel screensaver", 0, NULL, "#ifdef USE_GL"},
+	 25000, 1, 1, 0, 64, 1.0, "",
+	 "Shows an OpenGL advanced tunnel screensaver", 0, NULL, "#ifdef USE_GL"},
 #endif
 	{"ball",
 	 10000, 10, 20, -100, 64, 1.0, "",
@@ -162,7 +168,7 @@ LockStruct  LockProcs[] =
 	 "Shows a fading plot of sine squared", 0, NULL, NULL},
 #ifdef USE_GL
 	{"fire",
-	 10000, 800, 1, 400, 64, 1.0, "",
+	 10000, 800, 1, 0, 64, 1.0, "",
 	 "Shows a 3D fire-like image", 0, NULL, "#ifdef USE_GL"},
 #endif
 	{"flag",
@@ -340,7 +346,7 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef USE_GL
 	{"sballs",
-	 40000, 0, 10, 400, 64, 1.0, "",
+	 40000, 0, 10, 0, 64, 1.0, "",
 	 "Balls spinning like crazy in GL", 0, NULL, "#ifdef USE_GL"},
 #endif
         {"scooter",
@@ -384,7 +390,7 @@ LockStruct  LockProcs[] =
 	 "Shows colorful moving splines", 0, NULL, NULL},
 #ifdef USE_GL
 	{"sproingies",
-	 80000, 5, 0, 400, 64, 1.0, "",
+	 80000, 5, 0, 0, 64, 1.0, "",
 	 "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL, "#ifdef USE_GL"},
 	{"stairs",
 	 200000, 0, 1, 1, 64, 1.0, "",
@@ -418,8 +424,13 @@ LockStruct  LockProcs[] =
 	 "Shows an autoplaying tetris game", 0, NULL, NULL},
 #if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_TTF ) && defined( HAVE_GLTT )
 	{"text3d",
-	 100000, 1, 1, 1, 64, 1.0, "",
+	 100000, 1, 10, 1, 64, 1.0, "",
 	 "Shows 3D text", 0, NULL, "#if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_TTF ) && defined( HAVE_GLTT )"},
+#endif
+#if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_FREETYPE ) && defined( HAVE_FTGL )
+	{"text3d2",
+	 100000, 1, 10, 1, 64, 1.0, "",
+	 "Shows 3D text", 0, NULL, "#if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_FREETYPE ) && defined( HAVE_FTGL )"},
 #endif
 	{"thornbird",
 	 1000, 800, 16, 1, 64, 1.0, "",
