@@ -5,7 +5,7 @@
  */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)ant.c	5.00 2000/11/01 xlockmore";
+static const char sccsid[] = "@(#)ant.c	5.13 2004/06/23 xlockmore";
 
 #endif
 
@@ -54,6 +54,7 @@ static const char sccsid[] = "@(#)ant.c	5.00 2000/11/01 xlockmore";
 */
 
 #ifdef STANDALONE
+#define MODE_ant
 #define PROGCLASS "Ant"
 #define HACK_INIT init_ant
 #define HACK_DRAW draw_ant
@@ -80,11 +81,13 @@ static const char sccsid[] = "@(#)ant.c	5.00 2000/11/01 xlockmore";
  */
 
 #define DEF_NEIGHBORS  "0"      /* choose random value */
+#define DEF_RULE "A" /* All rules */
 #define DEF_TRUCHET  "False"
 #define DEF_EYES  "False"
 #define DEF_SHARPTURN  "False"
 
 static int  neighbors;
+static char *rule;
 static Bool truchet;
 static Bool eyes;
 static Bool sharpturn;
@@ -92,6 +95,7 @@ static Bool sharpturn;
 static XrmOptionDescRec opts[] =
 {
 	{(char *) "-neighbors", (char *) ".ant.neighbors", XrmoptionSepArg, (caddr_t) NULL},
+	{(char *) "-rule", (char *) ".ant.rule", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-truchet", (char *) ".ant.truchet", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+truchet", (char *) ".ant.truchet", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-eyes", (char *) ".ant.eyes", XrmoptionNoArg, (caddr_t) "on"},
@@ -102,6 +106,7 @@ static XrmOptionDescRec opts[] =
 static argtype vars[] =
 {
 	{(caddr_t *) & neighbors, (char *) "neighbors", (char *) "Neighbors", (char *) DEF_NEIGHBORS, t_Int},
+	{(caddr_t *) & rule, (char *) "rule", (char *) "Rule", (char *) DEF_RULE, t_String},
 	{(caddr_t *) & truchet, (char *) "truchet", (char *) "Truchet", (char *) DEF_TRUCHET, t_Bool},
 	{(caddr_t *) & eyes, (char *) "eyes", (char *) "Eyes", (char *) DEF_EYES, t_Bool},
    {(caddr_t *) & sharpturn, (char *) "sharpturn", (char *) "SharpTurn", (char *) DEF_SHARPTURN, t_Bool}
@@ -109,6 +114,7 @@ static argtype vars[] =
 static OptionStruct desc[] =
 {
 	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3 or 12"},
+	{(char *) "-rule string", (char *) "binary string for Turk's Ant"},
 	{(char *) "-/+truchet", (char *) "turn on/off Truchet lines"},
 	{(char *) "-/+eyes", (char *) "turn on/off eyes"},
 	{(char *) "-/+sharpturn", (char *) "turn on/off sharp turns (6, 8 or 12 neighbors only)"}
@@ -165,9 +171,9 @@ typedef struct {
 	unsigned char ncolors, nstates;
 	int         n;
 	int         redrawing, redrawpos;
-	int         truchet;	/* Only for Turk modes */
-	int         eyes;
-	int         sharpturn;
+	Bool        truchet;	/* Only for Turk modes */
+	Bool        eyes;
+	Bool        sharpturn;
 	statestruct machine[NUMSTIPPLES * STATES];
 	unsigned char *tape;
 	unsigned char *truchet_state;
@@ -227,7 +233,7 @@ static unsigned char tables[][3 * NUMSTIPPLES * STATES + 2] =
 {
 #if 0
   /* Here just so you can figure out notation */
-	{			/* Langton's ant */
+	{			/* Langton's ant  (-rule 10) */
 		2, 1,
 		1, TLS, 0, 0, TRS, 0
 	},
@@ -257,38 +263,7 @@ position_of_neighbor(antfarmstruct * ap, int dir, int *pcol, int *prow)
 {
 	int         col = *pcol, row = *prow;
 
-	if (ap->neighbors == 6) {
-		switch (dir) {
-			case 0:
-				col = (col + 1 == ap->ncols) ? 0 : col + 1;
-				break;
-			case 60:
-				if (!(row & 1))
-					col = (col + 1 == ap->ncols) ? 0 : col + 1;
-				row = (!row) ? ap->nrows - 1 : row - 1;
-				break;
-			case 120:
-				if (row & 1)
-					col = (!col) ? ap->ncols - 1 : col - 1;
-				row = (!row) ? ap->nrows - 1 : row - 1;
-				break;
-			case 180:
-				col = (!col) ? ap->ncols - 1 : col - 1;
-				break;
-			case 240:
-				if (row & 1)
-					col = (!col) ? ap->ncols - 1 : col - 1;
-				row = (row + 1 == ap->nrows) ? 0 : row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = (col + 1 == ap->ncols) ? 0 : col + 1;
-				row = (row + 1 == ap->nrows) ? 0 : row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (ap->neighbors == 4 || ap->neighbors == 8) {
+	if (ap->neighbors == 4 || ap->neighbors == 6 || ap->neighbors == 8) {
 		switch (dir) {
 			case 0:
 				col = (col + 1 == ap->ncols) ? 0 : col + 1;
@@ -297,9 +272,19 @@ position_of_neighbor(antfarmstruct * ap, int dir, int *pcol, int *prow)
 				col = (col + 1 == ap->ncols) ? 0 : col + 1;
 				row = (!row) ? ap->nrows - 1 : row - 1;
 				break;
+			case 60:
+				if (!(row & 1))
+					col = (col + 1 == ap->ncols) ? 0 : col + 1;
+				row = (!row) ? ap->nrows - 1 : row - 1;
+				break;
 			case 90:
 				row = (!row) ? ap->nrows - 1 : row - 1;
 				break;
+			case 120:
+				if (row & 1)
+					col = (!col) ? ap->ncols - 1 : col - 1;
+				row = (!row) ? ap->nrows - 1 : row - 1;
+				break;
 			case 135:
 				col = (!col) ? ap->ncols - 1 : col - 1;
 				row = (!row) ? ap->nrows - 1 : row - 1;
@@ -311,9 +296,19 @@ position_of_neighbor(antfarmstruct * ap, int dir, int *pcol, int *prow)
 				col = (!col) ? ap->ncols - 1 : col - 1;
 				row = (row + 1 == ap->nrows) ? 0 : row + 1;
 				break;
+			case 240:
+				if (row & 1)
+					col = (!col) ? ap->ncols - 1 : col - 1;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
 			case 270:
 				row = (row + 1 == ap->nrows) ? 0 : row + 1;
 				break;
+			case 300:
+				if (!(row & 1))
+					col = (col + 1 == ap->ncols) ? 0 : col + 1;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
 			case 315:
 				col = (col + 1 == ap->ncols) ? 0 : col + 1;
 				row = (row + 1 == ap->nrows) ? 0 : row + 1;
@@ -551,17 +546,22 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 				hex2.x = hex.x + ap->shape.hexagon[side + 1].x / 2;
 				hex2.y = hex.y + ap->shape.hexagon[side + 1].y / 2 + 1;
 				/* Lots of fudging here */
-				if (side == 1) {
+				if (side == 0) {
+					hex2.x += 1;
+				} else if (side == 1) {
 					hex2.x += (short) (ap->xs * 0.1 + 1);
 					hex2.y += (short) (ap->ys * 0.1 - ((ap->ys > 5) ? 1 : 0));
 				} else if (side == 2) {
-					hex2.x += (short) (ap->xs * 0.1);
+					hex2.x += (short) (ap->xs * 0.1 + 1);
+					hex2.y += 1;
+				} else if (side == 3) {
+					hex2.y += 1;
 				} else if (side == 4) {
 					hex2.x += (short) (ap->xs * 0.1);
 					hex2.y += (short) (ap->ys * 0.1 - 1);
 				} else if (side == 5) {
 					hex2.x += (short) (ap->xs * 0.5);
-					hex2.y += (short) (-ap->ys * 0.3 + 1);
+					hex2.y += (short) (-ap->ys * 0.3);
 				}
 				if (truchetstate == side % 3)
 					/* Crude approx of 120 deg, so it will not cause drawing errors. */
@@ -574,24 +574,24 @@ truchetcell(ModeInfo * mi, int col, int row, int truchetstate)
 	} else if (ap->neighbors == 4) {
 		if (truchetstate) {
 			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				 ap->xb + ap->xs * col - ap->xs / 2 + 1,
-				 ap->yb + ap->ys * row + ap->ys / 2 - 1,
+				 ap->xb + ap->xs * col - (ap->xs - 1) / 2,
+				 ap->yb + ap->ys * row + (ap->ys - 1) / 2,
 				 ap->xs - 2, ap->ys - 2,
 				 0 * 64, 90 * 64);
 			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				 ap->xb + ap->xs * col + ap->xs / 2 - 1,
-				 ap->yb + ap->ys * row - ap->ys / 2 + 1,
+				 ap->xb + ap->xs * col + (ap->xs - 1) / 2,
+				 ap->yb + ap->ys * row - (ap->ys - 1) / 2,
 				 ap->xs - 2, ap->ys - 2,
 				 -90 * 64, -90 * 64);
 		} else {
 			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				 ap->xb + ap->xs * col - ap->xs / 2 + 1,
-				 ap->yb + ap->ys * row - ap->ys / 2 + 1,
+				 ap->xb + ap->xs * col - (ap->xs - 1) / 2,
+				 ap->yb + ap->ys * row - (ap->ys - 1) / 2,
 				 ap->xs - 2, ap->ys - 2,
 				 0 * 64, -90 * 64);
 			XDrawArc(MI_DISPLAY(mi), MI_WINDOW(mi), MI_GC(mi),
-				 ap->xb + ap->xs * col + ap->xs / 2 - 1,
-				 ap->yb + ap->ys * row + ap->ys / 2 - 1,
+				 ap->xb + ap->xs * col + (ap->xs - 1) / 2,
+				 ap->yb + ap->ys * row + (ap->ys - 1) / 2,
 				 ap->xs - 2, ap->ys - 2,
 				 90 * 64, 90 * 64);
 		}
@@ -641,7 +641,7 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char color)
 		gc = MI_GC(mi);
 	} else if (MI_NPIXELS(mi) > 2) {
 		XSetForeground(MI_DISPLAY(mi), MI_GC(mi),
-			       MI_PIXEL(mi, ap->colors[color - 1]));
+			MI_PIXEL(mi, ap->colors[color - 1]));
 		gc = MI_GC(mi);
 	} else {
 		XGCValues   gcv;
@@ -880,6 +880,7 @@ getTable(ModeInfo * mi, int i)
 	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
 	int         j, total;
 	unsigned char *patptr;
+	statestruct *status;
 
 	patptr = &tables[i][0];
 	ap->ncolors = *patptr++;
@@ -890,7 +891,8 @@ getTable(ModeInfo * mi, int i)
 			       "ants %d, neighbors %d, table number %d, colors %d, states %d\n",
 			  ap->n, ap->neighbors, i, ap->ncolors, ap->nstates);
 	for (j = 0; j < total; j++) {
-		ap->machine[j].color = *patptr++;
+		status = &(ap->machine[j]);
+		status->color = *patptr++;
 		if (ap->sharpturn && ap->neighbors > 4) {
 			int         k = *patptr++;
 
@@ -922,28 +924,23 @@ getTable(ModeInfo * mi, int i)
 				default:
 					break;
 			}
-			ap->machine[j].direction = fromTableDirection(k, ap->neighbors);
+			status->direction = fromTableDirection(k, ap->neighbors);
 		} else {
-			ap->machine[j].direction = fromTableDirection(*patptr++, ap->neighbors);
+			status->direction = fromTableDirection(*patptr++, ap->neighbors);
 		}
-		ap->machine[j].next = *patptr++;
+		status->next = *patptr++;
 	}
 	ap->truchet = False;
 }
 
 static void
-getTurk(ModeInfo * mi, int i)
+getTurk(ModeInfo * mi, int logClass, int number)
 {
 	antfarmstruct *ap = &antfarms[MI_SCREEN(mi)];
-	int         power2, j, number, total;
+	int         power2, j, total;
 
-	/* To force a number, say <i = 2;> has  i + 2 (or 4) binary digits */
-	power2 = 1 << (i + 1);
-	/* Do not want numbers which in binary are all 1's. */
-	number = NRAND(power2 - 1) + power2;
-	/* To force a particular number, say <number = 10;> */
-
-	ap->ncolors = i + 2;
+	power2 = 1 << (logClass - 1);
+	ap->ncolors = logClass;
 	ap->nstates = 1;
 	total = ap->ncolors * ap->nstates;
 	for (j = 0; j < total; j++) {
@@ -964,8 +961,59 @@ getTurk(ModeInfo * mi, int i)
 	   (ap->neighbors == 3 || ap->neighbors == 4 || ap->neighbors == 6));
 	if (MI_IS_VERBOSE(mi))
 		(void) fprintf(stdout,
-		      "ants %d, neighbors %d, Turk's number %d, colors %d\n",
-			       ap->n, ap->neighbors, number, ap->ncolors);
+			"ants %d, neighbors %d, Turk's number 0x%x, colors %d\n",
+			ap->n, ap->neighbors, number, ap->ncolors);
+}
+
+static void
+parseRule(ModeInfo * mi)
+{
+	if (rule) {
+		int n = 0, l, number = 0, logClass = 0, power2;
+		Bool aZero = False;
+
+		while (rule[n]) {
+			if (rule[n] == 'T' || rule[n] == 't') {
+				n++;
+				while (rule[n]) {
+					number = number * 10 +
+						(int) (rule[n] - '0');
+					n++;
+				}
+				getTable(mi, number);
+				return;
+			}
+			if (rule[n] == 'A' || rule[n] == 'a') {
+				if (!NRAND(NUMSTIPPLES)) {
+					getTable(mi, (int) (NRAND(NTABLES)));
+					return;
+				} else {
+					logClass = (int) (NRAND(NUMSTIPPLES - 2)) + 2;
+					/* <logClass = 4;> has 4 binary digits */
+					power2 = 1 << (logClass - 1);
+					/* Do not want numbers which in binary are all 1's. */
+					number = NRAND(power2 - 1) + power2;
+					getTurk(mi, logClass, number);
+					return;
+				}
+			} else {
+				l = rule[n] - '0';
+				if (l >= 0 && l <= 1) {
+					number = (number << 1) + l;
+					if (logClass > 0 || l != 0)
+						logClass++;
+					if (logClass > 0 && l == 0)
+						aZero = True;
+				}
+			}
+			n++;
+		}
+		if (!aZero) {
+			getTurk(mi, 2, 2); /* Default to Langton's ant */
+		} else {
+			getTurk(mi, logClass, number);
+		}
+	}
 }
 
 static void
@@ -982,15 +1030,15 @@ free_ant(Display *display, antfarmstruct *ap)
 	}
 	ap->init_bits = 0;
 	if (ap->tape != NULL) {
-		(void) free((void *) ap->tape);
+		free(ap->tape);
 		ap->tape = (unsigned char *) NULL;
 	}
 	if (ap->ants != NULL) {
-		(void) free((void *) ap->ants);
+		free(ap->ants);
 		ap->ants = (antstruct *) NULL;
 	}
 	if (ap->truchet_state != NULL) {
-		(void) free((void *) ap->truchet_state);
+		free(ap->truchet_state);
 		ap->truchet_state = (unsigned char *) NULL;
 	}
 }
@@ -1018,8 +1066,8 @@ init_ant(ModeInfo * mi)
 			XGCValues   gcv;
 
 			gcv.fill_style = FillOpaqueStippled;
-			if ((ap->stippledGC = XCreateGC(display, window, GCFillStyle,
-					&gcv)) == None) {
+			if ((ap->stippledGC = XCreateGC(display, window,
+					GCFillStyle, &gcv)) == None) {
 				free_ant(display, ap);
 				return;
 			}
@@ -1035,7 +1083,7 @@ init_ant(ModeInfo * mi)
 	if (ap->n < -MINANTS) {
 		/* if ap->n is random ... the size can change */
 		if (ap->ants != NULL) {
-			(void) free((void *) ap->ants);
+			free(ap->ants);
 			ap->ants = (antstruct *) NULL;
 		}
 		ap->n = NRAND(-ap->n - MINANTS + 1) + MINANTS;
@@ -1060,14 +1108,13 @@ init_ant(ModeInfo * mi)
 			break;
 		}
 	}
-
 	if (ap->neighbors == 6) {
 		int         nccols, ncrows;
 
-		if (ap->width < 2)
-			ap->width = 2;
-		if (ap->height < 4)
-			ap->height = 4;
+		if (ap->width < 8)
+			ap->width = 8;
+		if (ap->height < 8)
+			ap->height = 8;
 		if (size < -MINSIZE) {
 			ap->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
@@ -1084,7 +1131,7 @@ init_ant(ModeInfo * mi)
 					       MINGRIDSIZE));
 		ap->xs = ap->ys;
 		nccols = MAX(ap->width / ap->xs - 2, 2);
-		ncrows = MAX(ap->height / ap->ys - 1, 2);
+		ncrows = MAX(ap->height / ap->ys - 1, 4);
 		ap->ncols = nccols / 2;
 		ap->nrows = 2 * (ncrows / 4);
 		ap->xb = (ap->width - ap->xs * nccols) / 2 + ap->xs / 2;
@@ -1168,10 +1215,7 @@ init_ant(ModeInfo * mi)
 		ap->eyes = eyes;
 		ap->sharpturn = sharpturn;
 	}
-	if (!NRAND(NUMSTIPPLES)) {
-		getTable(mi, (int) (NRAND(NTABLES)));
-	} else
-		getTurk(mi, (int) (NRAND(NUMSTIPPLES - 1)));
+	parseRule(mi);
 	if (MI_NPIXELS(mi) > 2)
 		for (i = 0; i < (int) ap->ncolors - 1; i++)
 			ap->colors[i] = (unsigned char) (NRAND(MI_NPIXELS(mi)) +
@@ -1184,14 +1228,14 @@ init_ant(ModeInfo * mi)
 		}
 	}
 	if (ap->tape != NULL) 
-		(void) free((void *) ap->tape);
+		free(ap->tape);
 	if ((ap->tape = (unsigned char *) calloc(ap->ncols * ap->nrows,
 			sizeof (unsigned char))) == NULL) {
 		free_ant(display, ap);
 		return;
 	}
 	if (ap->truchet_state != NULL)
-		(void) free((void *) ap->truchet_state);
+		free(ap->truchet_state);
 	if ((ap->truchet_state = (unsigned char *) calloc(ap->ncols * ap->nrows,
 			sizeof (unsigned char))) == NULL) {
 		free_ant(display, ap);
@@ -1282,13 +1326,13 @@ draw_ant(ModeInfo * mi)
 		}
 		anant->state = status->next;
 
-		/* Allow step first and turn */
+		/* Allow step first then turn */
 		old_dir = ((status->direction < ANGLES) ? anant->direction : old_dir);
-#if DEBUG
+#ifdef DEBUG
 		(void) printf("old_dir %d, col %d, row %d", old_dir, anant->col, anant->row);
 #endif
 		position_of_neighbor(ap, old_dir, &(anant->col), &(anant->row));
-#if DEBUG
+#ifdef DEBUG
 		(void) printf(", ncol %d, nrow %d\n", anant->col, anant->row);
 #endif
 		draw_anant(mi, anant->direction, anant->col, anant->row);
@@ -1323,7 +1367,7 @@ release_ant(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_ant(MI_DISPLAY(mi), &antfarms[screen]);
-		(void) free((void *) antfarms);
+		free(antfarms);
 		antfarms = (antfarmstruct *) NULL;
 	}
 }
