@@ -36,6 +36,7 @@ static const char sccsid[] = "@(#)bug.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_bug
 #define PROGCLASS "Bug"
 #define HACK_INIT init_bug
 #define HACK_DRAW draw_bug
@@ -90,7 +91,7 @@ ModStruct   bug_description =
 
 #endif
 
-#if DEBUG
+#ifdef DEBUG
 #include <assert.h>
 #endif
 
@@ -185,38 +186,7 @@ position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
 
 	/* NO WRAPING */
 
-	if (bp->neighbors == 6) {
-		switch (dir) {
-			case 0:
-				col = col + 1;
-				break;
-			case 60:
-				if (!(row & 1))
-					col = col + 1;
-				row = row - 1;
-				break;
-			case 120:
-				if (row & 1)
-					col = col - 1;
-				row = row - 1;
-				break;
-			case 180:
-				col = col - 1;
-				break;
-			case 240:
-				if (row & 1)
-					col = col - 1;
-				row = row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = col + 1;
-				row = row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (bp->neighbors == 4 || bp->neighbors == 8) {
+	if (bp->neighbors == 4 || bp->neighbors == 6 || bp->neighbors == 8) {
 		switch (dir) {
 			case 0:
 				col = col + 1;
@@ -225,9 +195,19 @@ position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
 				col = col + 1;
 				row = row - 1;
 				break;
+			case 60:
+				if (!(row & 1))
+					col = col + 1;
+				row = row - 1;
+				break;
 			case 90:
 				row = row - 1;
 				break;
+			case 120:
+				if (row & 1)
+					col = col - 1;
+				row = row - 1;
+				break;
 			case 135:
 				col = col - 1;
 				row = row - 1;
@@ -239,9 +219,19 @@ position_of_neighbor(bugfarmstruct * bp, int dir, int *pcol, int *prow)
 				col = col - 1;
 				row = row + 1;
 				break;
+			case 240:
+				if (row & 1)
+					col = col - 1;
+				row = row + 1;
+				break;
 			case 270:
 				row = row + 1;
 				break;
+			case 300:
+				if (!(row & 1))
+					col = col + 1;
+				row = row + 1;
+				break;
 			case 315:
 				col = col + 1;
 				row = row + 1;
@@ -754,7 +744,7 @@ flush_buglist(bugfarmstruct * bp)
 		bp->currbug->previous->next = bp->lastbug;
 		bp->lastbug->previous = bp->currbug->previous;
 		/*bp->arr[bp->currbug->info.col + bp->currbug->info.row * bp->ncols] = 0; */
-		(void) free((void *) bp->currbug);
+		free(bp->currbug);
 	}
 
 	while (bp->lasttemp->previous != bp->firsttemp) {
@@ -762,7 +752,7 @@ flush_buglist(bugfarmstruct * bp)
 		bp->currbug->previous->next = bp->lasttemp;
 		bp->lasttemp->previous = bp->currbug->previous;
 		/*bp->arr[bp->currbug->info.col + bp->currbug->info.row * bp->ncols] = 0; */
-		(void) free((void *) bp->currbug);
+		free(bp->currbug);
 	}
 }
 
@@ -771,19 +761,19 @@ free_buglist(bugfarmstruct *bp)
 {
 	if (bp->firstbug != NULL) {
 		flush_buglist(bp);
-		(void) free((void *) bp->firstbug);
+		free(bp->firstbug);
 		bp->firstbug = (BugList *) NULL;
 	}
 	if (bp->lastbug != NULL) {
-		(void) free((void *) bp->lastbug);
+		free(bp->lastbug);
 		bp->lastbug = (BugList *) NULL;
 	}
 	if (bp->lasttemp != NULL) {
-		(void) free((void *) bp->lasttemp);
+		free(bp->lasttemp);
 		bp->lasttemp = (BugList *) NULL;
 	}
 	if (bp->firsttemp != NULL) {
-		(void) free((void *) bp->firsttemp);
+		free(bp->firsttemp);
 		bp->firsttemp = (BugList *) NULL;
 	}
 }
@@ -841,7 +831,7 @@ removefrom_buglist(bugfarmstruct * bp, BugList * ptr)
 	ptr->previous->next = ptr->next;
 	ptr->next->previous = ptr->previous;
 	bp->arr[ptr->info.col + ptr->info.row * bp->ncols] = 0;
-	(void) free((void *) ptr);
+	free(ptr);
 }
 
 static void
@@ -1017,27 +1007,27 @@ free_bug(Display *display, bugfarmstruct *bp)
 	bp->init_bits = 0;
 	if (bp->firstbug != NULL) {
 		flush_buglist(bp);
-		(void) free((void *) bp->firstbug);
+		free(bp->firstbug);
 		bp->firstbug = (BugList *) NULL;
 	}
 	if (bp->lastbug != NULL) {
-		(void) free((void *) bp->lastbug);
+		free(bp->lastbug);
 		bp->lastbug = (BugList *) NULL;
 	}
 	if (bp->lasttemp != NULL) {
-		(void) free((void *) bp->lasttemp);
+		free(bp->lasttemp);
 		bp->lasttemp = (BugList *) NULL;
 	}
 	if (bp->firsttemp != NULL) {
-		(void) free((void *) bp->firsttemp);
+		free(bp->firsttemp);
 		bp->firsttemp = (BugList *) NULL;
 	}
 	if (bp->arr != NULL) {
-		(void) free((void *) bp->arr);
+		free(bp->arr);
 		bp->arr = (BugList **) NULL;
 	}
 	if (bp->bacteria != NULL) {
-		(void) free((void *) bp->bacteria);
+		free(bp->bacteria);
 		bp->bacteria = (char *) NULL;
 	}
 }
@@ -1106,10 +1096,10 @@ init_bug(ModeInfo * mi)
 
 	bp->width = MI_WIDTH(mi);
 	bp->height = MI_HEIGHT(mi);
-	if (bp->width < 4)
-		bp->width = 4;
-	if (bp->height < 4)
-		bp->height = 4;
+	if (bp->width < 8)
+		bp->width = 8;
+	if (bp->height < 8)
+		bp->height = 8;
 
 	if (size < -MINSIZE)
 		bp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(bp->width, bp->height) /
@@ -1130,6 +1120,8 @@ init_bug(ModeInfo * mi)
 		bp->ncols = nccols / 3;
 		if (bp->ncols & 1)
 			bp->ncols--;	/* Must be odd for no wrap */
+		if (bp->ncols < 2)
+			bp->ncols = 2;
 		if (!(bp->nrows & 1))
 			bp->nrows--;	/* Must be even for no wrap */
 	} else
@@ -1139,14 +1131,14 @@ init_bug(ModeInfo * mi)
 	bp->xb = (bp->width - bp->xs * nccols) / 2;
 	bp->yb = (bp->height - bp->ys * ncrows) / 2;
 	if (bp->arr != NULL)
-		(void) free((void *) bp->arr);
+		free(bp->arr);
 	if ((bp->arr = (BugList **) calloc(bp->ncols * bp->nrows,
 			sizeof (BugList *))) == NULL) {
 		free_bug(display, bp);
 		return;
 	}
 	if (bp->bacteria != NULL)
-		(void) free((void *) bp->bacteria);
+		free(bp->bacteria);
 	if ((bp->bacteria = (char *) calloc(bp->ncols * bp->nrows,
 			sizeof (char))) == NULL) {
 		free_bug(display, bp);
@@ -1299,13 +1291,13 @@ draw_bug(ModeInfo * mi)
 		row = bp->currbug->info.row;
 		colrow = col + row * bp->ncols;
 		if (bp->currbug->info.energy-- < 0) {	/* Time to die, Bug */
-#if DEBUG
+#ifdef DEBUG
 			assert(bp->currbug == bp->arr[colrow]);
 #endif
 			/* back up one or else in void */
 			bp->currbug = bp->currbug->previous;
 			removefrom_buglist(bp, bp->arr[colrow]);
-#if DEBUG
+#ifdef DEBUG
 			assert(bp->arr[colrow] == 0);
 #endif
 			bp->arr[colrow] = 0;
@@ -1343,7 +1335,7 @@ draw_bug(ModeInfo * mi)
 				bp->currbug->info.col = ncol;
 				bp->currbug->info.row = nrow;
 				bp->currbug->info.direction = absdir;
-#if DEBUG
+#ifdef DEBUG
 				assert(bp->arr[ncolrow] == 0);
 #endif
 				bp->arr[ncolrow] = bp->currbug;
@@ -1403,7 +1395,7 @@ release_bug(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_bug(MI_DISPLAY(mi), &bugfarms[screen]);
-		(void) free((void *) bugfarms);
+		free(bugfarms);
 		bugfarms = (bugfarmstruct *) NULL;
 	}
 }
