@@ -32,6 +32,7 @@ static const char sccsid[] = "@(#)drift.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_drift
 #define PROGCLASS "Drift"
 #define HACK_INIT init_drift
 #define HACK_DRAW draw_drift
@@ -266,11 +267,11 @@ static void
 free_drift(driftstruct *dp)
 {
 	if (dp->ncpoints != NULL) {
-		(void) free((void *) dp->ncpoints);
+		free(dp->ncpoints);
 		dp->ncpoints = (int *) NULL;
 	}
 	if (dp->cpts != NULL) {
-		(void) free((void *) dp->cpts);
+		free(dp->cpts);
 		dp->cpts = (XPoint *) NULL;
 	}
 }
@@ -658,7 +659,7 @@ release_drift(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_drift(&drifts[screen]);
-		(void) free((void *) drifts);
+		free(drifts);
 		drifts = (driftstruct *) NULL;
 	}
 }
