@@ -61,7 +61,7 @@ static const char sccsid[] = "@(#)euler2d.c	5.00 2000/11/01 xlockmore";
 
 #define DEF_EULERTAIL "10"
 
-#define DEBUG_POINTED_REGION    0
+/* #define USE_POINTED_REGION    1 */
 
 static int tail_len;
 static int variable_boundary = 1;
@@ -604,7 +604,8 @@ init_euler2d(ModeInfo * mi)
 			sp->p_coef[2*(k-2)+1] /= mag;
 		}
 
-#if DEBUG_POINTED_REGION
+#ifdef USE_POINTED_REGION
+		/* Five symmetric buldges */
 		for(k=2;k<=deg_p;k++){
 			sp->p_coef[2*(k-2)+0]=0;
 			sp->p_coef[2*(k-2)+1]=0;
