@@ -34,6 +34,7 @@ static const char sccsid[] = "@(#)flag.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_flag
 #define PROGCLASS "Flag"
 #define HACK_INIT init_flag
 #define HACK_DRAW draw_flag
@@ -120,7 +121,7 @@ ModStruct   flag_description =
 
 #include "flag.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define FLAG_NAME  image_name
 #include "flag.xpm"
 #define DEFAULT_XPM 0
@@ -144,7 +145,7 @@ ModStruct   flag_description =
 #define MAXINITSIZE 6
 #define MININITSIZE 2
 #define MINAMP 5
-#define MAXAMP 10
+#define MAXAMP 20
 #define MAXW(fp) (MAXSCALE * (fp)->image->width + 2 * MAXAMP + (fp)->pointsize + 2 * (fp)->sofs)
 #define MAXH(fp) (MAXSCALE * (fp)->image->height+ 2 * MAXAMP + (fp)->pointsize + 2 * (fp)->sofs)
 #define MINW(fp) (MINSCALE * (fp)->image->width + 2 * MINAMP + (fp)->pointsize)
@@ -171,8 +172,8 @@ typedef struct {
 	XImage     *logo;
 	Colormap    cmap;
 	unsigned long black;
-	int         graphics_format;
 	GC          backGC;
+	int         graphics_format;
 } flagstruct;
 
 static XFontStruct *messagefont = None;
@@ -211,25 +212,33 @@ static void
 affiche(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
-	int         x, y, xp, yp;
+	int         x, y, xp, yp, temp;
 	flagstruct *fp = &flags[MI_SCREEN(mi)];
 
 	for (x = 0; x < fp->image->width; x++)
 		for (y = fp->image->height - 1; y >= 0; y--) {
-			xp = (int) (fp->size * (float) x) +
-				fp->stab[(fp->sidx + x + y) % ANGLES];
-			yp = (int) (fp->size * (float) y) +
-				fp->stab[(fp->sidx + 4 * x + y + y) % ANGLES];
+			temp = (fp->sidx - x + y) % ANGLES;
+			if (temp < 0)
+				temp = ANGLES - 1 + temp;
+			xp = (int) (fp->size * (float) x) + fp->stab[temp];
+			temp = (fp->sidx - 4 * x + y + y) % ANGLES;
+			if (temp < 0)
+				temp = ANGLES - 1 + temp;
+			yp = (int) (fp->size * (float) y) + fp->stab[temp];
 			if (MI_NPIXELS(mi) <= 2 || fp->graphics_format < IS_XPM ||
 			    fp->choice == MESSAGE_FLAG) {
 				if (((int) !invert) ^ XGetPixel(fp->image, x, y))
 					XSetForeground(display, fp->backGC, fp->black);
 				else if (MI_NPIXELS(mi) <= 2)
 					XSetForeground(display, fp->backGC, MI_WHITE_PIXEL(mi));
-				else
+				else {
+					temp = (y - x + fp->sidx +
+						fp->startcolor) % MI_NPIXELS(mi);
+					if (temp < 0)
+						temp = MI_NPIXELS(mi) - 1 + temp;
 					XSetForeground(display, fp->backGC,
-						       MI_PIXEL(mi, (y + x + fp->sidx + fp->startcolor) %
-							    MI_NPIXELS(mi)));
+						MI_PIXEL(mi, temp));
+				}
 			} else {
 /*
  * PURIFY sometimes reports thousands of Array Bounds Reads and Free Memory
@@ -310,7 +319,7 @@ getText(ModeInfo * mi, XImage ** image)
 			  text1[strlen(text1) - 1] == '\n'))
 		text1[strlen(text1) - 1] = 0;
 	if ((text2 = (char *) strdup(text1)) == NULL) {
-		(void) free((void *) text1);
+		free(text1);
 		return False;
 	}
 
@@ -336,8 +345,8 @@ getText(ModeInfo * mi, XImage ** image)
 
 	if ((text_pixmap = XCreatePixmap(display, MI_WINDOW(mi),
 			width, height, 1)) == None) {
-		(void) free((void *) text1);
-		(void) free((void *) text2);
+		free(text1);
+		free(text2);
 		return False;
 	}
 
@@ -347,8 +356,8 @@ getText(ModeInfo * mi, XImage ** image)
 	if ((gc = XCreateGC(display, text_pixmap,
 		       GCFont | GCForeground | GCBackground, &gcv)) == None) {
 	    XFreePixmap(display, text_pixmap);
-		(void) free((void *) text1);
-		(void) free((void *) text2);
+		free(text1);
+		free(text2);
 		return False;
 	}
 	XFillRectangle(display, text_pixmap, gc, 0, 0, width, height);
@@ -374,8 +383,8 @@ getText(ModeInfo * mi, XImage ** image)
 				   line, strlen(line));
 		lines++;
 	}
-	(void) free((void *) text1);
-	(void) free((void *) text2);
+	free(text1);
+	free(text2);
 	/*XUnloadFont(display, messagefont->fid); */
 	XFreeGC(display, gc);
 
@@ -397,7 +406,7 @@ init_stuff(ModeInfo * mi)
 
 	if (!fp->logo) {
 		getImage(mi, &fp->logo, FLAG_WIDTH, FLAG_HEIGHT, FLAG_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 			 DEFAULT_XPM, FLAG_NAME,
 #endif
 			 &fp->graphics_format, &fp->cmap, &fp->black);
@@ -467,7 +476,7 @@ release_flag(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_flag(MI_DISPLAY(mi), &flags[screen]);
-		(void) free((void *) flags);
+		free(flags);
 		flags = (flagstruct *) NULL;
 	}
 	if (messagefont != None) {
@@ -528,10 +537,10 @@ init_flag(ModeInfo * mi)
 	} else {
 		int         depth = MI_DEPTH(mi);
 
-		if (depth >= 24 && depth < 32)
-			depth = 32;
 		if (fp->graphics_format < IS_XPM)
 			depth = 1;
+		if (depth >= 24 && depth < 32)
+			depth = 32;
 		if ((fp->image = XCreateImage(display, MI_VISUAL(mi), depth,
 			 (fp->graphics_format < IS_XPM) ? XYBitmap : ZPixmap,
 					 0, fp->logo->data,
@@ -540,8 +549,10 @@ init_flag(ModeInfo * mi)
 			free_flag(display, fp);
 			return;
 		}
-		fp->image->byte_order = LSBFirst;
-		fp->image->bitmap_bit_order = LSBFirst;
+		if (fp->graphics_format < IS_XPM) {
+			fp->image->byte_order = LSBFirst;
+			fp->image->bitmap_bit_order = LSBFirst;
+		}
 	}
 	fp->samp = MAXAMP;	/* Amplitude */
 	fp->sofs = 20;		/* ???????? */
@@ -639,7 +650,7 @@ refresh_flag(ModeInfo * mi)
 		return;
 	fp = &flags[MI_SCREEN(mi)];
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	/* This is needed when another program changes the colormap. */
 	free_flag(MI_DISPLAY(mi), fp);
 	init_flag(mi);
