@@ -52,12 +52,14 @@ static const char sccsid[] = "@(#)flow.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_flow
 #define PROGCLASS "Flow"
 #define HACK_INIT init_flow
 #define HACK_DRAW draw_flow
 #define flow_opts xlockmore_opts
 #define DEFAULTS "*delay: 1000 \n" \
 "*count: 1024 \n" \
+"*size: -10 \n" \
 "*cycles: 3000 \n" \
 "*ncolors: 200 \n"
 #define SMOOTH_COLORS
@@ -68,6 +70,8 @@ static const char sccsid[] = "@(#)flow.c	5.00 2000/11/01 xlockmore";
 
 #ifdef MODE_flow
 
+#define MINTRAIL 2
+
 ModeSpecOpt flow_opts =
 {0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
@@ -75,7 +79,7 @@ ModeSpecOpt flow_opts =
 ModStruct   flow_description = {
 	"flow", "init_flow", "draw_flow", "release_flow",
 	"refresh_flow", "init_flow", (char *) NULL, &flow_opts,
-	1000, 1024, 3000, 1, 64, 1.0, "",
+	1000, 1024, 3000, -10, 64, 1.0, "",
 	"Shows dynamic strange attractors", 0, NULL
 };
 
@@ -92,9 +96,11 @@ typedef struct {
 } Par;
 
 /* Macros */
-#define X(t,b)	(sp->p[t][b].x)
-#define Y(t,b)	(sp->p[t][b].y)
-#define Z(t,b)	(sp->p[t][b].z)
+#define IX(C)   ((C) * sp->beecount + sp->cnsegs[(C)])
+#define B(t,b)	(*(sp->p + (t) + (b) * sp->taillen))
+#define X(t,b)	(B((t),(b)).x)
+#define Y(t,b)	(B((t),(b)).y)
+#define Z(t,b)	(B((t),(b)).z)
 #define balance_rand(v)	((LRAND()/MAXRAND*(v))-((v)/2))	/* random around 0 */
 #define SCALE_X(A) (sp->width/2+sp->width/sp->size*(A))
 #define SCALE_Y(A) (sp->height/2+sp->height/sp->size*(A))
@@ -106,6 +112,7 @@ typedef struct {
 	int         height;
 	int         count;
 	double      size;
+	int         taillen;
 
 	int         beecount;	/* number of bees */
 	XSegment   *csegs;	    /* bee lines */
@@ -118,7 +125,7 @@ typedef struct {
 		double  depth;
 		double  height;
 	}           view;
-	dvector    *p[2];   /* bee positions x[time][bee#] */
+	dvector    *p;   /* bee positions x[time][bee#] */
 	struct {
 		double  theta;
 		double  dtheta;
@@ -186,29 +193,17 @@ Duffing(Par par, double x, double y, double z)
 	return d;
 }
 
+#define deallocate(p,t) if (p!=NULL) {free(p); p=(t*)NULL; }
+#define allocate(p,t,s) if ((p=(t*)malloc(sizeof(t)*s))==NULL)\
+{free_flow(sp);return;}
+
 static void
 free_flow(flowstruct *sp)
 {
-	if (sp->csegs != NULL) {
-		(void) free((void *) sp->csegs);
-		sp->csegs = (XSegment *) NULL;
-	}
-	if (sp->cnsegs != NULL) {
-		(void) free((void *) sp->cnsegs);
-		sp->cnsegs = (int *) NULL;
-	}
-	if (sp->old_segs != NULL) {
-		(void) free((void *) sp->old_segs);
-		sp->old_segs = (XSegment *) NULL;
-	}
-	if (sp->p[0] != NULL) {
-		(void) free((void *) sp->p[0]);
-		sp->p[0] = (dvector *) NULL;
-	}
-	if (sp->p[1] != NULL) {
-		(void) free((void *) sp->p[1]);
-		sp->p[1] = (dvector *) NULL;
-	}
+	deallocate(sp->csegs, XSegment);
+	deallocate(sp->cnsegs, int);
+	deallocate(sp->old_segs, XSegment);
+	deallocate(sp->p, dvector);
 }
 
 
@@ -238,6 +233,13 @@ init_flow(ModeInfo * mi)
 	sp->tumble.dphi = 0.001;
 	sp->view.height = 0;
 	sp->view.depth = 0; /* no perspective view */
+	sp->taillen = MI_SIZE(mi);
+	if (sp->taillen < -MINTRAIL) {
+		/* Change by sqr so its seems more variable */
+		sp->taillen = NRAND((int) sqrt((double) (-sp->taillen - MINTRAIL + 1)));
+		sp->taillen = sp->taillen * sp->taillen + MINTRAIL;
+        } else if (sp->taillen < MINTRAIL)
+                sp->taillen = MINTRAIL;
 
 	switch (NRAND(8)) {
 	case 0:
@@ -347,18 +349,11 @@ init_flow(ModeInfo * mi)
 	MI_CLEARWINDOW(mi);
 
 	/* Allocate memory. */
-	if (((sp->csegs = (XSegment *) malloc(sizeof (XSegment) *
-			sp->beecount * MI_NPIXELS(mi))) == NULL) ||
-	    ((sp->cnsegs = (int *) malloc(sizeof (int) *
-			MI_NPIXELS(mi))) == NULL) ||
-	    ((sp->old_segs = (XSegment *) malloc(sizeof (XSegment) *
-			sp->beecount)) == NULL) ||
-	    ((sp->p[0] = (dvector *) malloc(sizeof (dvector) *
-			sp->beecount)) == NULL) || 
-	    ((sp->p[1] = (dvector *) malloc(sizeof (dvector) *
-			sp->beecount)) == NULL)) {
-		free_flow(sp);
-		return;
+	if (sp->csegs == NULL) {
+	  allocate(sp->csegs, XSegment, sp->beecount * MI_NPIXELS(mi) * sp->taillen);
+	  allocate(sp->cnsegs, int, MI_NPIXELS(mi));
+	  allocate(sp->old_segs, XSegment, sp->beecount * sp->taillen);
+	  allocate(sp->p, dvector, sp->beecount * sp->taillen);
 	}
 
 	/* Initialize point positions, velocities, etc. */
@@ -377,7 +372,7 @@ draw_flow(ModeInfo * mi)
 	Window      window = MI_WINDOW(mi);
 	GC          gc = MI_GC(mi);
 	int         b, c, i;
-	int         col, ix;
+	int         col, begin, end;
 	double      M[3][3]; /* transformation matrix */
 	flowstruct *sp;
 
@@ -423,10 +418,9 @@ draw_flow(ModeInfo * mi)
 			}
 			continue;
 		}
-		/* Age the arrays. */
-		X(1, b) = X(0, b);
-		Y(1, b) = Y(0, b);
-		Z(1, b) = Z(0, b);
+		/* Age the tail. */
+		for(i = sp->taillen - 1; i > 0; i--)
+		  B(i, b) = B(i-1, b);
 
 		/* 2nd order Kunge Kutta */
 		{
@@ -447,7 +441,6 @@ draw_flow(ModeInfo * mi)
 
 		/* Colour according to bee */
 		col = b % (MI_NPIXELS(mi) - 1);
-		ix = col * sp->beecount + sp->cnsegs[col];
 
 		/* Fill the segment lists. */
 
@@ -517,35 +510,44 @@ draw_flow(ModeInfo * mi)
 			}
 		}
 
-		for(i=0; i<2; i++){
+		begin = 0; /* begin new trail */
+		end = MIN(sp->taillen, sp->count); /* short trails at first */
+		for(i=0; i < end; i++){
 			double x=X(i,b)-sp->centre.x;
 			double y=Y(i,b)-sp->centre.y;
 			double z=Z(i,b)-sp->centre.z;
-			double X=M[0][0]*x + M[0][1]*y + M[0][2]*z;
-			double Y=M[1][0]*x + M[1][1]*y + M[1][2]*z;
-			double Z=M[2][0]*x + M[2][1]*y + M[2][2]*z+sp->view.height;
+			double XM=M[0][0]*x + M[0][1]*y + M[0][2]*z;
+			double YM=M[1][0]*x + M[1][1]*y + M[1][2]*z;
+			double ZM=M[2][0]*x + M[2][1]*y + M[2][2]*z+sp->view.height;
 			double absx, absy;
 			if(sp->view.depth){
-				if(X <= 0) break;
-				absx=SCALE_X(sp->view.depth*Y/X);
-				absy=SCALE_Y(sp->view.depth*Z/X);
-				if(absx < -sp->width || absx > 2*sp->width ||
-				   absy < -sp->height || absy > 2*sp->height)
-					break;
+				if(XM <= 0){
+					/* off screen - new trail */
+					begin = i + 1;
+					continue;
+				}
+				absx=SCALE_X(sp->view.depth*YM/XM);
+				absy=SCALE_Y(sp->view.depth*ZM/XM);
 			}else{
-				absx=SCALE_X(X);
-				absy=SCALE_Y(Y);
+				absx=SCALE_X(XM);
+				absy=SCALE_Y(YM);
 			}
-			if(i){
-				sp->csegs[ix].x1 = (short) absx;
-				sp->csegs[ix].y1 = (short) absy;
-			}else{
-				sp->csegs[ix].x2 = (short) absx;
-				sp->csegs[ix].y2 = (short) absy;
+			if(absx <= 0 || absx >= sp->width ||
+			   absy <= 0 || absy >= sp->height) {
+				/* off screen - new trail */
+				begin = i + 1;
+				continue;
+			}
+			if(i > begin) {  /* complete previous segment */ 
+				sp->csegs[IX(col)].x2 = (short) absx;
+				sp->csegs[IX(col)].y2 = (short) absy;			  
+				sp->cnsegs[col]++;
+			}
+			if(i < end -1){  /* start new segment */
+				sp->csegs[IX(col)].x1 = (short) absx;
+				sp->csegs[IX(col)].y1 = (short) absy;
 			}
 		}
-		if(i == 2) /* both assigned */
-			sp->cnsegs[col]++;
     }
 	if (sp->count) { /* erase */
 		XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
@@ -583,7 +585,7 @@ release_flow(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_flow(&flows[screen]);
-		(void) free((void *) flows);
+		free(flows);
 		flows = (flowstruct *) NULL;
 	}
 }
