@@ -1,7 +1,7 @@
 /* atlantis --- Shows moving 3D sea animals */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)atlantis.c	5.01 2001/04/17 xlockmore";
+static const char sccsid[] = "@(#)atlantis.c	5.12 2004/03/03 xlockmore";
 
 #endif
 
@@ -30,12 +30,20 @@ static const char sccsid[] = "@(#)atlantis.c	5.01 2001/04/17 xlockmore";
  * Thanks goes also to Brian Paul for making it possible and inexpensive
  * to use OpenGL at home.
  *
- * My e-mail address is lassauge@mail.dotcom.fr
+ * My e-mail address is <lassauge AT users.sourceforge.net>
  *
  * Eric Lassauge  (May-13-1998)
  *
  * REVISION HISTORY:
  *
+ * E.Lassauge - 03-Feb-2004: Integrated diffs from xscreensaver-4.15 : corrected gradient stuff
+ *
+ * E.Lassauge - 09-Apr-2003: Corrected parse_image_data to finally have the texture !!!
+ *
+ * E.Lassauge - 21-Jan-2002: Integrated diffs from xscreensaver-4.00
+ *
+ * E.Lassauge - 28-Nov-2001: Integrated diffs from xscreensaver-3.34
+ *
  * Jamie Zawinski, 2-Apr-01:  - The fishies were inside out!  The back faces
  *                              were being drawn, not the front faces.
  *                            - Added a texture to simulate light from the
@@ -60,7 +68,6 @@ static const char sccsid[] = "@(#)atlantis.c	5.01 2001/04/17 xlockmore";
  *                        Add support for -/+ wireframe (t'was so easy to do!)
  *
  * TODO :
- *        - add a sort of background image
  *        - better handling of sizes and speeds
  *        - test standalone and module modes
  *        - purify it (!)
@@ -105,22 +112,30 @@ static const char sccsid[] = "@(#)atlantis.c	5.01 2001/04/17 xlockmore";
  * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
  */
 
+#define DEF_TEXTURE 	"True"
+#define DEF_GRADIENT 	"False"
+#define DEF_WHALESPEED  "250"
+
 #ifdef STANDALONE
-#define PROGCLASS "Atlantis"
-#define HACK_INIT init_atlantis
-#define HACK_DRAW draw_atlantis
-#define atlantis_opts xlockmore_opts
-#define DEFAULTS "*delay: 18000 \n" \
- "*count: 4 \n" \
- "*cycles: 100 \n" \
- "*size: 6000 \n" \
- "*showFps:    False \n" \
- "*wireframe:  False \n" \
- "*whalespeed: 250\n"
-#include "xlockmore.h"		/* from the xscreensaver distribution */
+# define MODE_atlantis
+# define PROGCLASS	"Atlantis"
+# define HACK_INIT	init_atlantis
+# define HACK_DRAW	draw_atlantis
+# define HACK_RESHAPE	reshape_atlantis
+# define atlantis_opts	xlockmore_opts
+# define DEFAULTS	"*delay:       25000 \n" \
+			 "*count:          4 \n" \
+			 "*showFPS:    False \n" \
+			 "*cycles:       100 \n" \
+			 "*size:        6000 \n" \
+			 "*wireframe:  False \n" \
+			 "*texture:    " DEF_TEXTURE    " \n" \
+			 "*gradient:   " DEF_GRADIENT   " \n" \
+			 "*whalespeed: " DEF_WHALESPEED " \n"
+# include "xlockmore.h"		/* from the xscreensaver distribution */
 #else /* !STANDALONE */
-#include "xlock.h"		/* from the xlockmore distribution */
-#include "visgl.h"
+# include "xlock.h"		/* from the xlockmore distribution */
+# include "visgl.h"
 #endif /* !STANDALONE */
 
 #ifdef MODE_atlantis
@@ -128,39 +143,41 @@ static const char sccsid[] = "@(#)atlantis.c	5.01 2001/04/17 xlockmore";
 #include "atlantis.h"
 #include <GL/glu.h>
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
-#include "xpm-ximage.h"
+#ifdef HAVE_XPM
+# include "xpm-ximage.h"
 
-#ifdef STANDALONE
-#include "../images/sea-texture.xpm"
-#else /* !STANDALONE */
-#include "pixmaps/sea-texture.xpm"
-#endif /* !STANDALONE */
+# ifdef STANDALONE
+#  include "../images/sea-texture.xpm"
+# else /* !STANDALONE */
+#  include "pixmaps/sea-texture.xpm"
+# endif /* !STANDALONE */
 #endif
 
-#define DEF_WHALESPEED  "250"
-#define DEF_TEXTURE     "True"
-
 static int  whalespeed;
 static int  do_texture;
+static int  do_gradient;
 
 static XrmOptionDescRec opts[] =
 {
-     {(char *) "-whalespeed", (char *) ".atlantis.whalespeed", XrmoptionSepArg, (caddr_t) NULL},
+     {(char *) "-whalespeed", (char *) ".atlantis.whalespeed", XrmoptionSepArg,(caddr_t)NULL},
      {(char *) "-texture",    (char *) ".atlantis.texture",    XrmoptionNoArg, (caddr_t)"True"},
      {(char *) "+texture",    (char *) ".atlantis.texture",    XrmoptionNoArg, (caddr_t)"False"},
+     {(char *) "-gradient",   (char *) ".atlantis.gradient",   XrmoptionNoArg, (caddr_t)"True"},
+     {(char *) "+gradient",   (char *) ".atlantis.gradient",   XrmoptionNoArg, (caddr_t)"False"},
 };
 
 static argtype vars[] =
 {
  {(caddr_t *) & whalespeed, (char *) "whalespeed", (char *) "WhaleSpeed", (char *) DEF_WHALESPEED, t_Int},
  {(caddr_t *) & do_texture, (char *) "texture",    (char *) "Texture",    (char *) DEF_TEXTURE,    t_Bool},
+ {(caddr_t *) & do_gradient,(char *) "gradient",   (char *) "Gradient",   (char *) DEF_GRADIENT,   t_Bool},
 };
 
 static OptionStruct desc[] =
 {
 	{(char *) "-whalespeed num", (char *) "speed of whales and the dolphin"},
-	{(char *) "-texture num",    (char *) "whether to introduce water-like distortion"},
+	{(char *) "-texture",        (char *) "whether to introduce water-like distortion"},
+	{(char *) "-gradient",       (char *) "whether to introduce gradient-filled background"},
 };
 
 ModeSpecOpt atlantis_opts =
@@ -170,28 +187,27 @@ ModeSpecOpt atlantis_opts =
 ModStruct   atlantis_description =
 {"atlantis", "init_atlantis", "draw_atlantis", "release_atlantis",
  "refresh_atlantis", "change_atlantis", (char *) NULL, &atlantis_opts,
- 18000, NUM_SHARKS, SHARKSPEED, SHARKSIZE, 64, 1.0, "",
+ 25000, NUM_SHARKS, SHARKSPEED, SHARKSIZE, 64, 1.0, "",
  "Shows moving sharks/whales/dolphin", 0, NULL};
 
 #endif
 
 static atlantisstruct *atlantis = (atlantisstruct *) NULL;
 
-static void
+#ifdef HAVE_XPM
+static Bool
 parse_image_data(ModeInfo *mi)
 {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
   atlantisstruct *ap = &atlantis[MI_SCREEN(mi)];
 
   if ((ap->texture = xpm_to_ximage (MI_DISPLAY(mi),
                                MI_VISUAL(mi),
                                MI_COLORMAP(mi),
                                sea_texture)) == None)
-    do_texture = False;
-#else
-  do_texture = False;
-#endif
+	return False;
+  return True;
 }
+#endif
 
 static void
 InitFishs(atlantisstruct * ap)
@@ -206,7 +222,7 @@ InitFishs(atlantisstruct * ap)
 		ap->sharks[i].v = 1.0;
 	}
 
-	/* Random whae direction */
+	/* Random whale direction */
 	ap->whaledir = LRAND() & 1;
 
 	ap->dolph.x = 30000.0;
@@ -287,9 +303,17 @@ Init(ModeInfo * mi)
             glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);
           }
 
-        if (!ap->wire && do_texture && !ap->texture)
-              parse_image_data (mi);
-        if (ap->wire || !do_texture)
+#ifdef HAVE_XPM
+        if (ap->wire)
+		do_texture = False;
+        if (do_texture && !ap->texture) {
+        	do_texture = parse_image_data (mi);
+            	clear_gl_error();
+	}
+#else
+	do_texture = False;
+#endif
+        if (!do_texture)
           {
             glDisable(GL_TEXTURE_2D);
           }
@@ -299,46 +323,50 @@ Init(ModeInfo * mi)
             GLfloat t_plane[] = { 0, 0, 1, 0 };
             GLfloat scale = 0.0005;
 
-
-            clear_gl_error();
+#ifdef HAVE_XPM
             glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                          ap->texture->width, ap->texture->height, 0,
                          GL_RGBA, GL_UNSIGNED_BYTE,
                          ap->texture->data);
-            check_gl_error("texture");
+            if (check_gl_error("texture")) {
+	       do_texture = False;
+               glDisable(GL_TEXTURE_2D);
+	    } else {
 
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+              glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 
-            glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+              glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 
-            glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-            glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
-            glTexGenfv(GL_S, GL_EYE_PLANE, s_plane);
-            glTexGenfv(GL_T, GL_EYE_PLANE, t_plane);
+              glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+              glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
+              glTexGenfv(GL_S, GL_EYE_PLANE, s_plane);
+              glTexGenfv(GL_T, GL_EYE_PLANE, t_plane);
 
-            glEnable(GL_TEXTURE_GEN_S);
-            glEnable(GL_TEXTURE_GEN_T);
-            glEnable(GL_TEXTURE_2D);
+              glEnable(GL_TEXTURE_GEN_S);
+              glEnable(GL_TEXTURE_GEN_T);
+              glEnable(GL_TEXTURE_2D);
 
-            glMatrixMode(GL_TEXTURE);
-            glLoadIdentity();
-            glScalef(scale, scale, 1);
-            glMatrixMode(GL_MODELVIEW);
+              glMatrixMode(GL_TEXTURE);
+              glLoadIdentity();
+              glScalef(scale, scale, 1);
+              glMatrixMode(GL_MODELVIEW);
+	    }
+#endif
           }
 
 	InitFishs(ap);
 
 	/* Add a little randomness */
-	fblue = ((float) (NRAND(50)) / 100.0) + 0.50;
+	fblue = ((float) (NRAND(30)) / 100.0) + 0.70;
 	fgreen = fblue * 0.56;
 	glClearColor(0.0, fgreen, fblue, 0.0);
 }
 
-static void
-Reshape(ModeInfo * mi, int width, int height)
+void
+reshape_atlantis(ModeInfo * mi, int width, int height)
 {
 	atlantisstruct *ap = &atlantis[MI_SCREEN(mi)];
 
@@ -350,6 +378,58 @@ Reshape(ModeInfo * mi, int width, int height)
 	glMatrixMode(GL_MODELVIEW);
 }
 
+/* Fill the background with a gradient -- thanks to 
+   Phil Carrig <pod@internode.on.net> for figuring out
+   how to do this more efficiently!
+ */
+static void
+clear_tank (atlantisstruct * ap)
+{
+  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+
+  if (do_gradient && !ap->wire)
+    {
+      GLfloat top[3] = { 0.00, 0.40, 0.70 };
+      GLfloat bot[3] = { 0.00, 0.05, 0.18 };
+
+      glMatrixMode(GL_PROJECTION);
+      glPushMatrix();
+      {
+        glLoadIdentity();
+        glMatrixMode(GL_MODELVIEW);
+        glPushMatrix();
+        {
+          glLoadIdentity();
+
+          /* save GL_COLOR_MATERIAL, GL_COLOR_MATERIAL_FACE, etc.
+             This stalls the pipeline, so it would be better to do this
+             with explicit enable/disable calls, but I can't figure
+             out how to undo the glEnable() and glColor() calls below!
+             Simply calling glDisable(GL_COLOR_MATERIAL) is insufficient!
+           */
+          glPushAttrib (GL_LIGHTING_BIT);
+          {
+            glEnable (GL_COLOR_MATERIAL);
+
+            glShadeModel(GL_SMOOTH);
+            glBegin(GL_QUADS);
+            glColor3f (bot[0], bot[1], bot[2]); glVertex3f (-1, -1, 1);
+            glColor3f (bot[0], bot[1], bot[2]); glVertex3f ( 1, -1, 1);
+            glColor3f (top[0], top[1], top[2]); glVertex3f ( 1,  1, 1);
+            glColor3f (top[0], top[1], top[2]); glVertex3f (-1,  1, 1);
+            glEnd();
+          }
+          glPopAttrib();
+        }
+        glPopMatrix();
+      }
+      glMatrixMode(GL_PROJECTION);
+      glPopMatrix();
+
+      glMatrixMode(GL_MODELVIEW);
+    }
+}
+
 static void
 Animate(atlantisstruct * ap)
 {
@@ -372,7 +452,7 @@ AllDisplay(atlantisstruct * ap)
 {
 	int         i;
 
-	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+	clear_tank(ap);
 
 	for (i = 0; i < ap->num_sharks; i++) {
 		glPushMatrix();
@@ -431,7 +511,7 @@ init_atlantis(ModeInfo * mi)
 		if ((ap->sharks = (fishRec *) calloc(ap->num_sharks,
 						sizeof (fishRec))) == NULL) {
 			/* free everything up to now */
-			(void) free((void *) atlantis);
+			(void)free((void *)atlantis);
 			atlantis = (atlantisstruct *) NULL;
 			return;
 		}
@@ -456,7 +536,7 @@ init_atlantis(ModeInfo * mi)
 	}
 	if ((ap->glx_context = init_GL(mi)) != NULL) {
 
-		Reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
+		reshape_atlantis(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
 		glDrawBuffer(GL_BACK);
 		Init(mi);
 		AllDisplay(ap);
@@ -491,6 +571,7 @@ draw_atlantis(ModeInfo * mi)
 
 	AllDisplay(ap);
 	Animate(ap);
+
 	if (MI_IS_FPS(mi)) do_fps (mi);
 	glXSwapBuffers(display, window);
 }
@@ -513,13 +594,13 @@ release_atlantis(ModeInfo * mi)
 			atlantisstruct *ap = &atlantis[screen];
 
 			if (ap->sharks)
-				(void) free((void *) ap->sharks);
+				(void)free((void *)ap->sharks);
 			if (ap->texture)
 			{
 				XDestroyImage(ap->texture);
 			}
 		}
-		(void) free((void *) atlantis);
+		(void)free((void *)atlantis);
 		atlantis = (atlantisstruct *) NULL;
 	}
 	FreeAllGL(mi);
