@@ -6,8 +6,6 @@ static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
 
 #endif
 
-#undef DEBUG_LISTS
-
 /*-
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted,
@@ -44,7 +42,8 @@ static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
  * Marcelo F. Vianna (Jun-01-1997)
  *
  * Revision History:
- * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge@mail.dotcom.fr>
+ * 05-Apr-2002: Removed all gllist uses (fix some bug with nvidia driver)
+ * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge AT users.sourceforge.net>
  * 01-Nov-2000: Allocation checks
  * 01-Jan-1998: Mode separated from escher and renamed
  * 08-Jun-1997: New scene implemented: "Impossible Cage" based in a M.C.
@@ -77,19 +76,21 @@ static const char sccsid[] = "@(#)cage.c	5.01 2001/03/01 xlockmore";
  * In real OpenGL, PseudoColor DO NOT support texture map (as far as I know).
  */
 
+#ifdef VMS
 /*-
  * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
  * otherwise caddr_t is not defined correctly
  */
-
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+#define MODE_cage
 #define PROGCLASS "Cage"
 #define HACK_INIT init_cage
 #define HACK_DRAW draw_cage
 #define cage_opts xlockmore_opts
-#define DEFAULTS "*delay: 25000 \n" \
+#define DEFAULTS "*delay: 80000 \n" \
  "*showFps  : False \n" \
  "*wireframe: False \n"
 #include "xlockmore.h"		/* from the xscreensaver distribution */
@@ -111,7 +112,7 @@ ModeSpecOpt cage_opts =
 ModStruct   cage_description =
 {"cage", "init_cage", "draw_cage", "release_cage",
  "draw_cage", "change_cage", (char *) NULL, &cage_opts,
- 25000, 1, 1, 1, 1.0, 4, "",
+ 80000, 1, 1, 1, 1.0, 4, "",
  "Shows the Impossible Cage, an Escher-like GL scene", 0, NULL};
 
 #endif
@@ -133,7 +134,6 @@ ModStruct   cage_description =
 typedef struct {
 	GLint       WindH, WindW;
 	GLfloat     step;
-	Bool        AreObjectsDefined[MaxObj];
 	GLXContext *glx_context;
 } cagestruct;
 
@@ -158,159 +158,143 @@ static float MaterialWhite[] =
 {0.7, 0.7, 0.7, 1.0};
 
 static cagestruct *cage = (cagestruct *) NULL;
-static GLuint objects = 0;
 
 #define PlankWidth      3.0
 #define PlankHeight     0.35
 #define PlankThickness  0.15
 
 static Bool 
-draw_woodplank(cagestruct * cp)
+draw_woodplank()
 {
-	if (!cp->AreObjectsDefined[ObjWoodPlank]) {
-		glNewList(objects + ObjWoodPlank, GL_COMPILE_AND_EXECUTE);
-		if (glGetError() != GL_NO_ERROR) {
-			return False;
-		}
-		glBegin(GL_QUADS);
-		glNormal3f(0, 0, 1);
-		glTexCoord2f(0, 0);
-		glVertex3f(-PlankWidth, -PlankHeight, PlankThickness);
-		glTexCoord2f(1, 0);
-		glVertex3f(PlankWidth, -PlankHeight, PlankThickness);
-		glTexCoord2f(1, 1);
-		glVertex3f(PlankWidth, PlankHeight, PlankThickness);
-		glTexCoord2f(0, 1);
-		glVertex3f(-PlankWidth, PlankHeight, PlankThickness);
-		glNormal3f(0, 0, -1);
-		glTexCoord2f(0, 0);
-		glVertex3f(-PlankWidth, PlankHeight, -PlankThickness);
-		glTexCoord2f(1, 0);
-		glVertex3f(PlankWidth, PlankHeight, -PlankThickness);
-		glTexCoord2f(1, 1);
-		glVertex3f(PlankWidth, -PlankHeight, -PlankThickness);
-		glTexCoord2f(0, 1);
-		glVertex3f(-PlankWidth, -PlankHeight, -PlankThickness);
-		glNormal3f(0, 1, 0);
-		glTexCoord2f(0, 0);
-		glVertex3f(-PlankWidth, PlankHeight, PlankThickness);
-		glTexCoord2f(1, 0);
-		glVertex3f(PlankWidth, PlankHeight, PlankThickness);
-		glTexCoord2f(1, 1);
-		glVertex3f(PlankWidth, PlankHeight, -PlankThickness);
-		glTexCoord2f(0, 1);
-		glVertex3f(-PlankWidth, PlankHeight, -PlankThickness);
-		glNormal3f(0, -1, 0);
-		glTexCoord2f(0, 0);
-		glVertex3f(-PlankWidth, -PlankHeight, -PlankThickness);
-		glTexCoord2f(1, 0);
-		glVertex3f(PlankWidth, -PlankHeight, -PlankThickness);
-		glTexCoord2f(1, 1);
-		glVertex3f(PlankWidth, -PlankHeight, PlankThickness);
-		glTexCoord2f(0, 1);
-		glVertex3f(-PlankWidth, -PlankHeight, PlankThickness);
-		glNormal3f(1, 0, 0);
-		glTexCoord2f(0, 0);
-		glVertex3f(PlankWidth, -PlankHeight, PlankThickness);
-		glTexCoord2f(1, 0);
-		glVertex3f(PlankWidth, -PlankHeight, -PlankThickness);
-		glTexCoord2f(1, 1);
-		glVertex3f(PlankWidth, PlankHeight, -PlankThickness);
-		glTexCoord2f(0, 1);
-		glVertex3f(PlankWidth, PlankHeight, PlankThickness);
-		glNormal3f(-1, 0, 0);
-		glTexCoord2f(0, 0);
-		glVertex3f(-PlankWidth, PlankHeight, PlankThickness);
-		glTexCoord2f(1, 0);
-		glVertex3f(-PlankWidth, PlankHeight, -PlankThickness);
-		glTexCoord2f(1, 1);
-		glVertex3f(-PlankWidth, -PlankHeight, -PlankThickness);
-		glTexCoord2f(0, 1);
-		glVertex3f(-PlankWidth, -PlankHeight, PlankThickness);
-		glEnd();
-		glEndList();
-		cp->AreObjectsDefined[ObjWoodPlank] = True;
-#ifdef DEBUG_LISTS
-		(void) printf("WoodPlank drawn SLOWLY\n");
-#endif
-	} else {
-		glCallList(objects + ObjWoodPlank);
-#ifdef DEBUG_LISTS
-		(void) printf("WoodPlank drawn quickly\n");
-#endif
-	}
+	glBegin(GL_QUADS);
+	glNormal3f(0, 0, 1);
+	glTexCoord2f(0, 0);
+	glVertex3f(-PlankWidth, -PlankHeight, PlankThickness);
+	glTexCoord2f(1, 0);
+	glVertex3f(PlankWidth, -PlankHeight, PlankThickness);
+	glTexCoord2f(1, 1);
+	glVertex3f(PlankWidth, PlankHeight, PlankThickness);
+	glTexCoord2f(0, 1);
+	glVertex3f(-PlankWidth, PlankHeight, PlankThickness);
+	glNormal3f(0, 0, -1);
+	glTexCoord2f(0, 0);
+	glVertex3f(-PlankWidth, PlankHeight, -PlankThickness);
+	glTexCoord2f(1, 0);
+	glVertex3f(PlankWidth, PlankHeight, -PlankThickness);
+	glTexCoord2f(1, 1);
+	glVertex3f(PlankWidth, -PlankHeight, -PlankThickness);
+	glTexCoord2f(0, 1);
+	glVertex3f(-PlankWidth, -PlankHeight, -PlankThickness);
+	glNormal3f(0, 1, 0);
+	glTexCoord2f(0, 0);
+	glVertex3f(-PlankWidth, PlankHeight, PlankThickness);
+	glTexCoord2f(1, 0);
+	glVertex3f(PlankWidth, PlankHeight, PlankThickness);
+	glTexCoord2f(1, 1);
+	glVertex3f(PlankWidth, PlankHeight, -PlankThickness);
+	glTexCoord2f(0, 1);
+	glVertex3f(-PlankWidth, PlankHeight, -PlankThickness);
+	glNormal3f(0, -1, 0);
+	glTexCoord2f(0, 0);
+	glVertex3f(-PlankWidth, -PlankHeight, -PlankThickness);
+	glTexCoord2f(1, 0);
+	glVertex3f(PlankWidth, -PlankHeight, -PlankThickness);
+	glTexCoord2f(1, 1);
+	glVertex3f(PlankWidth, -PlankHeight, PlankThickness);
+	glTexCoord2f(0, 1);
+	glVertex3f(-PlankWidth, -PlankHeight, PlankThickness);
+	glNormal3f(1, 0, 0);
+	glTexCoord2f(0, 0);
+	glVertex3f(PlankWidth, -PlankHeight, PlankThickness);
+	glTexCoord2f(1, 0);
+	glVertex3f(PlankWidth, -PlankHeight, -PlankThickness);
+	glTexCoord2f(1, 1);
+	glVertex3f(PlankWidth, PlankHeight, -PlankThickness);
+	glTexCoord2f(0, 1);
+	glVertex3f(PlankWidth, PlankHeight, PlankThickness);
+	glNormal3f(-1, 0, 0);
+	glTexCoord2f(0, 0);
+	glVertex3f(-PlankWidth, PlankHeight, PlankThickness);
+	glTexCoord2f(1, 0);
+	glVertex3f(-PlankWidth, PlankHeight, -PlankThickness);
+	glTexCoord2f(1, 1);
+	glVertex3f(-PlankWidth, -PlankHeight, -PlankThickness);
+	glTexCoord2f(0, 1);
+	glVertex3f(-PlankWidth, -PlankHeight, PlankThickness);
+	glEnd();
+
 	return True;
 }
 
 static Bool
-draw_impossiblecage(cagestruct * cp)
+draw_impossiblecage()
 {
 	glPushMatrix();
 	glRotatef(90, 0, 1, 0);
 	glTranslatef(0.0, PlankHeight - PlankWidth, -PlankThickness - PlankWidth);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 0, 1);
 	glTranslatef(0.0, PlankHeight - PlankWidth, PlankWidth - PlankThickness);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 1, 0);
 	glTranslatef(0.0, PlankWidth - PlankHeight, -PlankThickness - PlankWidth);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glTranslatef(0.0, PlankWidth - PlankHeight, 3 * PlankThickness - PlankWidth);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 0, 1);
 	glTranslatef(0.0, PlankWidth - PlankHeight, PlankWidth - PlankThickness);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glTranslatef(0.0, PlankWidth - PlankHeight, PlankWidth - 3 * PlankThickness);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glTranslatef(0.0, PlankHeight - PlankWidth, 3 * PlankThickness - PlankWidth);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 0, 1);
 	glTranslatef(0.0, PlankHeight - PlankWidth, PlankThickness - PlankWidth);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glTranslatef(0.0, PlankHeight - PlankWidth, PlankWidth - 3 * PlankThickness);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 1, 0);
 	glTranslatef(0.0, PlankHeight - PlankWidth, PlankWidth + PlankThickness);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 0, 1);
 	glTranslatef(0.0, PlankWidth - PlankHeight, PlankThickness - PlankWidth);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(90, 0, 1, 0);
 	glTranslatef(0.0, PlankWidth - PlankHeight, PlankWidth + PlankThickness);
-	if (!draw_woodplank(cp))
+	if (!draw_woodplank())
 		return False;
 	glPopMatrix();
 	return True;
@@ -330,7 +314,6 @@ reshape(ModeInfo * mi, int width, int height)
 	i = width / 512 + 1;
 	glLineWidth(i);
 	glPointSize(i);
-	cp->AreObjectsDefined[ObjWoodPlank] = False;
 }
 
 static void
@@ -386,14 +369,10 @@ release_cage(ModeInfo * mi)
 			cagestruct *cp = &cage[screen];
 
 			if (cp->glx_context) {
-				if (glIsList(objects)) {
-					glDeleteLists(objects, MaxObj);
-					objects = 0;
-				}
 				cp->glx_context = (GLXContext *) NULL;
 			}
 		}
-		(void) free((void *) cage);
+		free(cage);
 		cage = (cagestruct *) NULL;
 	}
 	FreeAllGL(mi);
@@ -416,12 +395,6 @@ init_cage(ModeInfo * mi)
 
 		reshape(mi, MI_WIDTH(mi), MI_HEIGHT(mi));
 		glDrawBuffer(GL_BACK);
-		if (!glIsList(objects))
-			if ((objects = glGenLists(MaxObj)) == 0) {
-				MI_CLEARWINDOW(mi);
-				release_cage(mi);
-				return;
-			}
 		pinit(mi);
 	} else {
 		MI_CLEARWINDOW(mi);
@@ -461,7 +434,7 @@ draw_cage(ModeInfo * mi)
 	glRotatef(cp->step * 100, 0, 0, 1);
 	glRotatef(25 + cos(cp->step * 5) * 6, 1, 0, 0);
 	glRotatef(204.5 - sin(cp->step * 5) * 8, 0, 1, 0);
-	if (!draw_impossiblecage(cp)) {
+	if (!draw_impossiblecage()) {
 		release_cage(mi);
 		return;
 	}
