@@ -2,7 +2,7 @@
 /* lament --- Shows Lemarchand's Box */
 
 #if !defined( lint ) && !defined( SABER )
-static const char sccsid[] = "@(#)lament.c	5.01 2001/03/01 xlockmore";
+static const char sccsid[] = "@(#)lament.c	5.03 2001/11/28 xlockmore";
 
 #endif
 
@@ -17,93 +17,92 @@ static const char sccsid[] = "@(#)lament.c	5.01 2001/03/01 xlockmore";
  * implied warranty.
  *
  * Revisions:
- * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge@mail.dotcom.fr>
+ * 28-Nov-2001: Integrated diffs from xscreensaver-3.34 - E. Lassauge <lassauge AT users.sourceforge.net>
+ * 01-Mar-2001: Added FPS stuff - Eric Lassauge <lassauge AT users.sourceforge.net>
+ * 28-Nov-2000: modified release part to add freeing of GL objects
+ * 21-Nov-2000: use new common xpm_to_ximage function - E.Lassauge
  * 01-Nov-2000: Allocation checks
  * 25-Jul-1998: Written by jwz
  */
 
-/* Animates Lemarchand's Box, the Lament Configuration.
+/* Animates Lemarchand's Box, the Lament Configuration.  By jwz, 25-Jul-98.
 
    TODO:
 
-   *  The "gold" color isn't quite right; it looks more like "yellow" than
-   "gold" to me.
+     *  The "gold" color isn't quite right; it looks more like "yellow" than
+        "gold" to me.
 
-   *  For some reason, the interior surfaces are shinier than the exterior
-   surfaces.  I don't understand why, but this should be remedied.
+     *  For some reason, the interior surfaces are shinier than the exterior
+        surfaces.  I don't understand why, but this should be remedied.
 
-   *  Perhaps use a slightly-bumpy or oily texture for the interior surfaces?
+     *  Perhaps use a slightly-bumpy or oily texture for the interior surfaces?
 
-   *  Some of the edges don't line up perfectly (since the images are not
-   perfectly symetrical.)  Something should be done about this; either
-   making the edges overlap slightly (instead of leaving gaps) or fixing
-   the images so that the edges may be symmetrical.
+     *  Some of the edges don't line up perfectly (since the images are not
+        perfectly symetrical.)  Something should be done about this; either
+        making the edges overlap slightly (instead of leaving gaps) or fixing
+        the images so that the edges may be symmetrical.
 
-   *  I want the gold leaf to seem to be raised up from the surface, but I
-   think this isn't possible with OpenGL.  Supposedly, OpenGL only
-   supports Gouraud shading (interpolating edge normals from face normals,
-   and shading smoothly) but bump-maps only work with Phong shading
-   (computing a normal for each rendered pixel.)
+     *  I want the gold leaf to seem to be raised up from the surface, but I
+        think this isn't possible with OpenGL.  Supposedly, OpenGL only 
+        supports Gouraud shading (interpolating edge normals from face normals,
+        and shading smoothly) but bump-maps only work with Phong shading
+        (computing a normal for each rendered pixel.)
 
-   *  As far as I can tell, OpenGL doesn't do shadows.  As a result, the
-   forward-facing interior walls are drawn bright, not dark.  If it was
-   casting shadows properly, it wouldn't matter so much that the edges
-   don't quite line up, because the lines would be black, and thus not
-   visible.  But the edges don't match up, and so the bright interior
-   faces show through, and that sucks.
+     *  As far as I can tell, OpenGL doesn't do shadows.  As a result, the
+        forward-facing interior walls are drawn bright, not dark.  If it was
+        casting shadows properly, it wouldn't matter so much that the edges
+        don't quite line up, because the lines would be black, and thus not
+        visible.  But the edges don't match up, and so the bright interior
+        faces show through, and that sucks.
 
-   But apparently there are tricky ways around this:
-   http://reality.sgi.com/opengl/tips/rts/
-   I think these techniques require GLUT, however, which isn't
-   (currently) required by any other xscreensaver hacks.
+	But apparently there are tricky ways around this:
+	http://reality.sgi.com/opengl/tips/rts/
+	I think these techniques require GLUT, however, which isn't 
+	(currently) required by any other xscreensaver hacks.
 
-   *  There should be strange lighting effects playing across the surface:
-   electric sparks, or little glittery blobs of light.
-   http://reality.sgi.com/opengl/tips/lensflare/ might provide guidance.
+     *  There should be strange lighting effects playing across the surface:
+        electric sparks, or little glittery blobs of light.  
+        http://reality.sgi.com/opengl/tips/lensflare/ might provide guidance.
 
-   *  Need to add some more modes, to effect the transition from the cube
-   shapes to the "spike" or "leviathan" shapes.  I have extensive notes
-   on how these transformations occur, but unfortunately, due to camera
-   trickery, the transitions require dematerializations which do not
-   preserve object volume.  But I suppose that's allowed, in
-   non-Euclidian or hyperdimensional spaces (since the extra mass could
-   simply be rotated along the axis to which one cannot point.)
+     *  Need to add some more modes, to effect the transition from the cube
+        shapes to the "spike" or "leviathan" shapes.  I have extensive notes
+        on how these transformations occur, but unfortunately, due to camera
+        trickery, the transitions require dematerializations which do not
+        preserve object volume.  But I suppose that's allowed, in
+        non-Euclidian or hyperdimensional spaces (since the extra mass could
+        simply be rotated along the axis to which one cannot point.)
 
-   The other hard thing about this is that the "leviathan" shapes contain
-   a much larger number of facets, and I modelled this whole thing by
-   hand, since I don't have any 3d-object-editing tools that I know how
-   to use (or that look like they would take any less than several months
-   to become even marginally proficient with...)
+        The other hard thing about this is that the "leviathan" shapes contain
+        a much larger number of facets, and I modelled this whole thing by 
+        hand, since I don't have any 3d-object-editing tools that I know how
+        to use (or that look like they would take any less than several months
+        to become even marginally proficient with...)
 
-   *  Perhaps there should be a table top, on which it casts a shadow?
-   And then multiple light sources (for multiple shadows)?
+     *  Perhaps there should be a table top, on which it casts a shadow?
+        And then multiple light sources (for multiple shadows)?
 
-   *  Needs music.  ("Hellraiser Themes" by Coil: TORSO CD161; also
-   duplicated on the "Unnatural History 2" compilation, WORLN M04699.)
+     *  Needs music.  ("Hellraiser Themes" by Coil: TORSO CD161; also
+        duplicated on the "Unnatural History 2" compilation, WORLN M04699.)
 
-   *  I'm not totally happy with the spinning motion; I like the
-   acceleration and deceleration, but it often feels like it's going too
-   fast, or not naturally enough, or something.
-
-   *  However, the motion is better than that used by gears, superquadrics,
-   etc.; so maybe I should make them all share the same motion code.
-
-   * E.Lassauge - 28-Nov-2000:  modified release part to add freeing of GL objects
-
-   * E.Lassauge - 21-Nov-2000:  use new common xpm_to_ximage function
+     *  I'm not totally happy with the spinning motion; I like the
+        acceleration and deceleration, but it often feels like it's going too
+        fast, or not naturally enough, or something.
 
+     *  However, the motion is better than that used by gears, superquadrics,
+        etc.; so maybe I should make them all share the same motion code.
  */
 
 #ifdef VMS
 #include "vms_x_fix.h"
+#include <X11/Intrinsic.h>
 #endif
 
-#include <X11/Intrinsic.h>
-
 #ifdef STANDALONE
+#define MODE_lament
 #define PROGCLASS	"Lament"
 #define HACK_INIT	init_lament
 #define HACK_DRAW	draw_lament
+#define HACK_RESHAPE	reshape_lament
 #define lament_opts	xlockmore_opts
 #define DEFAULTS	"*delay:	10000   \n"	\
 			"*showFps:      False   \n"     \
@@ -152,7 +151,7 @@ ModStruct   lament_description =
 
 #endif
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #include "xpm-ximage.h"
 
 #ifdef STANDALONE
@@ -191,11 +190,16 @@ typedef enum {
 
 } lament_type;
 
-static GLfloat exterior_color[] =
-{0.70, 0.60, 0.00, 1.00};
-static GLfloat interior_color[] =
-{0.25, 0.25, 0.20, 1.00};
-
+static GLfloat exterior_color[] = { 0.33, 0.22, 0.03, 1.00,  /* ambient    */
+                                    0.78, 0.57, 0.11, 1.00,  /* specular   */
+                                    0.99, 0.91, 0.81, 1.00,  /* diffuse    */
+                                   27.80                     /* shininess  */
+                                  };
+static GLfloat interior_color[] = { 0.20, 0.20, 0.15, 1.00,  /* ambient    */
+                                    0.40, 0.40, 0.32, 1.00,  /* specular   */
+                                    0.99, 0.99, 0.81, 1.00,  /* diffuse    */
+                                   50.80                     /* shininess  */
+                                  };
 
 typedef struct {
 	GLXContext *glx_context;
@@ -326,6 +330,15 @@ do_normal(GLfloat x1, GLfloat y1, GLfloat z1,
 /* Shorthand utilities for making faces, with proper normals.
  */
 
+static void
+set_colors (GLfloat *color)
+{
+  glMaterialfv(GL_FRONT, GL_AMBIENT, color+0);
+  glMaterialfv(GL_FRONT, GL_DIFFUSE, color+4);
+  glMaterialfv(GL_FRONT, GL_SPECULAR, color+8);
+  glMaterialfv(GL_FRONT, GL_SHININESS, color+12);
+}
+
 static void
 face3(GLint texture, GLfloat * color, Bool wire,
 		  GLfloat s1, GLfloat t1, GLfloat x1, GLfloat y1, GLfloat z1,
@@ -335,7 +348,7 @@ face3(GLint texture, GLfloat * color, Bool wire,
 #ifdef HAVE_GLBINDTEXTURE
 	glBindTexture(GL_TEXTURE_2D, texture);
 #endif /* HAVE_GLBINDTEXTURE */
-	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
+	set_colors(color);
 	do_normal(x1, y1, z1, x2, y2, z2, x3, y3, z3);
 	glBegin(wire ? GL_LINE_LOOP : GL_TRIANGLES);
 	glTexCoord2f(s1, t1);
@@ -357,7 +370,7 @@ face4(GLint texture, GLfloat * color, Bool wire,
 #ifdef HAVE_GLBINDTEXTURE
 	glBindTexture(GL_TEXTURE_2D, texture);
 #endif /* HAVE_GLBINDTEXTURE */
-	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
+	set_colors(color);
 	do_normal(x1, y1, z1, x2, y2, z2, x3, y3, z3);
 	glBegin(wire ? GL_LINE_LOOP : GL_QUADS);
 	glTexCoord2f(s1, t1);
@@ -382,7 +395,7 @@ face5(GLint texture, GLfloat * color, Bool wire,
 #ifdef HAVE_GLBINDTEXTURE
 	glBindTexture(GL_TEXTURE_2D, texture);
 #endif /* HAVE_GLBINDTEXTURE */
-	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color);
+	set_colors(color);
 	do_normal(x1, y1, z1, x2, y2, z2, x3, y3, z3);
 	glBegin(wire ? GL_LINE_LOOP : GL_POLYGON);
 	glTexCoord2f(s1, t1);
@@ -619,7 +632,7 @@ star(ModeInfo * mi, Bool top, Bool wire)
 #ifdef HAVE_GLBINDTEXTURE
 	glBindTexture(GL_TEXTURE_2D, lc->texids[top ? FACE_U : FACE_D]);
 #endif /* HAVE_GLBINDTEXTURE */
-	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, exterior_color);
+	set_colors(exterior_color);
 
 	i = 1;
 	do_normal(points[i + 0][0], points[i + 0][1], 0,
@@ -641,7 +654,7 @@ star(ModeInfo * mi, Bool top, Bool wire)
 #ifdef HAVE_GLBINDTEXTURE
 	glBindTexture(GL_TEXTURE_2D, 0);
 #endif /* HAVE_GLBINDTEXTURE */
-	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, interior_color);
+	set_colors(interior_color);
 
 /* Range changed from "countof(points) - 3" to avoid Stack array bounds read */
 	i = countof(points) - 9;
@@ -1253,7 +1266,7 @@ taser(ModeInfo * mi, Bool wire)
 #ifdef HAVE_GLBINDTEXTURE
 		glBindTexture(GL_TEXTURE_2D, lc->texids[FACE_E]);
 #endif /* HAVE_GLBINDTEXTURE */
-		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, exterior_color);
+		set_colors(exterior_color);
 
 		do_normal(0, body_face_points[(i * 5) + 0][0], body_face_points[(i * 5) + 0][1],
 			  0, body_face_points[(i * 5) + 1][0], body_face_points[(i * 5) + 1][1],
@@ -1381,7 +1394,7 @@ taser(ModeInfo * mi, Bool wire)
 #ifdef HAVE_GLBINDTEXTURE
 		glBindTexture(GL_TEXTURE_2D, lc->texids[FACE_E]);
 #endif /* HAVE_GLBINDTEXTURE */
-		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, exterior_color);
+		set_colors(exterior_color);
 
 		do_normal(
 				 0, lifter_face_points[(i * 5) + 0][0], lifter_face_points[(i * 5) + 0][1],
@@ -1528,7 +1541,7 @@ taser(ModeInfo * mi, Bool wire)
 #ifdef HAVE_GLBINDTEXTURE
 		glBindTexture(GL_TEXTURE_2D, lc->texids[FACE_E]);
 #endif /* HAVE_GLBINDTEXTURE */
-		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, exterior_color);
+		set_colors(exterior_color);
 
 		do_normal(
 				 0, slider_face_points[(i * 5) + 0][0], slider_face_points[(i * 5) + 0][1],
@@ -2155,7 +2168,7 @@ rotate(GLfloat * pos, GLfloat * v, GLfloat * dv, GLfloat max_v, Bool verbose)
  */
 
 static void
-reshape(int width, int height)
+reshape_lament(int width, int height)
 {
 	int         target_size = 180;
 	int         win_size = (width > height ? height : width);
@@ -2225,44 +2238,33 @@ gl_init(ModeInfo * mi)
 		do_texture = False;
 
 	if (!wire) {
-		static GLfloat pos0[] =
-		{-4.0, 2.0, 5.0, 1.0};
-		static GLfloat pos1[] =
-		{12.0, 5.0, 1.0, 1.0};
-		static GLfloat local[] =
-		{0.0};
-		static GLfloat ambient[] =
-		{0.3, 0.3, 0.3, 1.0};
-		static GLfloat spec[] =
-		{1.0, 1.0, 1.0, 1.0};
-		static GLfloat shine[] =
-		{100.0};
+		static GLfloat pos0[] = {-4.0, 2.0, 5.0, 1.0};
+		static GLfloat pos1[] = { 6.0, -1.0, 3.0, 1.0};
+		static GLfloat amb0[] = {0.7, 0.7, 0.7, 1.0};
+/*              static GLfloat amb1[] = { 0.7, 0.0, 0.0, 1.0 }; */
+                static GLfloat dif0[]  = { 1.0, 1.0, 1.0, 1.0 };
+                static GLfloat dif1[]  = { 0.3, 0.1, 0.1, 1.0 };
 
 		glLightfv(GL_LIGHT0, GL_POSITION, pos0);
 		glLightfv(GL_LIGHT1, GL_POSITION, pos1);
 
-		glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
-		glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
+		glLightfv(GL_LIGHT0, GL_AMBIENT, amb0);
+/*              glLightfv(GL_LIGHT1, GL_AMBIENT, amb1); */
+		glLightfv(GL_LIGHT0, GL_DIFFUSE, dif0);
+		glLightfv(GL_LIGHT1, GL_DIFFUSE, dif1);
 
-		glLightfv(GL_LIGHT0, GL_SPECULAR, spec);
-		glLightfv(GL_LIGHT1, GL_SPECULAR, spec);
-
-		glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local);
-		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, exterior_color);
-		glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
-		glMaterialfv(GL_FRONT, GL_SHININESS, shine);
+		set_colors(exterior_color);
 
 		glEnable(GL_LIGHTING);
 		glEnable(GL_LIGHT0);
-		glEnable(GL_LIGHT1);
-		glDisable(GL_LIGHT1);
+/*		glEnable(GL_LIGHT1); */
 
 		glEnable(GL_DEPTH_TEST);
 		glEnable(GL_TEXTURE_2D);
 		glEnable(GL_NORMALIZE);
 		glEnable(GL_CULL_FACE);
 	}
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	if (do_texture) {
 #ifdef HAVE_GLBINDTEXTURE
 		int         i;
@@ -2400,7 +2402,7 @@ init_lament(ModeInfo * mi)
 	lc->anim_pause = 300 + (LRAND() % 100);
 
 	if ((lc->glx_context = init_GL(mi)) != NULL) {
-		reshape(MI_WIDTH(mi), MI_HEIGHT(mi));
+		reshape_lament(MI_WIDTH(mi), MI_HEIGHT(mi));
 		if (!gl_init(mi)) {
 			MI_CLEARWINDOW(mi);
 		}
@@ -2425,11 +2427,14 @@ draw_lament(ModeInfo * mi)
 	if (!lc->glx_context)
 		return;
 
+	MI_IS_DRAWN(mi) = True;
+
 	glDrawBuffer(GL_BACK);
 
 	glXMakeCurrent(display, window, *(lc->glx_context));
 	draw(mi);
         if (MI_IS_FPS(mi)) do_fps (mi);
+
 	glFinish();
 	glXSwapBuffers(display, window);
 
@@ -2445,7 +2450,7 @@ draw_lament(ModeInfo * mi)
 
 	if (++tick > 500) {
 		tick = 0;
-		reshape(MI_WIDTH(mi), MI_HEIGHT(mi));
+		reshape_lament(MI_WIDTH(mi), MI_HEIGHT(mi));
 	}
 }
 
@@ -2471,7 +2476,7 @@ release_lament(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_lament(MI_DISPLAY(mi), &lcs[screen]);
-		(void) free((void *) lcs);
+		free(lcs);
 		lcs = (lament_configuration *) NULL;
 	}
 	FreeAllGL(mi);
