@@ -45,26 +45,24 @@ static const char sccsid[] = "@(#)morph3d.c	5.01 2001/03/01 xlockmore";
  * Marcelo F. Vianna (Feb-13-1997)
  *
  * Revision History:
- * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge@mail.dotcom.fr>
+ * 05-Apr-2002: Removed all gllist uses (fix some bug with nvidia driver)
+ * 01-Mar-2001: Added FPS stuff E.Lassauge <lassauge AT users.sourceforge.net>
  * 27-Jul-1997: Speed ups by Marcelo F. Vianna.
  * 08-May-1997: Speed ups by Marcelo F. Vianna.
  *
  */
 
-/*-
- * PURIFY 3.0a on SunOS4 reports an unitialized memory read on each of
- * the glCallList() functions below when using MesaGL 2.1.  This has
- * been fixed in MesaGL 2.2 and later releases.
- */
-
+#ifdef VMS
 /*-
  * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
  * otherwise caddr_t is not defined correctly
  */
 
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+#define MODE_moebius
 #define PROGCLASS "Morph3d"
 #define HACK_INIT init_morph3d
 #define HACK_DRAW draw_morph3d
@@ -343,237 +341,177 @@ static morph3dstruct *morph3d = (morph3dstruct *) NULL;
 static void
 draw_tetra(ModeInfo * mi)
 {
-	GLuint      list;
-
 	morph3dstruct *mp = &morph3d[MI_SCREEN(mi)];
 
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[0]);
 
-	if ((list = glGenLists(1)) == 0)
-		return;
-	glNewList(list, GL_COMPILE_AND_EXECUTE);
-	if (glGetError() != GL_NO_ERROR) {
-		glDeleteLists(list, 1);
-		return;
-	}
 	TRIANGLE(2, mp->seno, mp->edgedivisions, 0.5 / SQRT6, mp->VisibleSpikes);
-	glEndList();
 
 	glPushMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-tetraangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[1]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 0.5 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + tetraangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[2]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 0.5 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + tetraangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[3]);
-	glCallList(list);
-
-	glDeleteLists(list, 1);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 0.5 / SQRT6, mp->VisibleSpikes);
 }
 
 static void
 draw_cube(ModeInfo * mi)
 {
-	GLuint      list;
-
 	morph3dstruct *mp = &morph3d[MI_SCREEN(mi)];
 
-
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[0]);
 
-	if ((list = glGenLists(1)) == 0)
-		return;
-	glNewList(list, GL_COMPILE_AND_EXECUTE);
-	if (glGetError() != GL_NO_ERROR) {
-		glDeleteLists(list, 1);
-		return;
-	}
 	SQUARE(2, mp->seno, mp->edgedivisions, 0.5, mp->VisibleSpikes)
-		glEndList();
 
 	glRotatef(cubeangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[1]);
-	glCallList(list);
+	SQUARE(2, mp->seno, mp->edgedivisions, 0.5, mp->VisibleSpikes)
 	glRotatef(cubeangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[2]);
-	glCallList(list);
+	SQUARE(2, mp->seno, mp->edgedivisions, 0.5, mp->VisibleSpikes)
 	glRotatef(cubeangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[3]);
-	glCallList(list);
+	SQUARE(2, mp->seno, mp->edgedivisions, 0.5, mp->VisibleSpikes)
 	glRotatef(cubeangle, 0, 1, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[4]);
-	glCallList(list);
+	SQUARE(2, mp->seno, mp->edgedivisions, 0.5, mp->VisibleSpikes)
 	glRotatef(2 * cubeangle, 0, 1, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[5]);
-	glCallList(list);
-
-	glDeleteLists(list, 1);
+	SQUARE(2, mp->seno, mp->edgedivisions, 0.5, mp->VisibleSpikes)
 }
 
 static void
 draw_octa(ModeInfo * mi)
 {
-	GLuint      list;
-
 	morph3dstruct *mp = &morph3d[MI_SCREEN(mi)];
 
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[0]);
-
-	if ((list = glGenLists(1)) == 0)
-		return;
-	glNewList(list, GL_COMPILE_AND_EXECUTE);
-	if (glGetError() != GL_NO_ERROR) {
-		glDeleteLists(list, 1);
-		return;
-	}
 	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
-	glEndList();
 
 	glPushMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-180 + octaangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[1]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-octaangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[2]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-octaangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[3]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[4]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-180 + octaangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[5]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-octaangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[6]);
-	glCallList(list);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-octaangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[7]);
-	glCallList(list);
-
-	glDeleteLists(list, 1);
+	TRIANGLE(2, mp->seno, mp->edgedivisions, 1 / SQRT6, mp->VisibleSpikes);
 }
 
 static void
 draw_dodeca(ModeInfo * mi)
 {
-	GLuint      list;
-
 	morph3dstruct *mp = &morph3d[MI_SCREEN(mi)];
 
 #define TAU ((SQRT5+1)/2)
 
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[0]);
 
-	if ((list = glGenLists(1)) == 0)
-		return;
-	glNewList(list, GL_COMPILE_AND_EXECUTE);
-	if (glGetError() != GL_NO_ERROR) {
-		glDeleteLists(list, 1);
-		return;
-	}
 	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
-	glEndList();
 
 	glPushMatrix();
 	glRotatef(180, 0, 0, 1);
 	glPushMatrix();
 	glRotatef(-dodecaangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[1]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(-dodecaangle, cos72, sin72, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[2]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(-dodecaangle, cos72, -sin72, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[3]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(dodecaangle, cos36, -sin36, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[4]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(dodecaangle, cos36, sin36, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[5]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[6]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glRotatef(180, 0, 0, 1);
 	glPushMatrix();
 	glRotatef(-dodecaangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[7]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(-dodecaangle, cos72, sin72, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[8]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(-dodecaangle, cos72, -sin72, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[9]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(dodecaangle, cos36, -sin36, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[10]);
-	glCallList(list);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(dodecaangle, cos36, sin36, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[11]);
-	glCallList(list);
-
-	glDeleteLists(list, 1);
+	PENTAGON(1, mp->seno, mp->edgedivisions, sqr(TAU) * sqrt((TAU + 2) / 5) / 2, mp->VisibleSpikes);
 }
 
 static void
 draw_icosa(ModeInfo * mi)
 {
-	GLuint      list;
-
 	morph3dstruct *mp = &morph3d[MI_SCREEN(mi)];
 
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[0]);
 
-	if ((list = glGenLists(1)) == 0)
-		return;
-	glNewList(list, GL_COMPILE_AND_EXECUTE);
-	if (glGetError() != GL_NO_ERROR) {
-		glDeleteLists(list, 1);
-		return;
-	}
 	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
-	glEndList();
 
 	glPushMatrix();
 
@@ -581,100 +519,98 @@ draw_icosa(ModeInfo * mi)
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-icoangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[1]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[2]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[3]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[4]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[5]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-icoangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[6]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[7]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[8]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-icoangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[9]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[10]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-icoangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[11]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[12]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[13]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[14]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[15]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-icoangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[16]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[17]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPushMatrix();
 	glRotatef(180, 0, 1, 0);
 	glRotatef(-180 + icoangle, 0.5, -SQRT3 / 2, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[18]);
-	glCallList(list);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 	glPopMatrix();
 	glRotatef(180, 0, 0, 1);
 	glRotatef(-icoangle, 1, 0, 0);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mp->MaterialColor[19]);
-	glCallList(list);
-
-	glDeleteLists(list, 1);
+	TRIANGLE(1.5, mp->seno, mp->edgedivisions, (3 * SQRT3 + SQRT15) / 12, mp->VisibleSpikes);
 }
 
 static void
@@ -910,7 +846,7 @@ void
 release_morph3d(ModeInfo * mi)
 {
 	if (morph3d != NULL) {
-		(void) free((void *) morph3d);
+		free(morph3d);
 		morph3d = (morph3dstruct *) NULL;
 	}
 	FreeAllGL(mi);
