@@ -44,14 +44,16 @@ static const char sccsid[] = "@(#)pipes.c	5.00 2000/11/01 xlockmore";
  * 29-Apr-97: Efficiency speed-ups by Marcelo F. Vianna
  */
 
+#ifdef VMS
 /*-
  * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
  * otherwise caddr_t is not defined correctly
  */
-
 #include <X11/Intrinsic.h>
+#endif
 
 #ifdef STANDALONE
+#define MODE_pipes
 #define PROGCLASS "Pipes"
 #define HACK_INIT init_pipes
 #define HACK_DRAW draw_pipes
@@ -678,7 +680,7 @@ release_pipes(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_factory(MI_DISPLAY(mi), &pipes[screen]);
-		(void) free((void *) pipes);
+		free(pipes);
 		pipes = (pipesstruct *) NULL;
 	}
 	FreeAllGL(mi);
