@@ -23,7 +23,7 @@ static const char sccsid[] = "@(#)sballs.c	5.02 2001/03/10 xlockmore";
  * Mustata Bogdan (LoneRunner) <lonerunner@planetquake.com>
  * and can be found at http://www.cfxweb.net/lonerunner/
  *
- * Eric Lassauge  (November-07-2000) <lassauge@mail.dotcom.fr>
+ * Eric Lassauge  (November-07-2000) <lassauge AT users.sourceforge.net>
  * 				    http://lassauge.free.fr/linux.html
  *
  * REVISION HISTORY:
@@ -41,12 +41,13 @@ static const char sccsid[] = "@(#)sballs.c	5.02 2001/03/10 xlockmore";
  */
 
 #ifdef STANDALONE	/* xscreensaver mode */
+#define MODE_sballs
 #define PROGCLASS 	"Sballs"
 #define HACK_INIT 	init_sballs
 #define HACK_DRAW 	draw_sballs
 #define HACK_RESHAPE 	reshape_sballs
 #define sballs_opts 	xlockmore_opts
-#define DEFAULTS 	"*delay: 	10000 \n" \
+#define DEFAULTS 	"*delay: 	40000 \n" \
  			"*size: 	    0 \n" \
 			"*cycles:	   10 \n" \
  			"*object: 	    0 \n" \
@@ -55,7 +56,6 @@ static const char sccsid[] = "@(#)sballs.c	5.02 2001/03/10 xlockmore";
  			"*wireframe:  	False \n" \
  			"*texture:  	True  \n"
 
-#define MODE_sballs
 #include "xlockmore.h"		/* from the xscreensaver distribution */
 #else				/* !STANDALONE */
 #include "xlock.h"		/* from the xlockmore distribution */
@@ -72,10 +72,8 @@ static const char sccsid[] = "@(#)sballs.c	5.02 2001/03/10 xlockmore";
 #include <GL/glx.h>
 #include <GL/glu.h>
 
-#if defined( USE_XPM ) || defined( USE_XPMINC ) || defined( HAVE_XPM )
-/* USE_XPM & USE_XPMINC in xlock mode ; HAVE_XPM in xscreensaver mode */
+#ifdef HAVE_XPM
 #include "xpm-ximage.h"
-#define I_HAVE_XPM
 
 #ifdef STANDALONE
 #include "../images/face.xpm"
@@ -130,7 +128,7 @@ ModStruct sballs_description =
     /*
     delay,count,cycles,size,ncolors,sat
      */
-    10000, 0, 10, 400, 64, 1.0, "",
+    40000, 0, 10, 0, 64, 1.0, "",
     "balls spinning like crazy in GL", 0, NULL
 };
 #endif /* USE_MODULES */
@@ -416,7 +414,7 @@ static void inittextures(ModeInfo * mi)
 {
     sballsstruct *sb = &sballs[MI_SCREEN(mi)];
 
-#if defined( I_HAVE_XPM )
+#ifdef HAVE_XPM
     if (do_texture) {
 
 	glGenTextures(1, &sb->backid);
@@ -484,11 +482,11 @@ static void inittextures(ModeInfo * mi)
         sb->faceid = 0;       /* default textures */
         sb->backid = 0;
     }
-#else /* !I_HAVE_XPM */
+#else /* !HAVE_XPM */
   do_texture = False;
   sb->faceid = 0;       /* default textures */
   sb->backid = 0;
-#endif /* !I_HAVE_XPM */
+#endif /* !HAVE_XPM */
 }
 
 static void drawSphere(ModeInfo * mi,int sphere_num)
@@ -814,7 +812,7 @@ void release_sballs(ModeInfo * mi)
 		XDestroyImage(sb->ftexture);
 	    }
 	}
-	(void) free((void *) sballs);
+	free(sballs);
 	sballs = (sballsstruct *) NULL;
     }
     FreeAllGL(mi);
