@@ -16,7 +16,7 @@ static const char sccsid[] = "@(#)xpm-image.c       5.0 00/11/14 xlockmore";
  * implied warranty.
  *
  * Version extracted from xscreensaver 3.26 for xlockmore
- * by Eric Lassauge <lassauge@mail.dotcom.fr> http://lassauge.free.fr/linux.html
+ * by Eric Lassauge <lassauge AT users.sourceforge.net> http://lassauge.free.fr/linux.html
  * Parts of the code are from Jamie, the rest dealing with alpha channel is
  * from me.
  */
@@ -31,12 +31,8 @@ static const char sccsid[] = "@(#)xpm-image.c       5.0 00/11/14 xlockmore";
 
 #include <X11/Intrinsic.h>
 
-#if defined( USE_XPM ) || defined( USE_XPMINC ) /* whole file */
-#if USE_XPMINC
-#include <xpm.h>
-#else
-#include <X11/xpm.h>            /* Normal spot */
-#endif /* USE_XPMINC */
+#ifdef HAVE_XPM
+#include <X11/xpm.h>
 
 #include <X11/Xutil.h>
 
@@ -129,7 +125,7 @@ xpm_to_ximage(Display *dpy, Visual *visual, Colormap cmap, char **xpm_data)
 	(void) fprintf(stderr,"Unparsable color: %s\n",
 		xpm_image.colorTable[i].c_color);
         /* free everything up to now */
-        (void) free((void *) ximage->data);
+        free(ximage->data);
         XDestroyImage(ximage);
         return None;
     }
@@ -183,7 +179,7 @@ xpm_to_ximage(Display *dpy, Visual *visual, Colormap cmap, char **xpm_data)
   return ximage;
 }
 
-#else  /* !USE_XPM */
+#else  /* !HAVE_XPM */
 
 
 XImage *
@@ -193,4 +189,4 @@ xpm_to_ximage(Display *dpy, Visual *visual, Colormap cmap, char **xpm_data)
   return None;
 }
 
-#endif /* !USE_XPM */
+#endif /* !HAVE_XPM */
