@@ -32,6 +32,7 @@ static const char sccsid[] = "@(#)image.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_image
 #define PROGCLASS "Image"
 #define HACK_INIT init_image
 #define HACK_DRAW draw_image
@@ -68,7 +69,7 @@ ModStruct   image_description =
 
 #include "image.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define IMAGE_NAME	image_name
 #include "image.xpm"
 #define DEFAULT_XPM 1
@@ -114,7 +115,7 @@ static void
 free_image(Display * display, imagestruct * ip)
 {
 	if (ip->icons != NULL) {
-		(void) free((void *) ip->icons);
+		free(ip->icons);
 		ip->icons = (imagetype *) NULL;
 	}
 	free_stuff(display, ip);
@@ -133,7 +134,7 @@ init_stuff(ModeInfo * mi)
 
 	if (ip->logo == None) {
 		getImage(mi, &ip->logo, IMAGE_WIDTH, IMAGE_HEIGHT, IMAGE_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 			 DEFAULT_XPM, IMAGE_NAME,
 #endif
 			 &ip->graphics_format, &ip->cmap, &ip->black);
@@ -224,7 +225,7 @@ init_image(ModeInfo * mi)
 	if (ip->iconcount > ip->ncols * ip->nrows)
 		ip->iconcount = ip->ncols * ip->nrows;
 	if (ip->icons != NULL)
-		(void) free((void *) ip->icons);
+		free(ip->icons);
 	if ((ip->icons = (imagetype *) malloc(ip->iconcount *
 			sizeof (imagetype))) == NULL) {
 		free_image(MI_DISPLAY(mi), ip);
@@ -273,7 +274,7 @@ release_image(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_image(MI_DISPLAY(mi), &ims[screen]);
-		(void) free((void *) ims);
+		free(ims);
 		ims = (imagestruct *) NULL;
 	}
 }
@@ -281,7 +282,7 @@ release_image(ModeInfo * mi)
 void
 refresh_image(ModeInfo * mi)
 {
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	imagestruct *ip;
 
 	if (ims == NULL)
