@@ -43,6 +43,7 @@ static const char sccsid[] = "@(#)julia.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_julia
 #define PROGCLASS "Julia"
 #define HACK_INIT init_julia
 #define HACK_DRAW draw_julia
@@ -192,8 +193,8 @@ free_julia(Display *display, juliastruct *jp)
 
 		for (buffer = 0; buffer < jp->nbuffers; buffer++)
 			if (jp->pointBuffer[buffer] != NULL)
-				(void) free((void *) jp->pointBuffer[buffer]);
-		(void) free((void *) jp->pointBuffer);
+				free(jp->pointBuffer[buffer]);
+		free(jp->pointBuffer);
 		jp->pointBuffer = (XPoint **) NULL;
 	}
 	if (jp->stippledGC != None) {
@@ -231,6 +232,8 @@ init_julia(ModeInfo * mi)
 	jp->depth = MI_COUNT(mi);
 	if (jp->depth > 10)
 		jp->depth = 10;
+	if (jp->depth < 7)
+		jp->depth = 7;
 
 	if (trackmouse && !jp->cursor) {	/* Create an invisible cursor */
 		Pixmap      bit;
@@ -445,7 +448,7 @@ release_julia(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_julia(MI_DISPLAY(mi), &julias[screen]);
-		(void) free((void *) julias);
+		free(julias);
 		julias = (juliastruct *) NULL;
 	}
 }
