@@ -52,6 +52,7 @@ static const char sccsid[] = "@(#)kumppa.c	5.00 2000/11/01 xlockmore";
 #endif
 
 #ifdef STANDALONE
+#define MODE_kumppa
 #define PROGCLASS "kumppa"
 #define HACK_INIT init_kumppa
 #define HACK_DRAW draw_kumppa
@@ -318,31 +319,31 @@ free_kumppa(Display *display, kumppastruct *s)
 		}
 	}
 	if (s->acosinus != NULL) {
-		(void) free((void *) s->acosinus);
+		free(s->acosinus);
 		s->acosinus = (float *) NULL;
 	}
 	if (s->Xrotations != NULL) {
-		(void) free((void *) s->Xrotations);
+		free(s->Xrotations);
 		s->Xrotations = (int *) NULL;
 	}
 	if (s->Yrotations != NULL) {
-		(void) free((void *) s->Yrotations);
+		free(s->Yrotations);
 		s->Yrotations = (int *) NULL;
 	}
 	if (s->Xrottable != NULL) {
-		(void) free((void *) s->Xrottable);
+		free(s->Xrottable);
 		s->Xrottable = (int *) NULL;
 	}
 	if (s->Yrottable != NULL) {
-		(void) free((void *) s->Yrottable);
+		free(s->Yrottable);
 		s->Yrottable = (int *) NULL;
 	}
 	if (s->rotateX != NULL) {
-		(void) free((void *) s->rotateX);
+		free(s->rotateX);
 		s->rotateX = (int *) NULL;
 	}
 	if (s->rotateY != NULL) {
-		(void) free((void *) s->rotateY);
+		free(s->rotateY);
 		s->rotateY = (int *) NULL;
 	}
 }
@@ -364,13 +365,13 @@ make_rots(ModeInfo * mi, double xspeed, double yspeed)
 	iy = (double) (s->midy + 1) / (double) (s->rotsizeY);
 
 	if (s->Xrotations != NULL)
-		(void) free((void *) s->Xrotations);
+		free(s->Xrotations);
 	if (s->Yrotations != NULL)
-		(void) free((void *) s->Yrotations);
+		free(s->Yrotations);
 	if (s->Xrottable != NULL)
-		(void) free((void *) s->Xrottable);
+		free(s->Xrottable);
 	if (s->Yrottable != NULL)
-		(void) free((void *) s->Yrottable);
+		free(s->Yrottable);
 
 	if (((s->Xrotations = (int *) calloc((s->midx + 2),
 			sizeof (int))) == NULL) ||
@@ -444,11 +445,11 @@ make_rots(ModeInfo * mi, double xspeed, double yspeed)
 	}
 	s->Xrottable[a] = c;
 	if (s->rotateX)
-		(void) free((void *) s->rotateX);
+		free(s->rotateX);
 	if ((s->rotateX = (int *) calloc((maxi + 2) << 1,
 			sizeof (int))) == NULL) {
 		free_kumppa(MI_DISPLAY(mi), s);
-		(void) free((void *) chks);
+		free(chks);
 		return; 
 	}
 
@@ -510,15 +511,15 @@ make_rots(ModeInfo * mi, double xspeed, double yspeed)
 	}
 	s->Yrottable[a] = c;
 	if (s->rotateY)
-		(void) free((void *) s->rotateY);
+		free(s->rotateY);
 	if ((s->rotateY = (int *) calloc((maxi + 2) << 1,
 			sizeof (int))) == NULL) {
 		free_kumppa(MI_DISPLAY(mi), s);
-		/* (void) free((void *) chks); */
+		/* free(chks); */
 		/* return; */
 	}
 
-	(void) free((void *) chks);
+	free(chks);
 }
 
 #ifndef STANDALONE
@@ -649,7 +650,10 @@ init_kumppa(ModeInfo * mi)
 		XSetGraphicsExposures(display, MI_GC(mi), False);
 		s->c1 = NRAND(MI_NPIXELS(mi));
 	}
-	s->time = MI_CYCLES(mi) + NRAND(MI_CYCLES(mi));
+	if (MI_CYCLES(mi) < 1)
+		s->time = 1;
+	else
+		s->time = MI_CYCLES(mi) + NRAND(MI_CYCLES(mi));
 
 #if 0
 #ifdef HAVE_XDBE_EXTENSION
@@ -791,7 +795,10 @@ draw_kumppa(ModeInfo * mi)
 #endif /* HAVE_XDBE_EXTENSION */
 #endif
 	if (--s->time <= 0) {
-		s->time = MI_CYCLES(mi) + NRAND(MI_CYCLES(mi));
+		if (MI_CYCLES(mi) < 1)
+			s->time = 1;
+		else
+			s->time = MI_CYCLES(mi) + NRAND(MI_CYCLES(mi));
 		s->dir = s->dir * (-1);
 	}
 }
@@ -805,7 +812,7 @@ release_kumppa(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_kumppa(MI_DISPLAY(mi), &kumppas[screen]);
-		(void) free((void *) kumppas);
+		free(kumppas);
 		kumppas = (kumppastruct *) NULL;
 	}
 }
