@@ -37,6 +37,7 @@ static const char sccsid[] = "@(#)lisa.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_lisa
 #define PROGCLASS "Lisa"
 #define HACK_INIT init_lisa
 #define HACK_DRAW draw_lisa
@@ -114,7 +115,7 @@ if (MI_NPIXELS(mi) > 2) { \
 typedef struct lisafunc_struct {
 	double      xcoeff[2], ycoeff[2];
 	int         nx, ny;
-	int         index;
+	int         indx;
 } lisafuncs;
 
 typedef struct lisa_struct {
@@ -176,9 +177,9 @@ free_lisa(lisacons *lc)
 		int    lctr;
 
 		for (lctr = 0; lctr < lc->nlisajous; lctr++) {
-			(void) free((void *) lc->lisajous[lctr].lastpoint);
+			free(lc->lisajous[lctr].lastpoint);
 		}
-		(void) free((void *) lc->lisajous);
+		free(lc->lisajous);
 		lc->lisajous = (lisas *) NULL;
 	}
 }
@@ -326,7 +327,7 @@ drawlisa(ModeInfo * mi, lisas * loop)
 			   MI_GC(mi), np[pctr].x, np[pctr].y);
 #endif
 	}
-	(void) free((void *) lp);
+	free(lp);
 	loop->lastpoint = np;
 	return True;
 }
@@ -345,7 +346,7 @@ initlisa(ModeInfo * mi, lisas * loop)
 	} else
 		loop->color = MI_WHITE_PIXEL(mi);
 	loop->nsteps = MI_CYCLES(mi);
-	if (loop->nsteps == 0)
+	if (loop->nsteps < 1)
 		loop->nsteps = 1;
 	lc->maxcycles = (MAXCYCLES * loop->nsteps) - 1;
 	loop->melting = 0;
@@ -471,7 +472,7 @@ change_lisa(ModeInfo * mi)
 	lc->loopcount = 0;
 	for (lctr = 0; lctr < lc->nlisajous; lctr++) {
 		loop = &lc->lisajous[lctr];
-		loop->function[1] = &Function[(loop->function[0]->index + 1) %
+		loop->function[1] = &Function[(loop->function[0]->indx + 1) %
 					      NUMSTDFUNCS];
 		loop->melting = loop->nsteps - 1;
 		loop->nfuncs = 2;
@@ -540,7 +541,7 @@ release_lisa(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_lisa(&Lisa[screen]);
-		(void) free(Lisa);
+		free(Lisa);
 		Lisa = (lisacons *) NULL;
 	}
 }
