@@ -22,6 +22,7 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
  * other special, indirect and consequential damages.
  *
  * Revision History:
+ * 27-Nov-2001: interactive maze mode Ephraim Yawitz fyawitz@actcom.co.il
  * 01-Nov-2000: Allocation checks
  * 27-Oct-1997: xpm and ras capability added
  * 10-May-1997: Compatible with xscreensaver
@@ -69,6 +70,7 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_maze
 #define PROGCLASS "Maze"
 #define HACK_INIT init_maze
 #define HACK_DRAW draw_maze
@@ -77,7 +79,8 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
  "*cycles: 3000 \n" \
  "*size: -40 \n" \
  "*ncolors: 64 \n" \
- "*bitmap: \n"
+ "*bitmap: \n" \
+ "*trackmouse: False \n"
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -87,8 +90,32 @@ static const char sccsid[] = "@(#)maze.c	5.00 2000/11/01 xlockmore";
 
 #ifdef MODE_maze
 
+#define DEF_TRACKMOUSE  "False"
+static Bool trackmouse;
+
+#ifdef DISABLE_INTERACTIVE
 ModeSpecOpt maze_opts =
 {0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
+#else
+static XrmOptionDescRec opts[] =
+{
+  {(char *) "-trackmouse", (char *) ".maze.trackmouse", XrmoptionNoArg, (caddr_t) "on"},
+  {(char *) "+trackmouse", (char *) ".maze.trackmouse", XrmoptionNoArg, (caddr_t) "off"}
+};
+
+static argtype vars[] =
+{
+  {(caddr_t *) & trackmouse, (char *) "trackmouse", (char *) "TrackMouse", (char *) DEF_TRACKMOUSE, t_Bool}
+};
+
+static OptionStruct desc[] =
+{
+  {(char *) "-/+trackmouse", (char *) "turn on/off the tracking of the mouse"},
+};
+
+ModeSpecOpt maze_opts =
+{sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
+#endif
 
 #ifdef USE_MODULES
 ModStruct   maze_description =
@@ -108,7 +135,7 @@ ModStruct   maze_description =
 
 #include "maze.xbm"
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 #define ICON_NAME  image_name
 #include "maze.xpm"
 #define DEFAULT_XPM 1
@@ -133,12 +160,17 @@ ModStruct   maze_description =
 #define DOOR_OUT_BOTTOM	0x20
 #define DOOR_OUT_LEFT	0x10
 
+#define DIR_UP          0
+#define DIR_RIGHT       1
+#define DIR_DOWN        2
+#define DIR_LEFT        3
+
 #define START_SQUARE	0x2
 #define END_SQUARE	0x1
 
 typedef struct {
-	unsigned char x;
-	unsigned char y;
+	unsigned int x;
+	unsigned int y;
 	char        dir;
 } paths;
 
@@ -151,7 +183,7 @@ typedef struct {
 	int         xs, ys, logo_size_x, logo_size_y;
 	int         solving, current_path, stage;
 	int         cycles;
-	unsigned short *maze;
+	unsigned int *maze;
 	paths      *move_list;
 	paths      *save_path, *path;
 	GC          grayGC;
@@ -277,19 +309,19 @@ static void
 free_path(mazestruct * mp)
 {
 	if (mp->maze) {
-		(void) free((void *) mp->maze);
-		mp->maze = (unsigned short *) NULL;
+		free(mp->maze);
+		mp->maze = (unsigned int *) NULL;
 	}
 	if (mp->move_list) {
-		(void) free((void *) mp->move_list);
+		free(mp->move_list);
 		mp->move_list = (paths *) NULL;
 	}
 	if (mp->save_path) {
-		(void) free((void *) mp->save_path);
+		free(mp->save_path);
 		mp->save_path = (paths *) NULL;
 	}
 	if (mp->path) {
-		(void) free((void *) mp->path);
+		free(mp->path);
 		mp->path = (paths *) NULL;
 	}
 }
@@ -357,8 +389,8 @@ set_maze_sizes(ModeInfo * mi)
 	mp->yb = (mp->height - mp->nrows * mp->ys) / 2;
 	mp->maze_size = mp->ncols * mp->nrows;
 	if (!mp->maze)
-		if ((mp->maze = (unsigned short *) calloc(mp->maze_size,
-				 sizeof (unsigned short))) == NULL) {
+		if ((mp->maze = (unsigned int *) calloc(mp->maze_size,
+				 sizeof (unsigned int))) == NULL) {
 			free_maze(display, mp);
 			return False;
 		}
@@ -712,6 +744,67 @@ draw_maze_border(ModeInfo * mi)
 	draw_solid_square(mi, gc, mp->end_x, mp->end_y, mp->end_dir);
 }				/* end of draw_maze() */
 
+static void
+try_to_move(ModeInfo *mi, int dir)
+{                               /* based on solve_maze */
+	mazestruct *mp = &mazes[MI_SCREEN(mi)];
+	int colliding = mp->maze[mp->path[mp->current_path].x * mp->nrows +
+		mp->path[mp->current_path].y] &
+		(WALL_TOP >> dir); /*  Are we trying to go through a wall? */
+
+	if (!colliding && ((mp->current_path == 0) ||
+	/* We're either on the first path or we're not going backwards */
+		((dir != (unsigned char) (mp->path[mp->current_path - 1].dir +
+			2) % 4))) ) {
+		mp->path[mp->current_path].dir = dir;
+		enter_square(mi, mp->current_path);
+		mp->current_path++;
+#if 0
+		(void) fprintf(stderr,
+	"Calling draw_solid_square with params: x=%d, y= %d, dir=%d\n",
+			(int) (mp->path[mp->current_path].x),
+			(int) (mp->path[mp->current_path].y),
+			(int) (dir));
+#endif
+		/* The following call is supposed to make things look more
+		   intutive, i.e., that the square we think we are on is
+		   filled in.
+                   The direction has to be reversed to prevent it from
+		   drawing squares on top of lines.
+		*/
+		draw_solid_square(mi, mp->backGC,
+			(int) (mp->path[mp->current_path].x),
+			(int) (mp->path[mp->current_path].y),
+			(int) ((dir + 2) % 4));
+		if (mp->maze[mp->path[mp->current_path].x * mp->nrows +
+			  mp->path[mp->current_path].y] & START_SQUARE) {
+			mp->solving = 0;
+			return;
+		}
+	} else {
+		if (!colliding) {
+			draw_solid_square(mi, mp->grayGC,
+				(int) (mp->path[mp->current_path].x),
+				(int) (mp->path[mp->current_path].y),
+				(int) (mp->path[mp->current_path - 1].dir +
+				2) % 4);
+			mp->path[mp->current_path].dir = 4;
+			mp->current_path--;
+			draw_solid_square(mi, mp->grayGC,
+				(int) (mp->path[mp->current_path].x),
+				(int) (mp->path[mp->current_path].y),
+				(int) (mp->path[mp->current_path].dir));
+		}
+#if 0
+		else {
+			/* Beep if we can't go in that direction */
+			(void) fprintf(stdout, "\a");
+			(void) fflush(stdout);
+		}
+#endif
+	}
+}         /* end of try_to_move() */
+
 static void
 solve_maze(ModeInfo * mi)
 {				/* solve it with graphical feedback */
@@ -733,6 +826,12 @@ solve_maze(ModeInfo * mi)
 		mp->solving = 1;
 	}
 	if (++mp->path[mp->current_path].dir >= 4) {
+		if (mp->current_path == 0) {
+			/* this can happen if a person backs up from going
+			   the right way and then autosolves */
+			mp->path[mp->current_path].dir = -1;
+			return;
+		}
 		/* This draw is to fill in the dead ends,
 		   it ends up drawing more gray boxes then it needs to. */
 		draw_solid_square(mi, mp->grayGC,
@@ -754,6 +853,11 @@ solve_maze(ModeInfo * mi)
 				       2) % 4)))) {
 		enter_square(mi, mp->current_path);
 		mp->current_path++;
+		/* This next call is to make the appearance more 'intuitive' */
+		draw_solid_square(mi, mp->backGC,
+                                  (int) (mp->path[mp->current_path].x),
+                                  (int) (mp->path[mp->current_path].y),
+                         (int) (mp->path[mp->current_path - 1].dir + 2) % 4);
 		if (mp->maze[mp->path[mp->current_path].x * mp->nrows +
 			     mp->path[mp->current_path].y] & START_SQUARE) {
 			mp->solving = 0;
@@ -762,6 +866,38 @@ solve_maze(ModeInfo * mi)
 	}
 }				/* end of solve_maze() */
 
+static void
+mouse_maze(ModeInfo * mi)
+{	/* solve it with graphical feedback */
+	mazestruct *mp = &mazes[MI_SCREEN(mi)];
+	Window      r, c;
+	int         dx, dy, cx, cy;
+	unsigned int m;
+
+	(void) XQueryPointer(MI_DISPLAY(mi), MI_WINDOW(mi),
+	  &r, &c, &dx, &dy, &cx, &cy, &m);
+	dx = cx - (mp->path[mp->current_path].x * mp->xs + mp->xb) - mp->xs / 2;
+	dy = cy - (mp->path[mp->current_path].y * mp->ys + mp->yb) - mp->ys / 2;
+	if (cx <= mp->xb || cy <= mp->yb || cx > mp->ncols * mp->xs + mp->xb ||
+			cy > mp->nrows * mp->ys + mp->yb) {
+		solve_maze(mi);
+		return;
+	}
+	if (2 * abs(dx) / (mp->xs + 1) > 2 * abs(dy) / mp->ys) {
+		if (dx > 0) {
+			try_to_move(mi, DIR_RIGHT);
+		} else {
+			try_to_move(mi, DIR_LEFT);
+		}
+	} else if (2 * abs(dx) / mp->xs < 2 * abs(dy) / (mp->ys + 1)) {
+		if (dy > 0) {
+			try_to_move(mi, DIR_DOWN);
+		} else {
+			try_to_move(mi, DIR_UP);
+		}
+	}
+}
+
 static Bool
 init_stuff(ModeInfo * mi)
 {
@@ -771,7 +907,7 @@ init_stuff(ModeInfo * mi)
 
 	if (mp->logo == None) {
 		getImage(mi, &mp->logo, ICON_WIDTH, ICON_HEIGHT, ICON_BITS,
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 			 DEFAULT_XPM, ICON_NAME,
 #endif
 			 &mp->graphics_format, &mp->cmap, &mp->black);
@@ -858,7 +994,10 @@ draw_maze(ModeInfo * mi)
 	if (mp->graypix == None)
 		return;
 	if (mp->solving) {
-		solve_maze(mi);
+		if (trackmouse)
+			mouse_maze(mi);
+		else 
+			solve_maze(mi);
 		return;
 	}
 	switch (mp->stage) {
@@ -880,11 +1019,18 @@ draw_maze(ModeInfo * mi)
 			mp->stage++;
 			break;
 		case 3:
-			if (++mp->time > mp->cycles / 4)
+			if (++mp->time > mp->cycles / (4 * (1 + trackmouse)))
 				mp->stage++;
 			break;
 		case 4:
-			solve_maze(mi);
+			if (trackmouse) {
+				/* Initialize paths and draw a square or two */
+				int i;
+				for (i = 0; i < 4; i++)
+					solve_maze(mi);
+			} else {
+				solve_maze(mi);
+			}
 			mp->stage++;
 			break;
 		case 5:
@@ -902,7 +1048,7 @@ release_maze(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_maze(MI_DISPLAY(mi), &mazes[screen]);
-		(void) free((void *) mazes);
+		free(mazes);
 		mazes = (mazestruct *) NULL;
 	}
 }
@@ -918,7 +1064,7 @@ refresh_maze(ModeInfo * mi)
 	if (mp->graypix == None)
 		return;
 
-#if defined( USE_XPM ) || defined( USE_XPMINC )
+#ifdef HAVE_XPM
 	if (mp->graphics_format >= IS_XPM) {
 		/* This is needed when another program changes the colormap. */
 		free_maze(MI_DISPLAY(mi), mp);
