@@ -62,8 +62,6 @@ static XrmOptionDescRec opts[] =
 	{(char *) "-duration", (char *) ".random.duration", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-modelist", (char *) ".random.modelist", XrmoptionSepArg, (caddr_t) NULL},
 	{(char *) "-sequential", (char *) ".random.sequential", XrmoptionNoArg, (caddr_t) "on"},
-      {(char *) "+sequential", (char *) ".random.sequential", XrmoptionNoArg, (caddr_t) "off"},
-	{(char *) "-sequential", (char *) ".random.sequential", XrmoptionNoArg, (caddr_t) "on"},
       {(char *) "+sequential", (char *) ".random.sequential", XrmoptionNoArg, (caddr_t) "off"},
 	{(char *) "-fullrandom", (char *) ".random.fullrandom", XrmoptionNoArg, (caddr_t) "on"},
 	{(char *) "+fullrandom", (char *) ".random.fullrandom", XrmoptionNoArg, (caddr_t) "off"}
@@ -141,6 +139,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_atlantis
 	"atlantis",
 #endif
+#ifdef MODE_atunnels
+	"atunnels",
+#endif
 #ifdef MODE_bubble3d
 	"bubble3d",
 #endif
@@ -171,6 +172,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_morph3d
 	"morph3d",
 #endif
+#ifdef MODE_noof
+	"noof",
+#endif
 #ifdef MODE_pipes
 	"pipes",
 #endif
@@ -196,7 +200,10 @@ static char gl_modes[][MAXMODECHARS] =
 	"superquadrics",
 #endif
 #ifdef MODE_text3d
-	"text3d"
+	"text3d",
+#endif
+#ifdef MODE_text3d2
+	"text3d2",
 #endif
 };
 
@@ -240,6 +247,9 @@ static char write_modes[][MAXMODECHARS] =
 #ifdef MODE_crystal
 	"crystal",
 #endif
+#ifdef MODE_lyapunov
+	"lyapunov",
+#endif
 #ifdef MODE_mandelbrot
 	"mandelbrot",
 #endif
@@ -361,6 +371,9 @@ static char nice_modes[][MAXMODECHARS] =
 #ifdef MODE_petal
 	"petal",
 #endif
+#ifdef MODE_petri
+	"petri",
+#endif
 #ifdef MODE_puzzle
 	"puzzle",
 #endif
@@ -449,6 +462,12 @@ static char use3d_modes[][MAXMODECHARS] =
 static char mouse_modes[][MAXMODECHARS] =
 {
 #ifndef DISABLE_INTERACTIVE
+#ifdef MODE_maze
+	"maze",
+#endif
+#ifdef MODE_pacman
+	"pacman",
+#endif
 #ifdef MODE_solitare
 	"solitare",
 #endif
@@ -456,6 +475,7 @@ static char mouse_modes[][MAXMODECHARS] =
 	"tetris",
 #endif
 #endif
+/* Mostly harmless interaction */
 #ifdef MODE_eyes
 	"eyes",
 #endif
@@ -468,7 +488,7 @@ static char mouse_modes[][MAXMODECHARS] =
 #ifdef MODE_swarm
 	"swarm",
 #endif
-#ifdef MODE_swarm
+#ifdef MODE_t3d
 	"t3d"
 #endif
 };
@@ -480,6 +500,9 @@ static char automata_modes[][MAXMODECHARS] =
 #ifdef MODE_ant
 	"ant",
 #endif
+#ifdef MODE_ant3d
+	"ant3d",
+#endif
 #ifdef MODE_bug
 	"bug",
 #endif
@@ -501,6 +524,9 @@ static char automata_modes[][MAXMODECHARS] =
 #ifdef MODE_loop
 	"loop",
 #endif
+#ifdef MODE_petri
+	"petri",
+#endif
 #ifdef MODE_voters
 	"voters",
 #endif
@@ -688,6 +714,8 @@ pickMode(void)
 	int         mode, i;
 
 	if (mode_indexes == NULL) {
+		if (!nmodes)
+			return 0;
 		if ((mode_indexes = (int *) calloc(nmodes,
 				sizeof (int))) == NULL) {
 			if (sequential)
