@@ -33,6 +33,7 @@ static const char sccsid[] = "@(#)rotor.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_rotor
 #define PROGCLASS "Rotor"
 #define HACK_INIT init_rotor
 #define HACK_DRAW draw_rotor
@@ -110,11 +111,11 @@ static void
 free_rotor(rotorstruct *rp)
 {
 	if (rp->elements != NULL) {
-		(void) free((void *) rp->elements);
+		free(rp->elements);
 		rp->elements = (elem *) NULL;
 	}
 	if (rp->save != NULL) {
-		(void) free((void *) rp->save);
+		free(rp->save);
 		rp->save = (XPoint *) NULL;
 	}
 }
@@ -168,7 +169,7 @@ init_rotor(ModeInfo * mi)
 		if (rp->num < 0) {
 			rp->num = NRAND(-rp->num) + 1;
 			if (rp->elements != NULL) {
-				(void) free((void *) rp->elements);
+				free(rp->elements);
 				rp->elements = (elem *) NULL;
 			}
 		}
@@ -373,7 +374,7 @@ release_rotor(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_rotor(&rotors[screen]);
-		(void) free((void *) rotors);
+		free(rotors);
 		rotors = (rotorstruct *) NULL;
 	}
 }
