@@ -35,6 +35,7 @@ static const char sccsid[] = "@(#)spiral.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_spiral
 #define PROGCLASS "Spiral"
 #define HACK_INIT init_spiral
 #define HACK_DRAW draw_spiral
@@ -140,7 +141,10 @@ init_spiral(ModeInfo * mi)
 	MI_CLEARWINDOW(mi);
 
 	/* Init */
-	sp->nlength = MI_CYCLES(mi);
+	if (MI_CYCLES(mi) < 2)
+		sp->nlength = 2;
+	else
+		sp->nlength = MI_CYCLES(mi);
 
 	if (!sp->traildots)
 		if ((sp->traildots = (Traildots *) malloc(sp->nlength *
@@ -305,9 +309,9 @@ release_spiral(ModeInfo * mi)
 			spiralstruct *sp = &spirals[screen];
 
 			if (sp->traildots)
-				(void) free((void *) sp->traildots);
+				free(sp->traildots);
 		}
-		(void) free((void *) spirals);
+		free(spirals);
 		spirals = (spiralstruct *) NULL;
 	}
 }
