@@ -52,6 +52,7 @@ static const char sccsid[] = "@(#)star.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_star
 #define PROGCLASS "Star"
 #define HACK_INIT init_star
 #define HACK_DRAW draw_star
@@ -336,13 +337,13 @@ free_star(Display *display, starstruct *sp)
 	int         i;
 
 	if (sp->astars != NULL) {
-		(void) free((void *) sp->astars);
+		free(sp->astars);
 		sp->astars = (astar *) NULL;
 	}
 	if (sp->pixmaps != None) {
 		for (i = 0; i < sp->max_star_size; i++)
 			XFreePixmap(display, sp->pixmaps[i]);
-		(void) free((void *) sp->pixmaps);
+		free(sp->pixmaps);
 		sp->pixmaps = None;
 	}
 	if (sp->stippledGC != None) {
@@ -714,7 +715,7 @@ init_star(ModeInfo * mi)
 	sp->nstars = MI_COUNT(mi);
 	if (sp->nstars < -MIN_STARS) {
 		if (sp->astars) {
-			(void) free((void *) sp->astars);
+			free(sp->astars);
 			sp->astars = (astar *) NULL;
 		}
 		sp->nstars = NRAND(-sp->nstars - MIN_STARS + 1) + MIN_STARS;
@@ -790,7 +791,7 @@ release_star(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_star(MI_DISPLAY(mi), &stars[screen]);
-		(void) free((void *) stars);
+		free(stars);
 		stars = (starstruct *) NULL;
 	}
 }
