@@ -32,6 +32,7 @@ static const char sccsid[] = "@(#)wator.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_wator
 #define PROGCLASS "Wator"
 #define HACK_INIT init_wator
 #define HACK_DRAW draw_wator
@@ -342,7 +343,7 @@ removefrom_kindlist(watorstruct * wp, CellList * ptr)
 	ptr->previous->next = ptr->next;
 	ptr->next->previous = ptr->previous;
 	wp->arr[ptr->info.col + ptr->info.row * wp->ncols] = 0;
-	(void) free((void *) ptr);
+	free(ptr);
 }
 
 static Bool
@@ -399,7 +400,7 @@ flush_kindlist(watorstruct * wp, int kind)
 		wp->currkind->previous->next = wp->lastkind[kind];
 		wp->lastkind[kind]->previous = wp->currkind->previous;
 		/* wp->arr[wp->currkind->info.col + wp->currkind->info.row * wp->ncols] = 0; */
-		(void) free((void *) wp->currkind);
+		free(wp->currkind);
 	}
 }
 
@@ -631,16 +632,16 @@ free_wator(watorstruct *wp)
 	for (kind = 0; kind <= KINDS; kind++) {
 		if (wp->firstkind[kind] != NULL) {
 			flush_kindlist(wp, kind);
-			(void) free((void *) wp->firstkind[kind]);
+			free(wp->firstkind[kind]);
 			wp->firstkind[kind] = (struct _CellList *) NULL;
 		}
 		if (wp->lastkind[kind] != NULL) {
-			(void) free((void *) wp->lastkind[kind]);
+			free(wp->lastkind[kind]);
 			wp->lastkind[kind] = (struct _CellList *) NULL;
 		}
 	}
 	if (wp->arr != NULL) {
-		(void) free((void *) wp->arr);
+		free(wp->arr);
 		wp->arr = (CellList **) NULL;
 	}
 }
@@ -706,10 +707,10 @@ init_wator(ModeInfo * mi)
 	if (wp->neighbors == 6) {
 		int         nccols, ncrows, sides;
 
-		if (wp->width < 2)
-			wp->width = 2;
-		if (wp->height < 4)
-			wp->height = 4;
+		if (wp->width < 8)
+			wp->width = 8;
+		if (wp->height < 8)
+			wp->height = 8;
 		if (size < -MINSIZE)
 			wp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(wp->width, wp->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
@@ -724,7 +725,7 @@ init_wator(ModeInfo * mi)
 		wp->xs = wp->ys;
 		wp->pixelmode = True;
 		nccols = MAX(wp->width / wp->xs - 2, 2);
-		ncrows = MAX(wp->height / wp->ys - 1, 2);
+		ncrows = MAX(wp->height / wp->ys - 1, 4);
 		wp->ncols = nccols / 2;
 		wp->nrows = 2 * (ncrows / 4);
 		wp->xb = (wp->width - wp->xs * nccols) / 2 + wp->xs / 2;
@@ -804,7 +805,7 @@ init_wator(ModeInfo * mi)
 	wp->positions = wp->ncols * wp->nrows;
 
 	if (wp->arr != NULL)
-		(void) free((void *) wp->arr);
+		free(wp->arr);
 	if ((wp->arr = (CellList **) calloc(wp->positions,
 			sizeof (CellList *))) == NULL) {
 		free_wator(wp);
@@ -1021,7 +1022,7 @@ release_wator(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_wator(&wators[screen]);
-		(void) free((void *) wators);
+		free(wators);
 		wators = (watorstruct *) NULL;
 	}
 }
