@@ -86,6 +86,7 @@ static const char sccsid[] = "@(#)wire.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_wire
 #define PROGCLASS "Wire"
 #define HACK_INIT init_wire
 #define HACK_DRAW draw_wire
@@ -199,38 +200,7 @@ position_of_neighbor(circuitstruct * wp, int dir, int *pcol, int *prow)
 
 	/* NO WRAPING */
 
-	if (wp->neighbors == 6) {
-		switch (dir) {
-			case 0:
-				col = col + 1;
-				break;
-			case 60:
-				if (!(row & 1))
-					col = col + 1;
-				row = row - 1;
-				break;
-			case 120:
-				if (row & 1)
-					col = col - 1;
-				row = row - 1;
-				break;
-			case 180:
-				col = col - 1;
-				break;
-			case 240:
-				if (row & 1)
-					col = col - 1;
-				row = row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = col + 1;
-				row = row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (wp->neighbors == 4 || wp->neighbors == 8) {
+	if (wp->neighbors == 4 || wp->neighbors == 6 ||  wp->neighbors == 8) {
 		switch (dir) {
 			case 0:
 				col = col + 1;
@@ -239,9 +209,19 @@ position_of_neighbor(circuitstruct * wp, int dir, int *pcol, int *prow)
 				col = col + 1;
 				row = row - 1;
 				break;
+			case 60:
+				if (!(row & 1))
+					col = col + 1;
+				row = row - 1;
+				break;
 			case 90:
 				row = row - 1;
 				break;
+			case 120:
+				if (row & 1)
+					col = col - 1;
+				row = row - 1;
+				break;
 			case 135:
 				col = col - 1;
 				row = row - 1;
@@ -253,9 +233,19 @@ position_of_neighbor(circuitstruct * wp, int dir, int *pcol, int *prow)
 				col = col - 1;
 				row = row + 1;
 				break;
+			case 240:
+				if (row & 1)
+					col = col - 1;
+				row = row + 1;
+				break;
 			case 270:
 				row = row + 1;
 				break;
+			case 300:
+				if (!(row & 1))
+					col = col + 1;
+				row = row + 1;
+				break;
 			case 315:
 				col = col + 1;
 				row = row + 1;
@@ -393,7 +383,7 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 
 		wp->shape.hexagon[0].x = wp->xb + ccol * wp->xs;
 		wp->shape.hexagon[0].y = wp->yb + crow * wp->ys;
-		if (wp->xs == 1 && wp->ys == 1)
+		if (wp->xs <= 3 || wp->ys <= 3)
 			XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi), gc,
 			wp->shape.hexagon[0].x, wp->shape.hexagon[0].y);
 		else
@@ -515,7 +505,7 @@ free_state(circuitstruct * wp, int state)
 	while (wp->cellList[state]) {
 		current = wp->cellList[state];
 		wp->cellList[state] = wp->cellList[state]->next;
-		(void) free((void *) current);
+		free(current);
 	}
 	wp->ncells[state] = 0;
 }
@@ -549,7 +539,7 @@ draw_state(ModeInfo * mi, int state)
 			ccol = 2 * col + !(row & 1), crow = 2 * row;
 			wp->shape.hexagon[0].x = wp->xb + ccol * wp->xs;
 			wp->shape.hexagon[0].y = wp->yb + crow * wp->ys;
-			if (wp->xs == 1 && wp->ys == 1)
+			if (wp->xs <= 1 || wp->ys <= 1)
 				XDrawPoint(MI_DISPLAY(mi), MI_WINDOW(mi),
 					       gc, wp->shape.hexagon[0].x, wp->shape.hexagon[0].y);
 			else
@@ -579,7 +569,7 @@ draw_state(ModeInfo * mi, int state)
 		/* Finally get to draw */
 		XFillRectangles(MI_DISPLAY(mi), MI_WINDOW(mi), gc, rects, nrects);
 		/* Free up rects list and the appropriate part of the cellList */
-		(void) free((void *) rects);
+		free(rects);
 	} else {		/* TRI */
 		while (current) {
 			int         col, row, orient;
@@ -753,11 +743,11 @@ free_wire(Display *display, circuitstruct *wp)
 		wp->stippledGC = None;
 	}
 	if (wp->oldcells != NULL) {
-		(void) free((void *) wp->oldcells);
+		free(wp->oldcells);
 		wp->oldcells = (unsigned char *) NULL;
 	}
 	if (wp->newcells != NULL) {
-		(void) free((void *) wp->newcells);
+		free(wp->newcells);
 		wp->newcells = (unsigned char *) NULL;
 	}
 	free_list(wp);
@@ -771,7 +761,7 @@ release_wire(ModeInfo * mi)
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
 			free_wire(MI_DISPLAY(mi), &circuits[screen]);
-		(void) free((void *) circuits);
+		free(circuits);
 		circuits = (circuitstruct *) NULL;
 	}
 }
@@ -847,10 +837,10 @@ init_wire(ModeInfo * mi)
 	if (wp->neighbors == 6) {
 		int         nccols, ncrows;
 
-		if (wp->width < 4)
-			wp->width = 4;
-		if (wp->height < 4)
-			wp->height = 4;
+		if (wp->width < 8)
+			wp->width = 8;
+		if (wp->height < 8)
+			wp->height = 8;
 		if (size < -MINSIZE)
 			wp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(wp->width, wp->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
@@ -863,8 +853,8 @@ init_wire(ModeInfo * mi)
 			wp->ys = MIN(size, MAX(MINSIZE, MIN(wp->width, wp->height) /
 					       MINGRIDSIZE));
 		wp->xs = wp->ys;
-		nccols = MAX(wp->width / wp->xs - 2, 2);
-		ncrows = MAX(wp->height / wp->ys - 1, 2);
+		nccols = MAX(wp->width / wp->xs - 2, 16);
+		ncrows = MAX(wp->height / wp->ys - 1, 16);
 		wp->ncols = nccols / 2;
 		wp->nrows = ncrows / 2;
 		wp->nrows -= !(wp->nrows & 1);	/* Must be odd */
@@ -887,8 +877,8 @@ init_wire(ModeInfo * mi)
 			wp->ys = MIN(size, MAX(MINSIZE, MIN(wp->width, wp->height) /
 					       MINGRIDSIZE));
 		wp->xs = wp->ys;
-		wp->ncols = MAX(wp->width / wp->xs, 2);
-		wp->nrows = MAX(wp->height / wp->ys, 2);
+		wp->ncols = MAX(wp->width / wp->xs, 8);
+		wp->nrows = MAX(wp->height / wp->ys, 8);
 		wp->xb = (wp->width - wp->xs * wp->ncols) / 2;
 		wp->yb = (wp->height - wp->ys * wp->nrows) / 2;
 	} else {		/* TRI */
@@ -910,8 +900,8 @@ init_wire(ModeInfo * mi)
 			wp->ys = MIN(size, MAX(MINSIZE, MIN(wp->width, wp->height) /
 					       MINGRIDSIZE));
 		wp->xs = (int) (1.52 * wp->ys);
-		wp->ncols = (MAX(wp->width / wp->xs - 1, 4) / 2) * 2;
-		wp->nrows = (MAX(wp->height / wp->ys - 1, 2) / 2) * 2 - 1;
+		wp->ncols = (MAX(wp->width / wp->xs - 1, 8) / 2) * 2;
+		wp->nrows = (MAX(wp->height / wp->ys - 1, 8) / 2) * 2 - 1;
 		wp->xb = (wp->width - wp->xs * wp->ncols) / 2 + wp->xs / 2;
 		wp->yb = (wp->height - wp->ys * wp->nrows) / 2 + wp->ys;
 		for (orient = 0; orient < 2; orient++) {
@@ -934,10 +924,14 @@ init_wire(ModeInfo * mi)
 	wp->bncols = wp->ncols + 4;
 	wp->bnrows = wp->nrows + 4;
 
+	if (MI_IS_VERBOSE(mi))
+		(void) fprintf(stdout,
+			"neighbors %d, ncols %d, nrows %d\n",
+			wp->neighbors, wp->ncols, wp->nrows);
 	MI_CLEARWINDOW(mi);
 
 	if (wp->oldcells != NULL) {
-		(void) free((void *) wp->oldcells);
+		free(wp->oldcells);
 		wp->oldcells = (unsigned char *) NULL;
 	}
 	if ((wp->oldcells = (unsigned char *) calloc(wp->bncols * wp->bnrows,
@@ -947,7 +941,7 @@ init_wire(ModeInfo * mi)
 	}
 
 	if (wp->newcells != NULL) {
-		(void) free((void *) wp->newcells);
+		free(wp->newcells);
 		wp->newcells = (unsigned char *) NULL;
 	}
 	if ((wp->newcells = (unsigned char *) calloc(wp->bncols * wp->bnrows,
@@ -971,7 +965,7 @@ init_wire(ModeInfo * mi)
 void
 draw_wire(ModeInfo * mi)
 {
-	int         offset, i, j;
+	int         offset, i, j, found = 0;
 	unsigned char *z, *znew;
 	circuitstruct *wp;
 
@@ -990,6 +984,7 @@ draw_wire(ModeInfo * mi)
 			z = wp->oldcells + offset;
 			znew = wp->newcells + offset;
 			if (*z != *znew) {	/* Counting on once a space always a space */
+				found = 1;
 				*z = *znew;
 				if (!addtolist(mi, i - 2, j - 2, *znew - 1)) {
 					free_wire(MI_DISPLAY(mi), wp);
@@ -1003,7 +998,7 @@ draw_wire(ModeInfo * mi)
 			free_wire(MI_DISPLAY(mi), wp);
 			return;
 		}
-	if (++wp->generation > MI_CYCLES(mi)) {
+	if (++wp->generation > MI_CYCLES(mi) || !found) {
 		init_wire(mi);
 		return;
 	} else
