@@ -30,6 +30,7 @@ static const char sccsid[] = "@(#)xjack.c	5.00 2000/11/01 xlockmore";
  */
 
 #ifdef STANDALONE
+#define MODE_xjack
 #define PROGCLASS "XJack"
 #define HACK_INIT init_xjack
 #define HACK_DRAW draw_xjack
@@ -105,7 +106,7 @@ release_xjack(ModeInfo * mi)
 			if (jp->gc)
 				XFreeGC(display, jp->gc);
 		}
-		(void) free((void *) jacks);
+		free(jacks);
 		jacks = (jackstruct *) NULL;
 	}
 	if (mode_font != None) {
@@ -155,11 +156,11 @@ init_xjack(ModeInfo * mi)
 	jp->char_width = (mode_font->per_char
 				? mode_font->per_char['n' - mode_font->min_char_or_byte2].rbearing
 				: mode_font->min_bounds.rbearing);
-  if (!jp->char_width)
+	if (!jp->char_width)
 		jp->char_width = 1;
 	jp->line_height = jp->height + 1;
 
-	jp->columns = (jp->win_width - jp->hspace - jp->hspace) / jp->char_width;
+	jp->columns = MAX((jp->win_width - jp->hspace - jp->hspace) / jp->char_width, 2);
 	jp->rows = (MI_HEIGHT(mi) - jp->vspace - jp->vspace) / jp->line_height;
 
 	jp->left = 0xFF & (NRAND((jp->columns / 2) + 1));
@@ -198,6 +199,8 @@ draw_xjack(ModeInfo * mi)
 	if (jp->gc == None)
 		return;
 
+	MI_IS_DRAWN(mi) = True;
+
 	jp->word_length = 0;
 	for (s2 = jp->s; *s2 && *s2 != ' '; s2++)
 		jp->word_length++;
