@@ -37,7 +37,8 @@ MagickFileToImage(ModeInfo * mi, char *filename, XImage ** image)
 /* PURIFY reports a memory leak on the next line */
    GetImageInfo( &imageInfo );
    imageInfo.dither = 0;
-   strcpy( imageInfo.filename , filename );
+   (void) strcpy( imageInfo.filename , filename );
+   GetExceptionInfo( &exception );
    imageData = ReadImage ( &imageInfo, &exception );
    if ( imageData == (Image*) NULL )
      {
