@@ -520,6 +520,9 @@ extern void release_last_mode(ModeInfo *);
 #if defined( HAVE_TTF ) && defined( HAVE_GLTT )
 #define MODE_text3d
 #endif
+#if defined( HAVE_FREETYPE ) && defined( HAVE_FTGL )
+#define MODE_text3d2
+#endif
 #endif
 #endif
 #ifdef HAVE_CXX
@@ -536,6 +539,7 @@ extern void release_last_mode(ModeInfo *);
 #endif
 #endif
 #define MODE_ant
+#define MODE_ant3d
 #define MODE_apollonian
 #define MODE_ball
 #define MODE_bat
@@ -654,6 +658,14 @@ extern ModeHook refresh_ant;
 extern ModeSpecOpt ant_opts;
 #endif
 
+#ifdef MODE_ant3d
+extern ModeHook init_ant3d;
+extern ModeHook draw_ant3d;
+extern ModeHook release_ant3d;
+extern ModeHook refresh_ant3d;
+extern ModeSpecOpt ant3d_opts;
+#endif
+
 #ifdef MODE_apollonian
 extern ModeHook init_apollonian;
 extern ModeHook draw_apollonian;
@@ -1532,6 +1544,15 @@ extern ModeHook change_text3d;
 extern ModeSpecOpt text3d_opts;
 #endif
 
+#ifdef MODE_text3d2
+extern ModeHook init_text3d2;
+extern ModeHook draw_text3d2;
+extern ModeHook release_text3d2;
+extern ModeHook refresh_text3d2;
+extern ModeHook change_text3d2;
+extern ModeSpecOpt text3d2_opts;
+#endif
+
 #ifdef MODE_thornbird
 extern ModeHook init_thornbird;
 extern ModeHook draw_thornbird;
