@@ -95,7 +95,7 @@ XlockrmParseCommand(
     XrmQuark		quarks[100];
     XrmBinding		*start_bindings;
     XrmQuark		*start_quarks;
-    char		*optP, *argP = NULL, optchar, argchar = '\0';
+    char		*optP, *argP = (char *) NULL, optchar, argchar = '\0';
     int			matches;
     enum {DontCare, Check, NotSorted, Sorted} table_is_sorted;
 #if 0
