@@ -11,6 +11,9 @@ static const char sccsid[] = "@(#)passwd.c	5.00 2000/11/01 xlockmore";
  * Revision History:
  *
  * Changes maintained by David Bagley <bagleyd@tux.org>
+ * 23-Feb-2003: Timothy Reed <treed1@twcny.rr.com>
+ *              Updated PAM_conv to call over to xlock to display messages
+ *              and return responses to the calling PAM
  * 01-Nov-2000: SIA passwording for Tru64 added
  * 18-Feb-1999: allowroot option no longer ignored for PAM authentication.
  *              Sven Paas <Sven.Paas@t-online.de>
@@ -53,7 +56,11 @@ extern Bool grabmouse;
 extern Bool nolock;
 extern char *cpasswd;
 
-#if defined( __CYGWIN__ )
+#ifdef PAM
+#include <X11/Xlib.h>
+#endif
+
+#ifdef __CYGWIN__
 #include <crypt.h>
 #endif
 
@@ -80,11 +87,12 @@ static login_cap_t *rlc = NULL;
 #include <syslog.h>
 #endif
 
-#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
-#include <fcntl.h>
 #include <errno.h>
 extern int  errno;
 
+#if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
+#include <fcntl.h>
+
 void        get_multiple(struct passwd *);
 void        set_multiple(int uid);
 
@@ -266,7 +274,7 @@ char        global_user[PASSLENGTH];
 #endif
 
 #ifdef PAM
-#include <security/pam_appl.h>
+
 /* used to pass the password to the conversation function */
 static char *PAM_password;
 
@@ -275,9 +283,10 @@ static char *PAM_password;
  * Here we assume (for now, at least) that echo on means login name, and
  * echo off means password.
  */
-static int
-PAM_conv(int num_msg,
-	 /* const */ struct pam_message **msg,
+
+int
+PAM_conv(int num_msg, 
+	 const struct pam_message **msg,
 	 struct pam_response **resp,
 	 void *appdata_ptr)
 {
@@ -287,48 +296,86 @@ PAM_conv(int num_msg,
 
 #define COPY_STRING(s) (s) ? strdup(s) : NULL
 
-
 	reply = (struct pam_response *) malloc(sizeof (struct pam_response) *
 					       num_msg);
 
 	if (!reply)
 		return PAM_CONV_ERR;
 
-
 	for (replies = 0; replies < num_msg; replies++) {
 		switch (msg[replies]->msg_style) {
 			case PAM_PROMPT_ECHO_ON:
-				reply[replies].resp_retcode = PAM_SUCCESS;
-				reply[replies].resp = COPY_STRING(user);
+  			        PAM_putText( msg[replies], &reply[replies], True );
+#ifdef DEBUG
+			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_ON\n" );
+			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
+					 reply[replies].resp, reply[replies].resp_retcode );
+#endif
 				/* PAM frees resp */
 				break;
 			case PAM_PROMPT_ECHO_OFF:
-				reply[replies].resp_retcode = PAM_SUCCESS;
-				reply[replies].resp = COPY_STRING(PAM_password);
+			  if( strstr( msg[replies]->msg, "Password" ) == NULL )
+			  {
+			    PAM_putText( msg[replies], &reply[replies], False );
+			  }
+			  else
+			  {
+			    reply[replies].resp_retcode = PAM_SUCCESS;
+			    reply[replies].resp = COPY_STRING(PAM_password);
+			  }
+#ifdef DEBUG
+			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_OFF\n" );
+			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
+					 reply[replies].resp, reply[replies].resp_retcode );
+#endif
 				/* PAM frees resp */
 				break;
 			case PAM_TEXT_INFO:
-				/* ignore it... */
-				reply[replies].resp_retcode = PAM_SUCCESS;
-				reply[replies].resp = NULL;
+			  if( strstr( msg[replies]->msg, "Password" ) == NULL )
+			  {
+			    PAM_putText( msg[replies], &reply[replies], False );
+			  }
+			  else
+			  {
+			    reply[replies].resp_retcode = PAM_SUCCESS;
+			    reply[replies].resp = NULL;
+			  }
+#ifdef DEBUG
+			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_ON\n" );
+			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
+					 reply[replies].resp, reply[replies].resp_retcode );
+#endif
+				/* PAM frees resp */
 				break;
 			case PAM_ERROR_MSG:
-				/* ignore it... */
-				reply[replies].resp_retcode = PAM_SUCCESS;
-				reply[replies].resp = NULL;
+			  if( strstr( msg[replies]->msg, "Password" ) == NULL )
+			  {
+			    PAM_putText( msg[replies], &reply[replies], False );
+			  }
+			  else
+			  {
+			    reply[replies].resp_retcode = PAM_SUCCESS;
+			    reply[replies].resp = NULL;
+			  }
+#ifdef DEBUG
+			        (void) printf( "Back From PAM_putText: PAM_PROMPT_ECHO_ON\n" );
+			        (void) printf( "Response is: (%s)\n, Return Code is: (%d)\n", 
+					 reply[replies].resp, reply[replies].resp_retcode );
+#endif
 				break;
 			default:
 				/* Must be an error of some sort... */
-				(void) free((void *) reply);
+				free(reply);
 				return PAM_CONV_ERR;
 		}
 	}
 	*resp = reply;
 	return PAM_SUCCESS;
 }
-static struct pam_conv PAM_conversation =
+
+static const struct pam_conv PAM_conversation =
 {
-	&PAM_conv,
+	PAM_conv,
 	NULL
 };
 
@@ -337,12 +384,18 @@ static struct pam_conv PAM_conversation =
 #if !defined( ultrix ) && !defined( PAM )
 static char userpass[PASSLENGTH];
 static char rootpass[PASSLENGTH];
+#ifdef USE_XLOCK_GROUP
+#include <grp.h>
+static char *grouppass = (char *) NULL;
+static void getCryptedXlockGroupPasswds();
+static int checkGroupXlockPasswds(char *buffer);
+#endif
 
 #ifdef VMS
 static char root[] = ROOT;
 
 #endif
-#endif /* !ultrix */
+#endif /* !ultrix && !PAM */
 
 #ifdef DCE_PASSWD
 static int  usernet, rootnet;
@@ -353,10 +406,16 @@ static int  check_dce_net_passwd(char *, char *);
 #if defined( HAVE_KRB4 ) || defined( HAVE_KRB5 )
 #ifdef HAVE_KRB4
 #include <krb.h>
+static int krb_check_password(struct passwd *, char *);
 #else /* HAVE_KRB5 */
 #include <krb5.h>
+#include <com_err.h>
+int krb5_valid = 0;
+extern Display *dsp;
+extern Window parent;
+extern Bool parentSet;
+static int  krb_check_password(char *, char *);
 #endif
-static int  krb_check_password(struct passwd *, char *);
 
 #endif
 
@@ -376,6 +435,10 @@ extern char *crypt(const char *, const char *);
 #endif
 #endif
 
+#ifdef PASSWD_HELPER_PROGRAM
+static int	run_helper(const char *user, const char *pass);
+#endif
+
 #if (defined( USE_XLOCKRC ) || (!defined( OSF1_ENH_SEC ) && !defined( HP_PASSWDETC ) && !defined( VMS )))
 static struct passwd *
 my_passwd_entry(void)
@@ -409,6 +472,9 @@ my_passwd_entry(void)
 	if (!pw)
 		return (pw);
 #ifdef HAVE_SHADOW
+/*- 
+ * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
+ * if using NIS compatibility mode, and is internal to libc.so */
 	if ((spw = getspnam(pw->pw_name)) != NULL) {
 		char       *tmp;	/* swap */
 
@@ -443,7 +509,7 @@ checkDynamic()
 		(void) sprintf(buf,
 			     "%s: could not get user shell.\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	return (!(strcmp(pw->pw_shell, SAFELOG_SWCHECK) &&
 		  strcmp(pw->pw_shell, SAFELOG_IDENT)));
@@ -488,7 +554,7 @@ getUserName(void)
 		(void) sprintf(buf,
 			       "%s: could not get user password (fork failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) close(pipefd[1]);
 
@@ -502,7 +568,7 @@ getUserName(void)
 		(void) sprintf(buf,
 		 "%s: could not get user name (read failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	user[total] = 0;
 
@@ -511,7 +577,7 @@ getUserName(void)
 		(void) sprintf(buf,
 			       "%s: could not get user name (lookups failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 #else /* !HP_PASSWDETC */
 #ifdef OSF1_ENH_SEC
@@ -524,7 +590,7 @@ getUserName(void)
 		(void) sprintf(buf,
 			       "%s: could not get user name.\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) strcpy(user, pw->ufld.fd_name);
 #else /* !OSF1_ENH_SEC */
@@ -532,16 +598,65 @@ getUserName(void)
 	struct passwd *pw;
 	char       *buf;
 
+#ifdef HAVE_KRB5
+	krb5_context context;
+	krb5_principal me = NULL;
+	krb5_ccache cc = NULL;
+	krb5_error_code code;
+	char *krb5_user = NULL;
+	int got_context = 0;
+
+	krb5_valid = 0;
+
+	code = krb5_init_context(&context);
+
+	if (code == 0) {
+		got_context = 1;
+		code = krb5_cc_default(context, &cc);
+	}
+
+	if (code == 0)
+		code = krb5_cc_get_principal(context, cc, &me);
+
+	if (code == 0)
+		code = krb5_unparse_name(context, me, &krb5_user);
+
+	if (cc)
+		krb5_cc_close(context, cc);
+
+	if (me)
+		krb5_free_principal(context, me);
+
+	if (got_context)
+		krb5_free_context(context);
+
+	if (code == 0) {
+		if (strlen(krb5_user) < PASSLENGTH + 1) {
+			strncpy(user, krb5_user, PASSLENGTH);
+			krb5_valid = 1;
+		} else
+			code = 255;
+
+		free(krb5_user);
+	}
+
+	if (code) {
+
+#endif /* HAVE_KRB5 */
 	if (!(pw = my_passwd_entry())) {
 		/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 		buf = (char *) malloc(strlen(ProgramName) + 80);
 		(void) sprintf(buf,
 			       "%s: could not get user name.\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) strcpy(user, pw->pw_name);
 
+#ifdef HAVE_KRB5
+	}
+#endif /* HAVE_KRB5 */
+
 #if ( HAVE_FCNTL_H && (defined( USE_MULTIPLE_ROOT ) || defined( USE_MULTIPLE_USER )))
 	get_multiple(pw);
 #endif
@@ -561,10 +676,8 @@ gpasskey(char *pass)
 	static char saltchars[CPASSCHARS + 1] =
 	"abcdefghijklmnopwrstuvwxyzABCDEFGHIJKLMNOPWRSTUVWXYZ1234567890./";
 	char        pw[9], salt[3], *pw2;
-
 #if 0
 	char       *getpass(const char *);
-
 #endif
 
 	(void) strncpy(pw, (char *) getpass("Key: "), sizeof (pw));
@@ -601,7 +714,7 @@ gpass(void)
 				(void) sprintf(buf,
 					  "%s: Who are you?\n", ProgramName);
 				error(buf);
-				(void) free((void *) buf);	/* Should never get here */
+				free(buf);	/* Should never get here */
 			}
 			home = p->pw_dir;
 		}
@@ -728,7 +841,7 @@ getCryptedUserPasswd(void)
 		(void) sprintf(buf,
 			       "%s: could not get user password (fork failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) close(pipefd[1]);
 
@@ -742,7 +855,7 @@ getCryptedUserPasswd(void)
 		(void) sprintf(buf,
 			       "%s: could not get user password (read failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	user[total] = 0;
 
@@ -751,7 +864,7 @@ getCryptedUserPasswd(void)
 		(void) sprintf(buf,
 			       "%s: could not get user password (lookups failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 #else /* !HP_PASSWDETC */
 #ifdef OSF1_ENH_SEC
@@ -764,7 +877,7 @@ getCryptedUserPasswd(void)
 		(void) sprintf(buf,
 		"%s: could not get encrypted user password.\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) strcpy(userpass, pw->ufld.fd_encrypt);
 	/* also save encryption algorithm associated with encrypted password */
@@ -779,7 +892,7 @@ getCryptedUserPasswd(void)
 		(void) sprintf(buf,
 		"%s: could not get encrypted user password.\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	/*if ((pw = (struct passwd *) getpwuid(getuid())) == NULL) */
 	/* Check if there is any chance of unlocking the display later...  */
@@ -871,7 +984,7 @@ getCryptedRootPasswd(void)
 		(void) sprintf(buf,
 			       "%s: could not get root password (fork failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) close(pipefd[1]);
 
@@ -885,7 +998,7 @@ getCryptedRootPasswd(void)
 		(void) sprintf(buf,
 			       "%s: could not get root password (read failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	rootpass[total] = 0;
 
@@ -894,7 +1007,7 @@ getCryptedRootPasswd(void)
 		(void) sprintf(buf,
 			       "%s: could not get root password (lookups failed)\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 #else /* !HP_PASSWDETC */
 #ifdef OSF1_ENH_SEC
@@ -906,7 +1019,7 @@ getCryptedRootPasswd(void)
 		(void) sprintf(buf,
 		"%s: could not get encrypted root password.\n", ProgramName);
 		error(buf);
-		(void) free((void *) buf);	/* Should never get here */
+		free(buf);	/* Should never get here */
 	}
 	(void) strcpy(rootpass, pw->ufld.fd_encrypt);
 	/* also save encryption algorithm associated with encrypted password */
@@ -927,9 +1040,12 @@ getCryptedRootPasswd(void)
 			(void) sprintf(buf,
 				       "%s: could not get encrypted root password.\n", ProgramName);
 			error(buf);
-			(void) free((void *) buf);	/* Should never get here */
+			free(buf);	/* Should never get here */
 		}
 #ifdef HAVE_SHADOW
+/*-
+ * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
+ * if using NIS compatibility mode, and is internal to libc.so */
 	if ((spw = getspnam(pw->pw_name)) != NULL) {
 		char       *tmp;	/* swap */
 
@@ -972,14 +1088,17 @@ checkUser(char *buffer)
 		(void) strcpy(userpass, "*");
 	else {
 #ifdef HAVE_SHADOW
-	if ((spw = getspnam(pw->pw_name)) != NULL) {
-		char       *tmp;	/* swap */
+/*- 
+ * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
+ * if using NIS compatibility mode, and is internal to libc.so */
+		if ((spw = getspnam(pw->pw_name)) != NULL) {
+			char       *tmp;	/* swap */
 
-		tmp = pw->pw_passwd;
-		pw->pw_passwd = spw->sp_pwdp;
-		spw->sp_pwdp = tmp;
-	}
-	endspent();
+			tmp = pw->pw_passwd;
+			pw->pw_passwd = spw->sp_pwdp;
+			spw->sp_pwdp = tmp;
+		}
+		endspent();
 #endif
 		(void) strcpy(userpass, pw->pw_passwd);
 	}
@@ -1061,22 +1180,20 @@ checkPasswd(char *buffer)
 #endif
 
 #define PAM_BAIL if (pam_error != PAM_SUCCESS) { \
-    pam_end(pamh, 0); BAD_PAM_SETUID return 0; \
+    pam_end(pamh, 0); BAD_PAM_SETUID return False; \
 }
 #ifdef BAD_PAM
 	ruid = getuid();  /* the real user we are running as */
+	(void) seteuid(0); /* temporarily go to root so that pam can get shadow password */
 #endif
 	PAM_password = buffer;
 	pam_error = pam_start("xlock", user, &PAM_conversation, &pamh);
 	PAM_BAIL;
-#ifdef BAD_PAM
-	(void) seteuid(0); /* temporarily go to root so that pam can get shadow password */
-#endif
 	pam_error = pam_authenticate(pamh, 0);
 	if (pam_error != PAM_SUCCESS) {
                 if (!allowroot) {
                         pam_end(pamh, 0);
-                        return 0;
+                        return False;
                 }
 
 		/* Try as root; bail if no success there either */
@@ -1219,7 +1336,12 @@ checkPasswd(char *buffer)
 #if 0
 		if (!strcmp(userpass, "*"))
 #endif
-			if (krb_check_password((struct passwd *) getpwuid(getuid()), buffer2))
+#ifdef HAVE_KRB4
+			if (krb_check_password((struct passwd *) getpwuid(getuid
+()), buffer2))
+#else
+			if (krb_check_password(user, buffer2))
+#endif
 				return True;
 	}
 #endif /* !HAVE_KRB4 && !HAVE_KRB5 */
@@ -1257,7 +1379,8 @@ checkPasswd(char *buffer)
 			done = False;
 	}
 #if 0
-	(void) printf("buffer=%s, encrypt=%s, userpass=%s\n", buffer,
+	/* For debugging only! */
+	(void) printf("buffer=%s, encrypt=%s, userpass=%s,\n", buffer,
 		      (char *) crypt(buffer, userpass), userpass);
 #endif
 	if (!done && allowroot &&
@@ -1268,22 +1391,32 @@ checkPasswd(char *buffer)
 #endif
 	) {
 		done = True;	
-		if (!rootpass || !*buffer)
+		if (!(rootpass && *buffer))
 			/*
 			 * root has no password, don't let him in...
 			 */
+
 			done = False;
 #if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
 		else
 			syslog(SYSLOG_NOTICE, "%s: %s unlocked screen", ProgramName, ROOT);
 #endif
 	}
+#ifdef USE_XLOCK_GROUP
+	if (!done)
+		done = checkGroupXlockPasswds(buffer);
+#endif
 #endif /* !BSD_AUTH */
 #endif /* !ultrix */
 #endif /* !PAM */
 #endif /* !DCE_PASSWD */
 #endif /* !VMS */
 
+#ifdef PASSWD_HELPER_PROGRAM
+	if (!done && !(done = run_helper(user, buffer)) && allowroot)
+		done = run_helper(ROOT, buffer);
+#endif
+
 	return done;
 }
 
@@ -1334,7 +1467,7 @@ initDCE(void)
 				(void) sprintf(buf,
 					 "%s: out of memory\n", ProgramName);
 				error(buf);
-				(void) free((void *) buf);	/* Should never get here */
+				free(buf);	/* Should never get here */
 			default:
 				(void) fprintf(stderr,
 					       "%s: sec_login_setup_identity() returned status %d\n",
@@ -1348,7 +1481,7 @@ initDCE(void)
 			(void) sprintf(buf,
 			   "%s: could not get user password\n", ProgramName);
 			error(buf);
-			(void) free((void *) buf);	/* Should never get here */
+			free(buf);	/* Should never get here */
 		}
 		usernet = 0;
 		(void) strcpy(userpass, pwd->pw_passwd);
@@ -1381,7 +1514,7 @@ initDCE(void)
 					(void) sprintf(buf,
 					 "%s: out of memory\n", ProgramName);
 					error(buf);
-					(void) free((void *) buf);	/* Should never get here */
+					free(buf);	/* Should never get here */
 				default:
 					(void) fprintf(stderr,
 						       "%s: sec_login_setup_identity() returned status %d\n",
@@ -1522,7 +1655,7 @@ check_dce_net_passwd(char *usr, char *pass)
 			       error_string(error_status));
 	}
 	/* done with the copy of the password */
-	(void) free((void *) passcpy);
+	free(passcpy);
 
 	/* Refresh the context if we already have one */
 	if (current_context) {
@@ -1550,7 +1683,7 @@ check_dce_net_passwd(char *usr, char *pass)
 			}
 		}
 		/* done with the copy of the password */
-		(void) free((void *) passcpy);
+		free(passcpy);
 	}
 	/* make sure that the authentication service is not an imposter */
 	if (valid) {
@@ -1598,151 +1731,133 @@ krb_check_password(struct passwd *pwd, char *pass)
 #endif /* HAVE_KRB4 */
 
 #ifdef HAVE_KRB5
-/*-
- * Pretty much all of this was snatched out of the kinit from the Kerberos5b6
- * distribution.  The reason why I felt it was necessary to use kinit was
- * that if someone is locked for a long time, their credentials could expire,
- * so xlock must be able to get a new ticket.  -- dah <rodmur@ecst.csuchico.edu>
+/*
+ * This code has been completely rewritten from the original Kerberos code.
+ * It now uses the new krb5_get_init_creds() API, and uses a prompter callback
+ * (which lives in xlock.c, because it requires a lot of X-windows guts
+ * not exposed in this file).
+ *
+ * The code that would normally write out a new credential cache has been
+ * conditionalized under KRB5_WRITE_NEW_CCACHE (--enable-write-krb5 option
+ * to configure)
  */
-#define KRB5_DEFAULT_OPTIONS 		0
-#define KRB5_DEFAULT_LIFE 60*60*10	/* 10 hours */
+
+extern krb5_error_code xlock_prompter(krb5_context, void *, const char *,
+				      const char *, int, krb5_prompt []);
+	
 static int
-krb_check_password(struct passwd *pwd, char *pass)
+krb_check_password(char *user, char *pass)
 {
 	krb5_context kcontext;
-	krb5_timestamp now;
 	krb5_ccache ccache = NULL;
 	krb5_principal me;
-	krb5_principal server;
-	int         options = KRB5_DEFAULT_OPTIONS;
-	krb5_deltat lifetime = KRB5_DEFAULT_LIFE;
 	krb5_error_code code;
-	krb5_creds  my_creds;
-	char       *client_name;
-	krb5_address **addrs = (krb5_address **) 0;
-	krb5_preauthtype *preauth = NULL;
+	krb5_creds my_creds;
+	krb5_get_init_creds_opt options;
+	char *lifetimestr = NULL;
+	krb5_deltat lifetime;
+	int option;
 
-        krb5_deltat rlife = 0;
+	memset((void *) &my_creds, 0, sizeof (my_creds));
 
-	krb5_data   tgtname =
-	{
-		0,
-		KRB5_TGS_NAME_SIZE,
-		KRB5_TGS_NAME
-	};
+	krb5_get_init_creds_opt_init(&options);
 
-	krb5_init_context(&kcontext);
-	krb5_init_ets(kcontext);
-
-	if ((code = krb5_timeofday(kcontext, &now))) {
-		com_err(ProgramName, code, "while getting time of day");
-		return False;	/* seems better to deny access, than just exit, which
-				   was what happened in kinit  */
+	if ((code = krb5_init_context(&kcontext))) {
+		com_err(ProgramName, code, "while initializing Kerberos");
+		return 0;
 	}
-	if ((code = krb5_cc_default(kcontext, &ccache))) {
-		com_err(ProgramName, code, "while getting default ccache");
-		return False;
-	}
-	code = krb5_cc_get_principal(kcontext, ccache, &me);
-	if (code) {
-		if ((code = krb5_parse_name(kcontext, pwd->pw_name, &me))) {
-			com_err(ProgramName, code, "when parsing name %s", pwd->pw_name);
-			return False;
+
+	if (krb5_valid)
+		code = krb5_parse_name(kcontext, user, &me);
+	
+	if (! krb5_valid || (krb5_valid && code)) {
+		if ((code = krb5_cc_default(kcontext, &ccache))) {
+			com_err(ProgramName, code, "while getting default "
+				"ccache");
+			krb5_free_context(kcontext);
+			return 0;
+		}
+		code = krb5_cc_get_principal(kcontext, ccache, &me);
+		if (code) {
+			if ((code = krb5_parse_name(kcontext, user, &me))) {
+				com_err(ProgramName, code, "when parsing "
+					"name %s", user);
+				krb5_cc_close(kcontext, ccache);
+				krb5_free_context(kcontext);
+				return 0;
+			}
 		}
 	}
-	if ((code = krb5_unparse_name(kcontext, me, &client_name))) {
-		com_err(ProgramName, code, "when unparsing name");
-		return False;
-	}
-	(void) memset((char *) &my_creds, 0, sizeof (my_creds));
-
-
-	if ((code = krb5_build_principal_ext(kcontext, &server,
-				      krb5_princ_realm(kcontext, me)->length,
-					krb5_princ_realm(kcontext, me)->data,
-					     tgtname.length, tgtname.data,
-				      krb5_princ_realm(kcontext, me)->length,
-					krb5_princ_realm(kcontext, me)->data,
-					     0))) {
-		com_err(ProgramName, code, "while building server name");
-		return False;
-	}
-
-
-	my_creds.client = me;
-	my_creds.server = server;
-
-	my_creds.times.starttime = 0;
-
-	my_creds.times.renew_till = 0;
 
 	/*
-        ** find our default ticket, and copy out renewable, forwardable info.
-        */
-	{
-	krb5_cc_cursor cur;
-	krb5_creds creds;
-
-	if ((code = krb5_cc_start_seq_get(kcontext, ccache, &cur))) {
-	    com_err(ProgramName, code, "while starting to retrieve tickets");
-	} else {
-
-	    while (!(code = krb5_cc_next_cred(kcontext, ccache, &cur, &creds))) {
-		if (creds.server->length == 2 &&
-		        strcmp(creds.server->realm.data, me->realm.data) == 0 &&
-		        strcmp((char *)creds.server->data[0].data, "krbtgt") == 0 &&
-		        strcmp((char *)creds.server->data[1].data, me->realm.data) == 0 &&
-		        creds.times.endtime > now) {
-		    /*
-                    ** this is the krbtgt token to steal options from...
-		    */
-	    
-		    if (creds.times.renew_till) {
-		       options |= KDC_OPT_RENEWABLE;
-		       my_creds.times.renew_till = creds.times.renew_till;
-		    }
-		    if (creds.ticket_flags & TKT_FLG_FORWARDABLE) {
-		       options |= KDC_OPT_FORWARDABLE;
-		    }
-		}
-		krb5_free_cred_contents(kcontext, &creds);
-	    }
-        }
+	 * Set all of our various init_creds options
+	 */
+	
+	option = 0;
+	krb5_appdefault_boolean(kcontext, "xlock",
+				krb5_princ_realm(context, me), "forwardable",
+				option, &option);
+	krb5_get_init_creds_opt_set_forwardable(&options, option);
 
+	krb5_appdefault_string(kcontext, "xlock", krb5_princ_realm(context, me),
+			       "default_lifetime", "", &lifetimestr);
+	
+	if (lifetimestr && *lifetimestr) {
+		code = krb5_string_to_deltat(lifetimestr, &lifetime);
+		if (code == 0)
+			krb5_get_init_creds_opt_set_tkt_life(&options,
+							     lifetime);
 	}
 
-	if (strlen(pass) == 0)
-		(void) strcpy(pass, "*");
-	/* if pass is NULL, krb5_get_in_tkt_with_password will prompt with
-	   krb5_default_pwd_prompt1 for password, you don't want that in
-	   this application, most likely the user won't have '*' for a
-	   password -- dah <rodmur@ecst.csuchico.edu> */
+	if (lifetimestr)
+		free(lifetimestr);
 
-	code = krb5_get_in_tkt_with_password(kcontext, options, addrs,
-					     NULL, preauth, pass, 0,
-					     &my_creds, 0);
+	code = krb5_get_init_creds_password(kcontext, &my_creds, me, pass,
+					    xlock_prompter, NULL, 0, NULL,
+					    &options);
 
 	(void) memset(pass, 0, sizeof (pass));
 
 	if (code) {
-		if (code == KRB5KRB_AP_ERR_BAD_INTEGRITY)
+		if (code == KRB5KRB_AP_ERR_BAD_INTEGRITY) {
+			krb5_free_cred_contents(kcontext, &my_creds);
+			if (ccache)
+				krb5_cc_close(kcontext, ccache);
+			krb5_free_context(kcontext);
 			return False;	/* bad password entered */
-		else {
+		} else {
 			com_err(ProgramName, code, "while getting initial credentials");
+			krb5_free_cred_contents(kcontext, &my_creds);
+			if (ccache)
+				krb5_cc_close(kcontext, ccache);
+			krb5_free_context(kcontext);
 			return False;
 		}
 	}
-	code = krb5_cc_initialize(kcontext, ccache, me);
-	if (code != 0) {
-		com_err(ProgramName, code, "when initializing cache");
-		return False;
-	}
-	code = krb5_cc_store_cred(kcontext, ccache, &my_creds);
-	if (code) {
+
+#ifdef KRB5_WRITE_NEW_CCACHE
+	/*
+	 * Since the verification has succeeded, a failure at this point
+	 * will still result in returning a True.
+	 */
+	
+	if (!ccache && (code = krb5_cc_default(kcontext, &ccache)))
+		com_err(ProgramName, code, "while retrieving default "
+			"credential cache");
+
+	if (!code && (code = krb5_cc_initialize(kcontext, ccache, me)))
+		com_err(ProgramName, code, "while initializing credential "
+			"cache");
+	
+	if (!code && (code = krb5_cc_store_cred(kcontext, ccache, &my_creds)))
 		com_err(ProgramName, code, "while storing credentials");
-		return False;
-	}
-	krb5_free_principal(kcontext, server);
+
+#endif /* KRB5_WRITE_NEW_CCACHE */
+
+	if (ccache)
+		krb5_cc_close(kcontext, ccache);
+
+	krb5_free_cred_contents(kcontext, &my_creds);
 
 	krb5_free_context(kcontext);
 
@@ -1870,6 +1985,9 @@ set_multiple(int uid)
 			if (pw->pw_uid != uid)
 				continue;
 #ifdef HAVE_SHADOW
+/*- 
+ * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
+ * if using NIS compatibility mode, and is internal to libc.so */
 			if ((spw = getspnam(pw->pw_name)) != NULL) {
 				char       *tmp;	/* swap */
 
@@ -2031,6 +2149,9 @@ initPasswd(void)
 #endif
 		if (allowroot)
 			getCryptedRootPasswd();
+#ifdef USE_XLOCK_GROUP
+		getCryptedXlockGroupPasswds();
+#endif
 #endif /* !BSD_AUTH */
 	}
 #endif /* !ultrix && !DCE_PASSWD && !PAM */
@@ -2038,3 +2159,164 @@ initPasswd(void)
 	initDCE();
 #endif
 }
+
+#ifdef USE_XLOCK_GROUP
+/*
+**  FUNCTION  checkXlockgrp
+**  AUTHOR    Rich Anderson
+**  ABSTRACT  This function queries the NIS/NIS+/local group databases to check
+**            for the xlock group.  It then looks through the users within the
+**            group to check the inputted password belongs to member of the
+**            group.  It returns unlock.  valid = 1
+*/
+
+#ifndef GROUP
+#define GROUP "xlock"
+#endif
+
+#define PASSLENGTH1 (PASSLENGTH+1)
+static void
+getCryptedXlockGroupPasswds()
+{
+	int i = 0, num = 0;
+	struct passwd *pw;
+	struct group *grp;
+#ifdef HAVE_SHADOW
+	struct spwd *spw;
+#endif
+	if (!(grp = getgrnam(GROUP))) {
+		/* The group xlock does not exist. */
+		return;
+	}
+	if (!grp->gr_mem[0]) {
+		return;
+	}
+	while (grp->gr_mem[num]) {
+		num++;
+	}
+	if ((grouppass = (char *) malloc(num * PASSLENGTH1 * sizeof (char))) == NULL) {
+		return;
+	}
+	while (grp->gr_mem[i]) {
+		pw = (struct passwd *)getpwnam(grp->gr_mem[i]);
+
+		if (pw == NULL)
+			(void) strcpy(&grouppass[i * PASSLENGTH1], "*");
+		else {
+
+#ifdef HAVE_SHADOW
+/*- 
+ * PURIFY on Solaris 2 reports a 1024 byte memory leak on the following line
+ * if using NIS compatibility mode, and is internal to libc.so */
+			if ((spw = getspnam(pw->pw_name)) != NULL) {
+				char       *tmp;	/* swap */
+
+				tmp = pw->pw_passwd;
+				pw->pw_passwd = spw->sp_pwdp;
+				spw->sp_pwdp = tmp;
+			}
+			endspent();
+#endif
+			(void) strcpy(&grouppass[i * PASSLENGTH1], pw->pw_passwd);
+		}
+		i++;
+	}
+}
+
+int checkGroupXlockPasswds(char *buffer)
+{
+	struct group *grp;
+	int i = 0;
+	int unlock = 0;
+
+	if (!(grp = getgrnam(GROUP))) {
+		/* The group xlock does not exist. */
+		return False;
+	}
+	if (!grouppass)
+		return False;
+	if (!*buffer) {
+		/* The password buffer is blank. */
+		return False;
+	}
+	while (grp->gr_mem[i]) {
+#ifdef OSF1_ENH_SEC
+		unlock = (!strcmp((char *) dispcrypt(buffer, &grouppass[i * PASSLENGTH1], user_oldcrypt), &grouppass[i * PASSLENGTH1]));
+#else
+		unlock = (!strcmp((char *) crypt(buffer, &grouppass[i * PASSLENGTH1]), &grouppass[i * PASSLENGTH1]));
+#endif
+#if 0
+		/* For debugging only! */
+		(void) printf("gbuffer=%s, encrypt=%s, grouppass=%s,\n", buffer,
+		      (char *) crypt(buffer, grouppass[i]), &grouppass[i * PASSLENGTH1]);
+#endif
+		if (unlock)  {
+#if ( HAVE_SYSLOG_H && defined( USE_SYSLOG ))
+			syslog(SYSLOG_NOTICE, "%s: %s unlocked screen for %s", ProgramName, grp->gr_mem[i], user);
+#endif
+			return unlock;
+		}
+       		i++;
+	}
+	return False;
+}
+#endif
+
+#ifdef PASSWD_HELPER_PROGRAM
+#include <signal.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+
+static int
+run_helper(const char *user, const char *pass)
+{
+  sigset_t sigset;
+  int pfd[2], status;
+  pid_t pid;
+
+  if (pipe(pfd) < 0)
+    return 0;
+
+  sigemptyset(&sigset);
+  sigaddset(&sigset, SIGCHLD);
+  sigprocmask(SIG_BLOCK, &sigset, &sigset);
+
+  if ((pid = fork()) < 0) {
+    close(pfd[0]);
+    close(pfd[1]);
+    return 0;
+  }
+
+  if (pid == 0) {
+    close(pfd[1]);
+    if (pfd[0] != 0)
+      dup2(pfd[0], 0);
+
+    /* Helper is invoked as helper service-name [user] */
+    execlp(PASSWD_HELPER_PROGRAM, PASSWD_HELPER_PROGRAM,"xlock", user, NULL);
+    error("Failed to execute " PASSWD_HELPER_PROGRAM);
+    exit(1);
+  }
+
+  close(pfd[0]);
+
+  /* Write out password to helper process */
+  if (!pass)
+    pass = "";
+  write(pfd[1], pass, strlen(pass));
+  close(pfd[1]);
+
+  while (waitpid(pid, &status, 0) < 0) {
+    if (errno == EINTR)
+      continue;
+    sigprocmask(SIG_SETMASK, &sigset, NULL);
+    return 0;
+  }
+
+  sigprocmask(SIG_SETMASK, &sigset, NULL);
+
+  if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
+    return 0;
+  return 1;
+}
+#endif
