@@ -9,10 +9,14 @@ typedef struct LockStruct_s
 static LockStruct LockProcs[] =
 {
 {(char *) "ant", (char *) "Shows Langton's and Turk's generalized ants"},
+{(char *) "ant3d", (char *) "Shows 3D ants"},
 {(char *) "apollonian", (char *) "Shows Apollonian Circles"},
 #ifdef USE_GL
 {(char *) "atlantis", (char *) "Shows moving sharks/whales/dolphin"},
 #endif
+#ifdef USE_GL
+{(char *) "atunnels", (char *) "Shows an OpenGL advanced tunnel screensaver"},
+#endif
 {(char *) "ball", (char *) "Shows bouncing balls"},
 {(char *) "bat", (char *) "Shows bouncing flying bats"},
 {(char *) "blot", (char *) "Shows Rorschach's ink blot test"},
@@ -98,10 +102,12 @@ static LockStruct LockProcs[] =
 #endif
 {(char *) "mountain", (char *) "Shows Papo's mountain range"},
 {(char *) "munch", (char *) "Shows munching squares"},
+{(char *) "nose", (char *) "Shows SGI Diatoms"},
 {(char *) "nose", (char *) "Shows a man with a big nose runs around spewing out messages"},
 {(char *) "pacman", (char *) "Shows Pacman(tm)"},
 {(char *) "penrose", (char *) "Shows Penrose's quasiperiodic tilings"},
 {(char *) "petal", (char *) "Shows various GCD Flowers"},
+{(char *) "petri", (char *) "Shows a mold simulation in a petri dish"},
 #ifdef USE_GL
 {(char *) "pipes", (char *) "Shows a selfbuilding pipe system"},
 #endif
@@ -155,6 +161,9 @@ static LockStruct LockProcs[] =
 #if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_TTF ) && defined( HAVE_GLTT )
 {(char *) "text3d", (char *) "Shows 3D text"},
 #endif
+#if defined(USE_GL) && defined(HAVE_CXX) && defined( HAVE_FREETYPE ) && defined( HAVE_FTGL )
+{(char *) "text3d2", (char *) "Shows 3D text"},
+#endif
 {(char *) "thornbird", (char *) "Shows an animated bird in a thorn bush fractal map"},
 {(char *) "tik_tak", (char *) "Shows rotating polygons"},
 {(char *) "triangle", (char *) "Shows a triangle mountain range"},
