@@ -12,6 +12,13 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
  * See xlock.c for copying information.
  *
  * Revision History:
+ * Jun-03: Code added from Xlockup by Thad Phetteplace
+ *     tdphette@dexter.glaci.com 
+ *   Sun -- works (both timeouts seem connected to the same device)
+ *   Linux -- mouse timeout works, keyboard timeout does not work
+ *   Cygwin -- mouse timout does not work, keyboard timeout does not work
+ *   Alpha -- "does something"  ;)
+ *   Since keyboard timeout does not seem to help, it was removed.
  * Nov-96: Continual minor improvements by Charles Vidal and David Bagley.
  * Oct-96: written.
  */
@@ -23,15 +30,16 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
   3. Integer and floating point and string input.
  */
 
-/* 
+/*
  * 01/2001
- * I'm confused because the source is old and really not well programming 
+ * I'm confused because the source is old and really not well programming
  * I put boolean option and option in the menubar
- * I have had the option.h 
+ * I have had the option.h
  */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <time.h>
 #include <sys/signal.h>
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -44,6 +52,7 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
 
 #ifdef VMS
 #include "vms_x_fix.h"
@@ -55,7 +64,6 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 #include <X11/Xlocale.h>
 #endif
 
-/* #include <Xm/XmAll.h> Does not work on my version of Lesstif */
 #include <Xm/PanedW.h>
 #include <Xm/RowColumn.h>
 #include <Xm/ToggleB.h>
@@ -64,6 +72,7 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 #include <Xm/CascadeB.h>
 #include <Xm/Form.h>
 #include <Xm/Label.h>
+#include <Xm/Scale.h>
 #include <Xm/Separator.h>
 
 #if USE_XMU
@@ -86,25 +95,40 @@ static const char sccsid[] = "@(#)xmlock.c	4.08 98/02/18 xlockmore";
 
 #include "bitmaps/m-xlock.xbm"	/* icon bitmap */
 
+#define SKIPDELAY 60000 /* 1 minute */
+#define MAXMIN 120
+
 /* like an enum */
 enum ACTION {
   LAUNCH=0,
-  ROOT,   
-  WINDOW 
+  ROOT,
+  WINDOW
 	};
 
 /* extern variable */
-extern char *c_Options[OPTIONS];
 extern Widget Menuoption;
 extern Widget Toggles[TOGGLES];
+extern OptionStruct Opt[];
 
 extern char *r_Toggles[];
 
 static pid_t numberprocess = -1;	/* PID of xlock */
 
+/*************************
+ * To know the number of element
+ * in the Opt Array
+**************************/
+extern int getNumberofElementofOpt();
+
 /* Widget */
 Widget      toplevel;
 static Widget Labelxlock;
+static Widget      timeoutSlider;
+
+static time_t timenow;
+
+/* If this worked on all systems it would be 30 min or 1800 */
+static long timeout = 0;
 
 static Widget ScrolledListModes, PushButtons[PUSHBUTTONS];
 
@@ -124,32 +148,22 @@ static char *r_PushButtons[PUSHBUTTONS] =
 	(char *) "In Window",
 };
 
-char       *r_Options[OPTIONS] =
-{
-	(char *) "username",
-	(char *) "password",
-	(char *) "info",
-	(char *) "validate",
-	(char *) "invalid",
-	(char *) "geometry",
-	/*(char *) "font",
-	(char *) "program"*/
-};
-
 static int  numberinlist = 0;
 
+static void checkTime();
 /* CallBack */
 
-void exitcallback(Widget w, XtPointer client_data, XtPointer call_data) {
-			if (numberprocess != -1) {
-				(void) kill(numberprocess, SIGKILL);
-				/*(void) wait(&n);*/
-			}
-			exit(0);
+void exitcallback(Widget w, XtPointer client_data, XtPointer call_data)
+{
+	if (numberprocess != -1) {
+		(void) kill(numberprocess, SIGKILL);
+		/*(void) wait(&n);*/
 	}
+	exit(0);
+}
 
 static void
-f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
+callLocker(int choice)
 {
 	int         i;
 	char        command[500];
@@ -162,7 +176,7 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 
 	(void) sprintf(command, "%s ", XLOCK);
 
-/* booleans (+/-) options */
+	/* booleans (+/-) options */
 
 	for (i = 0; i < TOGGLES; i++) {
 		if (XmToggleButtonGetState(Toggles[i])) {
@@ -171,15 +185,21 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 			(void) strcat(command, " ");
 		}
 	}
-	for (i = 0; i < OPTIONS; i++)
-		if (c_Options[i] != NULL) {
+	/*
+		sizeof(Opt)/sizeof(OptionStruct) is not good to
+		know the number of element in Opt
+		so I have made a function getNumberofElementofOpt
+	*/
+	for (i = 0; i < getNumberofElementofOpt(); i++)
+		if (Opt[i].userdata != NULL) {
 			(void) strcat(command, "-");
-			(void) strcat(command, r_Options[i]);
+			(void) strcat(command, Opt[i].cmd);
 			(void) strcat(command, " ");
-			(void) strcat(command, c_Options[i]);
+			(void) strcat(command, Opt[i].userdata);
 			(void) strcat(command, " ");
 		}
-	switch ((int) client_data) {
+
+	switch (choice) {
 		case LAUNCH:
 			/* the default value then nothing to do */
 			break;
@@ -200,10 +220,59 @@ f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
 	(void) printf("%s\n", command);
 	(void) lib$spawn(&vms_image, 0, 0, &mask);
 #else
-	(void) strcat(command, " & ");
+	if (choice != LAUNCH)
+		(void) strcat(command, " & ");
 	(void) printf("%s\n", command);
 	(void) system(command);
 #endif
+	if (choice == LAUNCH)
+		(void) XtAppAddTimeOut(XtWidgetToApplicationContext(toplevel),
+			SKIPDELAY, (XtTimerCallbackProc) checkTime, toplevel);
+}
+
+/*----------------------------------------------------------*/
+/* Code taken from Xlockup by Thad Phetteplace              */
+/*    tdphette@dexter.glaci.com used by permission          */
+/* CHECKTIME: This routine is called periodicaly by the     */
+/*    openwin event handler.  It checks the last access     */
+/*    time/date stamp on the mouse and keyboard devices     */
+/*    and compares it to the timeout value.                 */
+/*----------------------------------------------------------*/
+
+static void checkTime()
+{
+  struct stat statmouse;
+#if 0
+  struct stat statkeybd;
+#endif
+
+  (void) time( &timenow );                   /* get current time */
+  /* next does not work on Cygwin */
+  (void) stat("/dev/mouse", &statmouse );   /* get mouse status */
+#if 0
+  /* does not seem to help on Solaris and hinders on Linux */
+  (void) stat("/dev/kbd", &statkeybd );     /* get keyboard status */
+#endif
+
+  /* check if last access time was larger than timeout */
+  if (timeout > 0 && (timenow-statmouse.st_atime >= timeout) /* &&
+      (timenow-statkeybd.st_atime >= timeout)*/) {
+     callLocker(LAUNCH);
+  }
+#ifdef DEBUG
+  printf("timeout %ld,  timenow - amouse %ld\n",
+    timeout, timenow - statmouse.st_atime);
+  printf("timeout %ld,  timenow - akeybd %ld\n",
+    timeout, timenow - statkeybd.st_atime);
+#endif
+  (void) XtAppAddTimeOut(XtWidgetToApplicationContext(toplevel),
+	SKIPDELAY, (XtTimerCallbackProc) checkTime, toplevel);
+}
+
+static void
+f_PushButtons(Widget w, XtPointer client_data, XtPointer call_data)
+{
+	callLocker((int) client_data);
 }
 
 static void
@@ -229,11 +298,18 @@ f_ScrolledListModes(Widget w, XtPointer client_data, XtPointer call_data)
 		(void) fprintf(stderr, "Fork error\n");
 	else if (numberprocess == 0) {
 		(void) execlp(XLOCK, XLOCK, "-parent", numberwidget,
-			      "-mode", str, "-geometry", WINDOW_GEOMETRY, "-delay", "100000",
-			      "-nolock", "-inwindow", "+install", 0);
+			"-mode", str, "-geometry", WINDOW_GEOMETRY,
+			"-delay", "100000",
+			"-nolock", "-inwindow", "+install", 0);
 	}
 }
 
+static void
+TimeoutSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+{
+	timeout = cbs->value * 60;
+}
+
 /* Setup Widget */
 
 /**********************/
@@ -283,8 +359,8 @@ Setup_Widget(Widget father)
 	Labelxlock = XmCreateLabel(father, (char *) "Window", args, ac);
 	XtManageChild(Labelxlock);
 
-/* buttons in the bottom */
- ac = 0;
+	/* buttons in the bottom */
+	 ac = 0;
 
 	XtSetArg(args[ac], XmNorientation, XmHORIZONTAL);
 	ac++;
@@ -303,35 +379,44 @@ Setup_Widget(Widget father)
 		XtSetArg(args[ac], XmNlabelString, label_str);
 		ac++;
 #endif
-		PushButtons[i] = XmCreatePushButton(PushButtonRow, r_PushButtons[i],
-						    args, ac);
+		PushButtons[i] = XmCreatePushButton(PushButtonRow,
+			 r_PushButtons[i], args, ac);
 #ifndef USE_MB
 		XmStringFree(label_str);
 #endif
-		XtAddCallback(PushButtons[i], XmNactivateCallback, f_PushButtons,
-			      (XtPointer) i);
+		XtAddCallback(PushButtons[i], XmNactivateCallback,
+			f_PushButtons, (XtPointer) i);
 		XtManageChild(PushButtons[i]);
 	}
-/* 
- *  Fro the exit button
+/*
+ *  For the exit button
  */
-		ac = 0;
+	ac = 0;
 #ifndef USE_MB
-		label_str = XmStringCreate((char *) "Exit",
-			XmSTRING_DEFAULT_CHARSET);
-		XtSetArg(args[ac], XmNlabelString, label_str);
-		ac++;
+	label_str = XmStringCreate((char *) "Exit",
+		XmSTRING_DEFAULT_CHARSET);
+	XtSetArg(args[ac], XmNlabelString, label_str);
+	ac++;
 #endif
-		exitwidgetB = XmCreatePushButton(PushButtonRow,
-			(char *) "Exit", args, ac);
+	exitwidgetB = XmCreatePushButton(PushButtonRow,
+		(char *) "Exit", args, ac);
 #ifndef USE_MB
-		XmStringFree(label_str);
+	XmStringFree(label_str);
 #endif
-		XtAddCallback(exitwidgetB, XmNactivateCallback, exitcallback,
-			(XtPointer) NULL);
-		XtManageChild(exitwidgetB);
+	XtAddCallback(exitwidgetB, XmNactivateCallback, exitcallback,
+		(XtPointer) NULL);
+	XtManageChild(exitwidgetB);
+	timeoutSlider = XtVaCreateManagedWidget("timeout",
+		xmScaleWidgetClass, PushButtonRow,
+		XtVaTypedArg, XmNtitleString, XmRString, "Timeout", 8,
+		XmNminimum, 0,
+		XmNmaximum, MAXMIN,
+		XmNvalue, timeout / 60,
+		XmNshowValue, True,
+		XmNorientation, XmHORIZONTAL, NULL);
+	XtAddCallback(timeoutSlider, XmNvalueChangedCallback,
+		(XtCallbackProc) TimeoutSlider, (XtPointer) NULL);
 
-	
 	XtManageChild(PushButtonRow);
 
 /* list and toggles in row like that (row(list)(TogglesRow(toggles...))) */
@@ -412,8 +497,8 @@ main(int argc, char **argv)
 		&argc, argv);
 	XtSetArg(args[0], XtNiconPixmap,
 		 XCreateBitmapFromData(XtDisplay(toplevel),
-				       RootWindowOfScreen(XtScreen(toplevel)),
-			    (char *) image_bits, image_width, image_height));
+			RootWindowOfScreen(XtScreen(toplevel)),
+			(char *) image_bits, image_width, image_height));
 	XtSetValues(toplevel, args, 1);
 	/* creation Widget */
 /* PURIFY 4.0.1 on Solaris 2 reports an unitialized memory read on the next
@@ -423,10 +508,12 @@ main(int argc, char **argv)
 	setup_Option(Menuoption);
 	XtManageChild(form);
 	XtRealizeWidget(toplevel);
+	(void) XtAppAddTimeOut(XtWidgetToApplicationContext(toplevel),
+		SKIPDELAY, (XtTimerCallbackProc) checkTime, toplevel);
 #if USE_XMU
+	/* With this handler you can use editres */
 	XtAddEventHandler(toplevel, (EventMask) 0, TRUE,
-			  (XtEventHandler) _XEditResCheckMessages, NULL);
-/* With this handler you can use editres */
+		(XtEventHandler) _XEditResCheckMessages, NULL);
 #endif
 	XtMainLoop();
 #ifdef VMS
